@@ -34,8 +34,10 @@
 
 #include <application.h>
 #include <expr.h>
+#include <expr-name.h>
 #include <value.h>
 #include <sheet.h>
+#include <sheet-view.h>
 #include <number-match.h>
 #include <cell.h>
 #include <parse-util.h>
@@ -47,13 +49,13 @@
 #include <position.h>
 #include <ranges.h>
 #include <io-context.h>
+#include <command-context.h>
 #include <workbook-view.h>
 #include <workbook.h>
 #include <error-info.h>
 #include <parse-util.h>
 
 #include <gsf/gsf-input-textline.h>
-#include <ctype.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -71,6 +73,8 @@ typedef struct {
 	size_t buffer_size;
 	size_t line_len;
 	int zoom;
+	GSList *sheet_order;
+	GSList *std_names, *real_names;
 } ApplixReadState;
 
 /* #define NO_DEBUG_APPLIX */
@@ -282,7 +286,7 @@ applix_read_colormap (ApplixReadState *state)
 
 		iter = pos = buffer + strlen (buffer) - 1;
 		for (count = 6; --count >= 0; pos = iter) {
-			while (--iter > buffer && isdigit ((unsigned char)*iter))
+			while (--iter > buffer && g_ascii_isdigit (*iter))
 				;
 
 			if (iter <= buffer || *iter != ' ')
@@ -414,8 +418,8 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 		/* Allocate the new style */
 		style = mstyle_new_default ();
 
-		mstyle_set_content_hidden (style, is_protected);
-		mstyle_set_content_locked (style, is_invisible);
+		mstyle_set_content_locked (style, is_protected);
+		mstyle_set_content_hidden (style, is_invisible);
 
 		if (sep[1] == '\'')
 			sep += 2;
@@ -430,7 +434,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 				continue;
 			}
 
-			if (isdigit ((unsigned char)*sep)) {
+			if (g_ascii_isdigit (*sep)) {
 				StyleHAlignFlags a;
 				switch (*sep) {
 				case '1' : a = HALIGN_LEFT; break;
@@ -487,7 +491,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 						break;
 					}
 
-					if (!isdigit ((unsigned char)sep[1]) ||
+					if (!g_ascii_isdigit (sep[1]) ||
 					    (0 == (id = strtol (sep+1, &end, 10))) ||
 					    sep+1 == end ||
 					    id < 1 || id > 16)
@@ -517,7 +521,7 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					/* What is 'Gf' ? */
 					if (sep[1] == 'f')
 						sep += 2;
-					else while (isdigit (*(unsigned char *)(++sep)))
+					else while (g_ascii_isdigit (*(++sep)))
 						;
 					break;
 
@@ -525,10 +529,10 @@ applix_parse_style (ApplixReadState *state, unsigned char **buffer)
 					/* comma 'CO' */
 					if (sep[1] == 'O') {
 						++sep;
-						format_prefix = "#,###";
+						format_prefix = "#,##0";
 					} else
 						/* FIXME : what currency to use for differnt locales */
-						format_prefix = "$ #,###";
+						format_prefix = "$ #,##0";
 
 					format_suffix = "";
 
@@ -800,8 +804,23 @@ applix_read_attributes (ApplixReadState *state)
 }
 
 static Sheet *
-applix_get_sheet (ApplixReadState *state, unsigned char **buffer,
-		  char const separator)
+applix_fetch_sheet (ApplixReadState *state, char const *name)
+{
+	Sheet *sheet = workbook_sheet_by_name (state->wb, name);
+
+	if (sheet == NULL) {
+		sheet = sheet_new (state->wb, name);
+		workbook_sheet_attach (state->wb, sheet, NULL);
+		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
+		sheet_flag_recompute_spans (sheet);
+	}
+
+	return sheet;
+}
+
+static Sheet *
+applix_parse_sheet (ApplixReadState *state, unsigned char **buffer,
+		    char const separator)
 {
 	Sheet *sheet;
 
@@ -814,14 +833,7 @@ applix_get_sheet (ApplixReadState *state, unsigned char **buffer,
 	}
 
 	*tmp = '\0';
-	sheet = workbook_sheet_by_name (state->wb, *buffer);
-	if (sheet == NULL) {
-		sheet = sheet_new (state->wb, *buffer);
-		workbook_sheet_attach (state->wb, sheet, NULL);
-		sheet_set_zoom_factor (sheet, (double )(state->zoom) / 100., FALSE, FALSE);
-		sheet_flag_recompute_spans (sheet);
-	}
-
+	sheet = applix_fetch_sheet (state, *buffer);
 	*buffer = tmp+1;
 	return sheet;
 }
@@ -833,7 +845,7 @@ applix_parse_cellref (ApplixReadState *state, unsigned char *buffer,
 {
 	int len;
 
-	*sheet = applix_get_sheet (state, &buffer, separator);
+	*sheet = applix_parse_sheet (state, &buffer, separator);
 
 	/* Get cell addr */
 	if (*sheet && cellpos_parse (buffer, pos, FALSE, &len))
@@ -871,21 +883,18 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 {
 	Sheet *sheet = NULL;
 	unsigned char *name = buffer + 19;
+	unsigned char *tmp;
 	gboolean ignore;
-	int len = strlen (name);
 
-	g_return_val_if_fail (len > 1 && name [len-1] == '~', -1);
+	tmp = strchr (name, ':');
+	if (tmp == NULL)
+		return 0;
+	*tmp =  '\0';
 
-	/* I wish there were some docs on this, names sometimes appear to be
-	 * SHEET_LETTER:name~ and sometimes
-	 * SHEET_LETTER:name:~ and sometimes
-	 */
-	if (name [len-2] == ':')
-		name [len-2] = '\0';
-	else
-		name [len-1] = '\0';
-
-	ignore = !strcmp (name, "Current"); /* ignore current */
+	ignore = tmp[1] != '~';
+	if (!ignore)
+		state->sheet_order = g_slist_prepend (state->sheet_order,
+			applix_fetch_sheet (state, name));
 
 	while (NULL != (buffer = applix_get_line (state))) {
 		if (!a_strncmp (buffer, "View End, Name: ~"))
@@ -944,7 +953,7 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 				sheet_row_set_size_pixels (sheet, row,
 							  applix_height_to_pixels (height),
 							  TRUE);
-			} while (ptr[0] == ' ' && isdigit ((unsigned char) ptr[1]));
+			} while (ptr[0] == ' ' && g_ascii_isdigit (ptr[1]));
 		} else if (!a_strncmp (buffer, "View Column Widths: ")) {
 			char const *ptr = buffer + 19;
 			char const *tmp;
@@ -965,7 +974,7 @@ applix_read_view (ApplixReadState *state, unsigned char *buffer)
 				sheet_col_set_size_pixels (sheet, col,
 							   applix_width_to_pixels (width),
 							   TRUE);
-			} while (ptr[0] == ' ' && isalpha ((unsigned char) ptr[1]));
+			} while (ptr[0] == ' ' && g_ascii_isalpha (ptr[1]));
 		}
 	}
 
@@ -1082,7 +1091,7 @@ applix_read_cells (ApplixReadState *state)
 				if (expr == NULL) {
 					(void) applix_parse_error (state, _("%s!%s : unable to parse '%s'\n     %s"),
 								   cell->base.sheet->name_quoted, cell_name (cell),
-								   expr_string, perr.message);
+								   expr_string, perr.err->message);
 					parse_error_free (&perr);
 					expr = gnm_expr_new_constant (value_new_string (expr_string));
 				} else if (is_array) {
@@ -1108,7 +1117,7 @@ applix_read_cells (ApplixReadState *state)
 						     (gpointer)expr);
 			} else {
 				const char *key = expr_string + strlen (expr_string);
-				while (key > expr_string && !isspace ((unsigned char)key[-1]))
+				while (key > expr_string && !g_ascii_isspace (key[-1]))
 					key--;
 #if 0
 				printf ("Shared '%s'\n", expr_string);
@@ -1152,7 +1161,7 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 {
 	unsigned char *tmp;
 	Range	r;
-	Sheet *sheet = applix_get_sheet (state, &ptr, ' ');
+	Sheet *sheet = applix_parse_sheet (state, &ptr, ' ');
 
 	if (ptr == NULL)
 		return -1;
@@ -1182,7 +1191,7 @@ applix_read_row_list (ApplixReadState *state, unsigned char *ptr)
 			return applix_parse_error (state, "Invalid row format attr index");
 
 	/* Just for kicks they added a trailing space */
-	} while (tmp[0] && isdigit ((unsigned char)tmp[1]));
+	} while (tmp[0] && g_ascii_isdigit (tmp[1]));
 
 	return 0;
 }
@@ -1191,10 +1200,29 @@ static gboolean
 applix_read_sheet_table (ApplixReadState *state)
 {
 	unsigned char *ptr;
-	while (NULL != (ptr = applix_get_line (state)))
+	unsigned char *std_name, *real_name;
+	while (NULL != (ptr = applix_get_line (state))) {
 	       if (!a_strncmp (ptr, "END SHEETS TABLE"))
 		       return FALSE;
-#warning TODO
+
+	       /* Sheet A: ~Foo~ */
+	       std_name = ptr + 6;
+	       ptr = strchr (std_name, ':');
+	       if (ptr == NULL)
+		       continue;
+	       *ptr = '\0';
+
+	       real_name = ptr + 3;
+	       ptr = strchr (real_name, '~');
+	       if (ptr == NULL)
+		       continue;
+	       *ptr = '\0';
+
+	       state->std_names  = g_slist_prepend (state->std_names,
+						    g_strdup (std_name));
+	       state->real_names = g_slist_prepend (state->real_names,
+						    g_strdup (real_name));
+	}
 	return TRUE;
 }
 
@@ -1209,9 +1237,12 @@ applix_read_header_footer (ApplixReadState *state)
 }
 
 static gboolean
-applix_read_relative_name (ApplixReadState *state, char *buffer)
+applix_read_absolute_name (ApplixReadState *state, char *buffer)
 {
 	char *end;
+	RangeRef ref;
+	ParsePos pp;
+	GnmExpr const *expr;
 
 	/* .ABCDe. Coordinate: A:B2..A:C4 */
 	/* Spec guarantees that there are no dots in the name */
@@ -1222,24 +1253,55 @@ applix_read_relative_name (ApplixReadState *state, char *buffer)
 	if (end == NULL)
 		return TRUE;
 	*end = '\0';
-#if 0
-	GnmExpr *expr = gnm_expr_parse_str (expr_string+1,
-		parse_pos_init_cell (&pos, cell),
-		GNM_EXPR_PARSE_USE_APPLIX_CONVENTIONS |
-		GNM_EXPR_PARSE_CREATE_PLACEHOLDER_FOR_UNKNOWN_FUNC,
-		&applix_rangeref_parse,
-		&perr);
-#endif
-	puts (buffer);
+	end = strchr (end + 1, ':');
+	if (end == NULL)
+		return TRUE;
+	applix_rangeref_parse (&ref, end+2,
+		parse_pos_init (&pp, state->wb, NULL, 0, 0));
+	ref.a.col_relative = ref.b.col_relative =
+		ref.a.row_relative = ref.b.row_relative = FALSE;
+
+	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
+	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+
 	return FALSE;
 }
 
 static gboolean
-applix_read_absolute_name (ApplixReadState *state, char *buffer)
+applix_read_relative_name (ApplixReadState *state, char *buffer)
 {
+	int dummy;
+	char *end;
+	RangeRef ref, flag;
+	ParsePos pp;
+	GnmExpr const *expr;
+
 	/* .abcdE. tCol:0 tRow:0 tSheet:0 bCol:1 bRow:2 bSheet: 0 tColAbs:0 tRowAbs:0 tSheetAbs:1 bColAbs:0 bRowAbs:0 bSheetAbs:1 */
 	/* Spec guarantees that there are no dots in the name */
-	puts (buffer);
+	buffer = strchr (buffer, '.');
+	if (buffer == NULL)
+		return TRUE;
+	end = strchr (++buffer, '.');
+	if (end == NULL)
+		return TRUE;
+	*end = '\0';
+	if (12 != sscanf (end + 2,
+			  " tCol:%d tRow:%d tSheet:%d bCol:%d bRow:%d bSheet: %d tColAbs:%d tRowAbs:%d tSheetAbs:%d bColAbs:%d bRowAbs:%d bSheetAbs:%d",
+			  &ref.a.col, &ref.a.row, &dummy, &ref.b.col, &ref.b.row, &dummy,
+			  &flag.a.col, &flag.a.row, &dummy, &flag.b.col, &flag.b.row, &dummy))
+		return TRUE;
+
+	ref.a.col_relative = (flag.a.col == 0);
+	ref.b.col_relative = (flag.b.col == 0);
+	ref.a.row_relative = (flag.a.row == 0);
+	ref.b.row_relative = (flag.b.row == 0);
+
+	ref.a.sheet = ref.b.sheet = NULL;
+	expr = gnm_expr_new_constant (value_new_cellrange_unsafe (&ref.a, &ref.b));
+	parse_pos_init (&pp, state->wb, NULL,
+		MAX (-ref.a.col, 0), MAX (-ref.a.row, 0));
+	expr_name_add (&pp, buffer, expr, NULL, TRUE);
+
 	return FALSE;
 }
 
@@ -1294,7 +1356,8 @@ applix_read_impl (ApplixReadState *state)
 			d (0, printf ("Applix load : Saved with revision %d.%d",
 				      major_rev, minor_rev););
 		} else if (!a_strncmp (buffer, "Current Doc Real Name:")) {
-			real_name = g_strdup (buffer + 22);
+			g_free (real_name);
+			real_name = NULL;  /* FIXME? g_strdup (buffer + 22); */
 
 		} else if (!strcmp (buffer, "COLORMAP")) {
 			if (applix_read_colormap (state))
@@ -1317,11 +1380,11 @@ applix_read_impl (ApplixReadState *state)
 				return applix_parse_error (state, "Invalid view");
 
 		} else if (!a_strncmp (buffer, "Default Label Style")) {
-			if (1 != sscanf (buffer, "Default Label Style %s", default_text_format))
+			if (1 != sscanf (buffer, "Default Label Style %127s", default_text_format))
 				return applix_parse_error (state, "invalid default label style");
 
 		} else if (!a_strncmp (buffer, "Default Number Style")) {
-			if (1 != sscanf (buffer, "Default Number Style %s", default_number_format))
+			if (1 != sscanf (buffer, "Default Number Style %127s", default_number_format))
 				return applix_parse_error (state, "invalid default number style");
 
 		} else if (!a_strncmp (buffer, "Document Column Width:")) {
@@ -1352,7 +1415,7 @@ applix_read_impl (ApplixReadState *state)
 				return applix_parse_error (state, "Absolute named range");
 		} else if (!a_strncmp (buffer, REL_NAMED_RANGE)) {
 			if (applix_read_relative_name (state, buffer + sizeof (REL_NAMED_RANGE)))
-				return applix_parse_error (state, "Absolute named range");
+				return applix_parse_error (state, "Relative named range");
 		} else if (!a_strncmp (buffer, "Row List")) {
 			if (applix_read_row_list (state, buffer + sizeof ("Row List")))
 				return applix_parse_error (state, "row list");
@@ -1395,6 +1458,7 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	int i;
 	int res;
 	ApplixReadState	state;
+	GSList *ptr, *renamed_sheets;
 
 	/* Init the state variable */
 	state.input	  = gsf_input_textline_new (src);
@@ -1409,6 +1473,9 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	state.buffer      = NULL;
 	state.buffer_size = 0;
 	state.line_len    = 80;
+	state.sheet_order = NULL;
+	state.std_names   = NULL;
+	state.real_names  = NULL;
 
 	/* Actually read the workbook */
 	res = applix_read_impl (&state);
@@ -1417,6 +1484,25 @@ applix_read (IOContext *io_context, WorkbookView *wb_view, GsfInput *src)
 	if (state.buffer)
 		g_free (state.buffer);
 
+	state.sheet_order = g_slist_reverse (state.sheet_order);
+	workbook_sheet_reorder (state.wb, state.sheet_order,  NULL);
+	g_slist_free (state.sheet_order);
+
+	renamed_sheets = NULL;
+	for (ptr = state.std_names; ptr != NULL ; ptr = ptr->next)
+		renamed_sheets = g_slist_prepend (renamed_sheets,
+			workbook_sheet_by_name (state.wb, ptr->data));
+	renamed_sheets = g_slist_reverse (renamed_sheets);
+	workbook_sheet_reorganize (state.wb, renamed_sheets, NULL,
+		state.real_names, state.std_names,
+		NULL, NULL, NULL, NULL, NULL, NULL,
+		COMMAND_CONTEXT (io_context));
+	g_slist_free (renamed_sheets);
+	g_slist_foreach (state.std_names, (GFunc)g_free, NULL);
+	g_slist_free (state.std_names);
+	g_slist_foreach (state.real_names, (GFunc)g_free, NULL);
+	g_slist_free (state.real_names);
+
 	/* Release the shared expressions and styles */
 	g_hash_table_foreach_remove (state.exprs, &cb_remove_expr, NULL);
 	g_hash_table_destroy (state.exprs);
