@@ -6,7 +6,6 @@
  *    Jon K Hellan (hellan@acm.org)
  *
  * (C) 1999, 2000 Jon K Hellan
- * excel_iconv* family of functions (C) 2001 by Vlad Harchev <hvv@hippo.ru>
  **/
 
 #include <gnumeric-config.h>
@@ -19,7 +18,6 @@
 
 #include <stdio.h>
 #include <string.h>
-#include <locale.h>
 
 #ifdef HAVE_LANGINFO_H
 #include <langinfo.h>
@@ -60,14 +58,16 @@
  * Makes a TwoWayTable. Returns the table.
  */
 TwoWayTable *
-two_way_table_new (GHashFunc    hash_func,
-		   GCompareFunc key_compare_func,
-		   gint         base)
+two_way_table_new (GHashFunc      hash_func,
+		   GCompareFunc   key_compare_func,
+		   gint           base,
+		   GDestroyNotify key_destroy_func)
 {
 	TwoWayTable *table = g_new (TwoWayTable, 1);
 
 	g_return_val_if_fail (base >= 0, NULL);
-	table->all_keys    = g_hash_table_new (g_direct_hash, g_direct_equal);
+	table->all_keys    = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+						    key_destroy_func, NULL);
 	table->unique_keys = g_hash_table_new (hash_func, key_compare_func);
 	table->idx_to_key  = g_ptr_array_new ();
 	table->base        = base;
@@ -178,7 +178,6 @@ two_way_table_key_to_idx (const TwoWayTable *table, gconstpointer key)
 gpointer
 two_way_table_idx_to_key (const TwoWayTable *table, gint idx)
 {
-
 	g_return_val_if_fail (idx - table->base >= 0, NULL);
 	g_return_val_if_fail (idx - table->base < (int)table->idx_to_key->len,
 			      NULL);
@@ -372,123 +371,3 @@ lookup_font_base_char_width (char const *name, double size_pts,
 	/* Use a rough heuristic for unknown fonts. */
 	return .5625 * size_pts;
 }
-
-typedef struct
-{
-	char const * const * keys;/*NULL-terminated list*/
-	int value;
-} s_hash_entry;
-
-/* here is a list of languages for which cp1251 is used on Windows*/
-static char const * const cyr_locales[] =
-{
-	"be", "be_BY", "bulgarian", "bg", "bg_BG", "mk", "mk_MK",
-	"russian", "ru", "ru_RU", "ru_UA", "sp", "sp_YU", "sr", "sr_YU",
-	"ukrainian", "uk", "uk_UA", NULL
-};
-
-
-/* here is a list of languages for which cp for cjk is used on Windows*/
-static char const * const jp_locales[] =
-{
-	"japan", "japanese", "ja", "ja_JP", NULL
-};
-
-static char const * const zhs_locales[] =
-{
-	"chinese-s", "zh", "zh_CN", NULL
-};
-
-static char const * const kr_locales[] =
-{
-	"korean", "ko", "ko_KR", NULL
-};
-
-static char const * const zht_locales[] =
-{
-	"chinese-t", "zh_HK", "zh_TW", NULL
- };
- 
-static s_hash_entry const win_codepages[]=
-{
-	{ cyr_locales , 1251 },
-	{ jp_locales  , 932 },
-	{ zhs_locales , 936 },
-	{ kr_locales  , 949 },
-	{ zht_locales , 950 },
-	{ NULL } /*terminator*/
-};
-
-guint
-excel_iconv_win_codepage (void)
-{
-	static guint codepage = 0;
-
-	if (codepage == 0) {
-		char *lang;
-
-		if ((lang = getenv("WINDOWS_LANGUAGE")) == NULL) {
-			char const *locale = setlocale (LC_CTYPE, NULL);
-			if (locale != NULL) {
-				char const *lang_sep = strchr (locale, '.');
-				if (lang_sep)
-					lang = g_strndup (locale, lang_sep - locale);
-				else
-					lang = g_strdup (locale); /* simplifies exit */
-			}
-		}
-
-		if (lang != NULL) {
-			s_hash_entry const * entry;
-
-			for (entry = win_codepages; entry->keys; ++entry) {
-				char const* const * key;
-				for (key = entry->keys; *key; ++key)
-					if (!g_strcasecmp (*key, lang)) {
-						g_free (lang);
-						return (codepage = entry->value);
-					}
-			}
-			g_free (lang);
-		}
-		codepage = 1252; /* default */
-	}
-	return codepage;
-}
-
-/*these two will figure out which charset names to use*/
-GIConv
-excel_iconv_open_for_import (guint codepage)
-{
-	GIConv iconv_handle;
-
-	if (codepage != 1200 && codepage != 1201) {
-		char* src_charset = g_strdup_printf ("CP%d", codepage);
-		iconv_handle = g_iconv_open ("UTF-8", src_charset);
-		g_free (src_charset);
-		if (iconv_handle != (GIConv)(-1))
-			return iconv_handle;
-		g_warning ("Unknown codepage %d", codepage);
-		return (GIConv)(-1);
-	}
-
-	/* this is 'compressed' unicode.  unicode characters 0000->00FF
-	 * which looks the same as 8859-1.  I have no idea what little
-	 * endian vs bigendian has to do with this.  There is only 1
-	 * byte, and it would certainly not be useful to keep the low
-	 * byte as 0.
-	 */
-	return g_iconv_open ("UTF-8", "ISO-8859-1");
-}
-
-GIConv
-excel_iconv_open_for_export (void)
-{
-	static char* dest_charset = NULL;
-	GIConv iconv_handle;
-
-	if (!dest_charset)
-		dest_charset = g_strdup_printf ("CP%d", excel_iconv_win_codepage());
-	iconv_handle = g_iconv_open (dest_charset, "UTF-8");
-	return iconv_handle;
-}
