@@ -37,6 +37,7 @@
 #include <parse-util.h>
 #include <cell.h>
 #include <str.h>
+#include <ranges.h>
 #include <sheet.h>
 #include <workbook.h>
 #include <format.h>
@@ -162,6 +163,7 @@ retrieve_format_info (Sheet *sheet, int col, int row)
 	return info;
 }
 
+/* TODO : turn this into a range based routine */
 static Value *
 gnumeric_cell (FunctionEvalInfo *ei, Value **argv)
 {
@@ -226,7 +228,7 @@ gnumeric_cell (FunctionEvalInfo *ei, Value **argv)
 	 * the worksheet name, but they can't make any other changes to CELL?!
 	 */
 	} else if (!g_ascii_strcasecmp (info_type, "filename")) {
-		char const *name = ei->pos->sheet->workbook->filename;
+		char const *name = workbook_get_filename (ei->pos->sheet->workbook);
 
 		if (name == NULL)
 			return value_new_string ("");
@@ -1206,6 +1208,7 @@ static Value *
 cb_countblank (Sheet *sheet, int col, int row,
 	       Cell *cell, void *user_data)
 {
+	cell_eval (cell);
 	if (!cell_is_blank (cell))
 		*((int *)user_data) -= 1;
 	return NULL;
@@ -1214,14 +1217,17 @@ cb_countblank (Sheet *sheet, int col, int row,
 static Value *
 gnumeric_countblank (FunctionEvalInfo *ei, Value **args)
 {
-	/* FIXME : This does not handle 3D references */
-	int count =
-		value_area_get_height (ei->pos, args[0]) *
-		value_area_get_width (ei->pos, args[0]);
+	Sheet *start_sheet, *end_sheet;
+	Range r;
+	int count;
 
+	rangeref_normalize (&args[0]->v_range.cell, ei->pos,
+		&start_sheet, &end_sheet, &r);
+	count = range_width (&r) * range_height	(&r);
+	if (start_sheet != end_sheet && end_sheet != NULL)
+		count *= 1 + abs (end_sheet->index_in_wb - start_sheet->index_in_wb);
 	workbook_foreach_cell_in_range (ei->pos, args[0],
-					CELL_ITER_IGNORE_BLANK,
-					&cb_countblank, &count);
+		CELL_ITER_IGNORE_BLANK, &cb_countblank, &count);
 
 	return value_new_int (count);
 }
@@ -1815,48 +1821,73 @@ gnumeric_getenv (FunctionEvalInfo *ei, Value **argv)
 
 /***************************************************************************/
 
-const ModulePluginFunctionInfo info_functions[] = {
+const GnmFuncDescriptor info_functions[] = {
 	{ "cell",	"sr", N_("info_type, cell"), &help_cell,
-	  gnumeric_cell, NULL, NULL, NULL },
-        { "countblank",	"r",  N_("range"), &help_countblank,
-	  gnumeric_countblank, NULL, NULL, NULL },
-	{ "error",	"s",  N_("text"), &help_error,
-	  gnumeric_error, NULL, NULL, NULL },
-	{ "error.type",	"S", N_("value"), &help_error_type,
-	  gnumeric_error_type, NULL, NULL, NULL },
-	{ "expression",	"r",   N_("cell"), &help_expression,
-	  gnumeric_expression, NULL, NULL, NULL },
+	  gnumeric_cell, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_SUBSET_WITH_EXTENSIONS, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "error.type",	"E", N_("value"), &help_error_type,
+	  gnumeric_error_type, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "info",	"s", N_("info_type"), &help_info,
-	  gnumeric_info, NULL, NULL, NULL },
+	  gnumeric_info, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "isblank",	NULL, N_("value"), &help_isblank,
-	  NULL, gnumeric_isblank, NULL, NULL },
-	{ "iserr",	"S",   N_("value"), &help_iserr,
-	  gnumeric_iserr, NULL, NULL, NULL },
-	{ "iserror",	"S",   N_("value"), &help_iserror,
-	  gnumeric_iserror, NULL, NULL, NULL },
-	{ "iseven",	"S", N_("value"), &help_iseven,
-	  gnumeric_iseven, NULL, NULL, NULL },
-	{ "islogical",	"S", N_("value"), &help_islogical,
-	  gnumeric_islogical, NULL, NULL, NULL },
-	{ "isna",	"S",   N_("value"), &help_isna,
-	  gnumeric_isna, NULL, NULL, NULL },
-	{ "isnontext",	"S", N_("value"), &help_isnontext,
-	  gnumeric_isnontext, NULL, NULL, NULL },
-	{ "isnumber",	"S", N_("value"), &help_isnumber,
-	  gnumeric_isnumber, NULL, NULL, NULL },
+	  NULL, gnumeric_isblank, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "iserr",	"E",   N_("value"), &help_iserr,
+	  gnumeric_iserr, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "iserror",	"E",   N_("value"), &help_iserror,
+	  gnumeric_iserror, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "iseven",	"f", N_("value"), &help_iseven,
+	  gnumeric_iseven, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "islogical",	"E", N_("value"), &help_islogical,
+	  gnumeric_islogical, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "isna",	"E",   N_("value"), &help_isna,
+	  gnumeric_isna, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "isnontext",	"E", N_("value"), &help_isnontext,
+	  gnumeric_isnontext, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "isnumber",	"E", N_("value"), &help_isnumber,
+	  gnumeric_isnumber, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "isodd",	"S", N_("value"), &help_isodd,
-	  gnumeric_isodd, NULL, NULL, NULL },
+	  gnumeric_isodd, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "isref",	NULL, N_("value"), &help_isref,
-	  NULL, gnumeric_isref, NULL, NULL },
-	{ "istext",	"S", N_("value"), &help_istext,
-	  gnumeric_istext, NULL, NULL, NULL },
+	  NULL, gnumeric_isref, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "istext",	"E", N_("value"), &help_istext,
+	  gnumeric_istext, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "n",		"S", N_("value"), &help_n,
-	  gnumeric_n, NULL, NULL, NULL },
+	  gnumeric_n, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "na",		"",  "", &help_na,
-	  gnumeric_na, NULL, NULL, NULL },
-	{ "type",	"S", N_("value"), &help_type,
-	  gnumeric_type, NULL, NULL, NULL },
+	  gnumeric_na, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "type",	"E", N_("value"), &help_type,
+	  gnumeric_type, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
+/* XL stores this in statistical ? */
+        { "countblank",	"r",  N_("range"), &help_countblank,
+	  gnumeric_countblank, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
+	{ "error",	"s",  N_("text"), &help_error,
+	  gnumeric_error, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
+	{ "expression",	"r",   N_("cell"), &help_expression,
+	  gnumeric_expression, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
 	{ "getenv",	"s", N_("string"), &help_getenv,
-	  gnumeric_getenv, NULL, NULL, NULL },
+	  gnumeric_getenv, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
+
         {NULL}
 };
