@@ -1020,7 +1020,7 @@ static const char *help_log = {
 	   "@DESCRIPTION="
 	   "LOG computes the logarithm of @x in the given base @base.  "
 	   "If no @base is given LOG returns the logarithm in base 10. "
-	   "@base must be > 0. and can not equal 1.\n\n"
+	   "@base must be > 0. and cannot equal 1.\n\n"
 	   "* This function is Excel compatible.\n"
 	   "\n"
 	   "@EXAMPLES=\n"
@@ -1192,33 +1192,36 @@ static const char *help_mod = {
 	   "@SEEALSO=INT,FLOOR,CEIL")
 };
 
+/*
+ * MOD(-1,-3) = -1
+ * MOD(2,-3) = -2
+ * MOD(10.6,2) = 0.6
+ * MOD(-10.6,2) = 1.4
+ * MOD(10.6,-2) = -0.6
+ * MOD(-10.6,-2) = -1.4
+ */
+
 static Value *
 gnumeric_mod (FunctionEvalInfo *ei, Value **argv)
 {
-	int a, b;
+	gnum_float a, b, babs, r;
 
-	a = value_get_as_int (argv[0]);
-	b = value_get_as_int (argv[1]);
+	a = value_get_as_float (argv[0]);
+	b = value_get_as_float (argv[1]);
 
 	if (b == 0)
 		return value_new_error (ei->pos, gnumeric_err_DIV0);
-	else if (b > 0) {
-		if (a >= 0)
-			return value_new_int (a % b);
-		else {
-			int c = (-a) % b;
-			return value_new_int (c ? b - c : 0);
-		}
-	} else {
-		b = -b;
-		a = -a;
-		if (a >= 0)
-			return value_new_int (-(a % b));
-		else {
-			int c = (-a) % b;
-			return value_new_int (c ? c - b : 0);
-		}
+
+	babs = gnumabs (b);
+	r = fmodgnum (gnumabs (a), babs);
+	if (r > 0) {
+		if ((a < 0) != (b < 0))
+			r = babs - r;
+		if (b < 0)
+			r = -r;
 	}
+
+	return value_new_float (r);
 }
 
 /***************************************************************************/
@@ -1575,7 +1578,7 @@ gnumeric_even (FunctionEvalInfo *ei, Value **argv)
 		number = -number;
 	}
 	ceiled = ceilgnum (number);
-	if (fmod (ceiled, 2) == 0)
+	if (fmodgnum (ceiled, 2) == 0)
 	        if (number > ceiled)
 		        return value_new_int ((int) (sign * (ceiled + 2)));
 		else
@@ -1613,7 +1616,7 @@ gnumeric_odd (FunctionEvalInfo *ei, Value **argv)
 		number = -number;
 	}
 	ceiled = ceilgnum (number);
-	if (fmod (ceiled, 2) == 1)
+	if (fmodgnum (ceiled, 2) == 1)
 	        if (number > ceiled)
 		        return value_new_int ((int) (sign * (ceiled + 2)));
 		else
@@ -1965,6 +1968,10 @@ gnumeric_mround (FunctionEvalInfo *ei, Value **argv)
 	number = value_get_as_float (argv[0]);
 	multiple = value_get_as_float (argv[1]);
 
+	/* Weird, but XL compatible.  */
+	if (multiple == 0)
+		return value_new_int (0);
+
 	if ((number > 0 && multiple < 0)
 	    || (number < 0 && multiple > 0))
 		return value_new_error (ei->pos, gnumeric_err_NUM);
@@ -1975,8 +1982,8 @@ gnumeric_mround (FunctionEvalInfo *ei, Value **argv)
 		multiple = -multiple;
 	}
 
-	mod = fmod (number, multiple);
-	div = number-mod;
+	mod = fmodgnum (number, multiple);
+	div = number - mod;
 
         return value_new_float (sign * (
 		div + ((mod + accuracy_limit >= multiple / 2) ? multiple : 0)));
@@ -2608,104 +2615,40 @@ static const char *help_seriessum = {
 	   "@SEEALSO=COUNT,SUM")
 };
 
-typedef struct {
-        gnum_float sum;
-        gnum_float x;
-        gnum_float n;
-        gnum_float m;
-} math_seriessum_t;
-
-static Value *
-callback_function_seriessum (const EvalPos *ep, Value *value,
-			     void *closure)
+static int
+range_seriessum (const gnum_float *xs, int n, gnum_float *res)
 {
-	math_seriessum_t *mm = closure;
-	gnum_float coefficient;
+	if (n >= 3) {
+		gnum_float x = xs[0];
+		gnum_float N = xs[1];
+		gnum_float m = xs[2];
+		gnum_float sum = 0;
 
-	if (!VALUE_IS_NUMBER (value))
-		return VALUE_TERMINATE;
+		gnum_float x_m = powgnum (x, m);
+		gnum_float xpow = powgnum (x, N);
+		int i;
 
-	coefficient = value_get_as_float (value);
+		for (i = 3; i < n; i++) {
+			sum += xs[i] * xpow;
+			xpow *= x_m;
+		}
 
-	mm->sum += coefficient * powgnum (mm->x, mm->n);
-	mm->n += mm->m;
-
-	return NULL;
+		*res = sum;
+		return 0;
+	} else
+		return 1;
 }
 
+
 static Value *
 gnumeric_seriessum (FunctionEvalInfo *ei, GnmExprList *nodes)
 {
-        math_seriessum_t p;
-        GnmExpr         *tree;
-	Value            *val;
-	gnum_float       x, n, m;
-
-	if (nodes == NULL)
-		return value_new_error (ei->pos, gnumeric_err_NUM);
-
-	/* Get x */
-	tree = (GnmExpr *) nodes->data;
-	if (tree == NULL)
-		return value_new_error (ei->pos, gnumeric_err_NUM);
-
-	val = gnm_expr_eval (tree, ei->pos, 0);
-	if (!val) return NULL;
-	if (!VALUE_IS_NUMBER (val)) {
-		value_release (val);
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-	}
-
-	x = value_get_as_float (val);
-	value_release (val);
-	nodes = nodes->next;
-
-	/* Get n */
-	tree = (GnmExpr *) nodes->data;
-	if (tree == NULL)
-		return value_new_error (ei->pos, gnumeric_err_NUM);
-
-	val = gnm_expr_eval (tree, ei->pos, 0);
-	if (!val) return NULL;
-	if (! VALUE_IS_NUMBER (val)) {
-		value_release (val);
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-	}
-
-	n = value_get_as_int (val);
-	value_release (val);
-
-	nodes = nodes->next;
-	if (nodes == NULL)
-		return value_new_error (ei->pos, gnumeric_err_NUM);
-
-	/* Get m */
-	tree = (GnmExpr *) nodes->data;
-	if (tree == NULL)
-		return value_new_error (ei->pos, gnumeric_err_NUM);
-
-	val = gnm_expr_eval (tree, ei->pos, 0);
-	if (!val) return NULL;
-	if (! VALUE_IS_NUMBER (val)) {
-		value_release (val);
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-	}
-
-	m = value_get_as_float (val);
-	value_release (val);
-	nodes = nodes->next;
-
-	p.n = n;
-	p.m = m;
-	p.x = x;
-	p.sum = 0;
-
-	if (function_iterate_argument_values (ei->pos,
-		callback_function_seriessum,
-		&p, nodes, TRUE, CELL_ITER_IGNORE_BLANK) != NULL)
-		return value_new_error (ei->pos, gnumeric_err_VALUE);
-
-	return value_new_float (p.sum);
+	return float_range_function (nodes, ei,
+				     range_seriessum,
+				     COLLECT_IGNORE_STRINGS |
+				     COLLECT_IGNORE_BOOLS |
+				     COLLECT_IGNORE_BLANKS,
+				     gnumeric_err_NUM);
 }
 
 /***************************************************************************/
@@ -2751,8 +2694,8 @@ validate_range_numeric_matrix (const EvalPos *ep, Value * matrix,
 	Value *res;
 	int cell_count = 0;
 
-	*cols = value_area_get_width (ep, matrix);
-	*rows = value_area_get_height (ep, matrix);
+	*cols = value_area_get_width (matrix, ep);
+	*rows = value_area_get_height (matrix, ep);
 
 	/* No checking needed for arrays */
 	if (matrix->type == VALUE_ARRAY)
@@ -2810,7 +2753,7 @@ gnumeric_minverse (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols; c++)
 	        for (r = 0; r < rows; r++) {
 		        Value const * a =
-			      value_area_get_x_y (ep, values, c, r);
+			      value_area_get_x_y (values, c, r, ep);
 		        *(matrix + r + c * cols) = value_get_as_float (a);
 		}
 
@@ -2887,14 +2830,14 @@ gnumeric_mmult (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols_a; c++)
 	        for (r = 0; r < rows_a; r++) {
 		        Value const * a =
-			     value_area_get_x_y (ep, values_a, c, r);
+			     value_area_get_x_y (values_a, c, r, ep);
 		        A[r + c * rows_a] = value_get_as_float (a);
 		}
 
 	for (c = 0; c < cols_b; c++)
 	        for (r = 0; r < rows_b; r++) {
 		        Value const * b =
-			     value_area_get_x_y (ep, values_b, c, r);
+			     value_area_get_x_y (values_b, c, r, ep);
 		        B[r + c * rows_b] = value_get_as_float (b);
 		}
 
@@ -2961,7 +2904,7 @@ gnumeric_mdeterm (FunctionEvalInfo *ei, Value **argv)
 	for (c = 0; c < cols; c++)
 	        for (r = 0; r < rows; r++) {
 		        Value const * a =
-			      value_area_get_x_y (ep, values, c, r);
+			      value_area_get_x_y (values, c, r, ep);
 		        *(matrix + r + c * cols) = value_get_as_float (a);
 		}
 
@@ -3018,8 +2961,8 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 					   GNM_EXPR_EVAL_PERMIT_NON_SCALAR |
 					   GNM_EXPR_EVAL_PERMIT_EMPTY);
 
-		thissizex = value_area_get_width (ei->pos, val);
-		thissizey = value_area_get_height (ei->pos, val);
+		thissizex = value_area_get_width (val, ei->pos);
+		thissizey = value_area_get_height (val, ei->pos);
 
 		if (i == 0) {
 			sizex = thissizex;
@@ -3031,8 +2974,7 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 		for (y = 0; y < thissizey; y++) {
 			for (x = 0; x < thissizex; x++) {
 				/* FIXME: efficiency worries?  */
-				const Value *v = value_area_fetch_x_y (ei->pos,
-								       val, x, y);
+				Value const *v = value_area_fetch_x_y (val, x, y, ei->pos);
 				if (v->type == VALUE_ERROR) {
 					/*
 					 * We carefully tranverse the argument
@@ -3086,127 +3028,193 @@ gnumeric_sumproduct (FunctionEvalInfo *ei, GnmExprList *args)
 
 /***************************************************************************/
 
-const ModulePluginFunctionInfo math_functions[] = {
+const GnmFuncDescriptor math_functions[] = {
 	{ "abs",     "f", N_("number"),    &help_abs,
-	  gnumeric_abs, NULL, NULL, NULL },
+	  gnumeric_abs, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "acos",    "f", N_("number"),    &help_acos,
-	  gnumeric_acos, NULL, NULL, NULL },
+	  gnumeric_acos, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "acosh",   "f", N_("number"),    &help_acosh,
-	  gnumeric_acosh, NULL, NULL, NULL },
+	  gnumeric_acosh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "asin",    "f", N_("number"),    &help_asin,
-	  gnumeric_asin, NULL, NULL, NULL },
+	  gnumeric_asin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "asinh",   "f", N_("number"),    &help_asinh,
-	  gnumeric_asinh, NULL, NULL, NULL },
+	  gnumeric_asinh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atan",    "f", N_("number"),    &help_atan,
-	  gnumeric_atan, NULL, NULL, NULL },
+	  gnumeric_atan, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atanh",   "f", N_("number"),    &help_atanh,
-	  gnumeric_atanh, NULL, NULL, NULL },
+	  gnumeric_atanh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "atan2",   "ff", N_("xnum,ynum"), &help_atan2,
-	  gnumeric_atan2, NULL, NULL, NULL },
+	  gnumeric_atan2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "cos",     "f", N_("number"),    &help_cos,
-	  gnumeric_cos, NULL, NULL, NULL },
+	  gnumeric_cos, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "cosh",    "f", N_("number"),    &help_cosh,
-	  gnumeric_cosh, NULL, NULL, NULL },
-	{ "countif", "r?", N_("range,criteria"), &help_countif,
-	  gnumeric_countif, NULL, NULL, NULL },
+	  gnumeric_cosh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
+
+/* MS Excel puts this in statistical */
+	{ "countif", "rS", N_("range,criteria"), &help_countif,
+	  gnumeric_countif, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
 	{ "ceil",    "f", N_("number"),    &help_ceil,
-	  gnumeric_ceil, NULL, NULL, NULL },
+	  gnumeric_ceil, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "ceiling", "ff", N_("number,significance"), &help_ceiling,
-	  gnumeric_ceiling, NULL, NULL, NULL },
+	  gnumeric_ceiling, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "degrees", "f", N_("number"),    &help_degrees,
-	  gnumeric_degrees, NULL, NULL, NULL },
+	  gnumeric_degrees, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "even",    "f", N_("number"),    &help_even,
-	  gnumeric_even, NULL, NULL, NULL },
+	  gnumeric_even, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "exp",     "f", N_("number"),    &help_exp,
-	  gnumeric_exp, NULL, NULL, NULL },
+	  gnumeric_exp, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "fact",    "f", N_("number"),    &help_fact,
-	  gnumeric_fact, NULL, NULL, NULL },
+	  gnumeric_fact, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
+/* MS Excel puts this in the engineering functions */
 	{ "factdouble", "f", N_("number"), &help_factdouble,
-	  gnumeric_factdouble, NULL, NULL, NULL },
+	  gnumeric_factdouble, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+
 	{ "fib", "f", N_("number"), &help_fib,
-	  gnumeric_fib, NULL, NULL, NULL },
+	  gnumeric_fib, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "combin",  "ff", N_("n,k"),      &help_combin,
-	  gnumeric_combin, NULL, NULL, NULL },
+	  gnumeric_combin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "floor",   "f|f", N_("number"),  &help_floor,
-	  gnumeric_floor, NULL, NULL, NULL },
+	  gnumeric_floor, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "gcd",     0, N_("number,number"), &help_gcd,
-	  NULL, gnumeric_gcd, NULL, NULL },
+	  NULL, gnumeric_gcd, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "int",     "f", N_("number"),    &help_int,
-	  gnumeric_int, NULL, NULL, NULL },
+	  gnumeric_int, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "lcm",     0, "",            &help_lcm,
-	  NULL, gnumeric_lcm, NULL, NULL },
+	  NULL, gnumeric_lcm, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "ln",      "f", N_("number"),    &help_ln,
-	  gnumeric_ln, NULL, NULL, NULL },
+	  gnumeric_ln, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log",     "f|f", N_("number,base"), &help_log,
-	  gnumeric_log, NULL, NULL, NULL },
+	  gnumeric_log, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log2",    "f", N_("number"),    &help_log2,
-	  gnumeric_log2, NULL, NULL, NULL },
+	  gnumeric_log2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "log10",   "f", N_("number"),    &help_log10,
-	  gnumeric_log10, NULL, NULL, NULL },
+	  gnumeric_log10, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mod",     "ff", N_("numerator,denominator"), &help_mod,
-	  gnumeric_mod, NULL, NULL, NULL },
+	  gnumeric_mod, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mround",  "ff", N_("number,multiple"), &help_mround,
-	  gnumeric_mround, NULL, NULL, NULL },
+	  gnumeric_mround, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "multinomial", 0, "", &help_multinomial,
-	  NULL, gnumeric_multinomial, NULL, NULL },
+	  NULL, gnumeric_multinomial, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "odd" ,    "f", N_("number"),    &help_odd,
-	  gnumeric_odd, NULL, NULL, NULL },
+	  gnumeric_odd, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "power",   "ff", N_("base,exponent"),      &help_power,
-	  gnumeric_power, NULL, NULL, NULL },
+	  gnumeric_power, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "g_product", 0, N_("number"),    &help_g_product,
-	  NULL, gnumeric_g_product, NULL, NULL },
+	  NULL, gnumeric_g_product, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "quotient" , "ff", N_("numerator,denominator"), &help_quotient,
-	  gnumeric_quotient, NULL, NULL, NULL },
+	  gnumeric_quotient, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "radians", "f", N_("number"),    &help_radians,
-	  gnumeric_radians, NULL, NULL, NULL },
+	  gnumeric_radians, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "roman",      "f|f", N_("number,type"), &help_roman,
-	  gnumeric_roman, NULL, NULL, NULL },
+	  gnumeric_roman, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "round",      "f|f", N_("number,digits"), &help_round,
-	  gnumeric_round, NULL, NULL, NULL },
+	  gnumeric_round, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "rounddown",  "f|f", N_("number,digits"), &help_rounddown,
-	  gnumeric_rounddown, NULL, NULL, NULL },
+	  gnumeric_rounddown, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "roundup",    "f|f", N_("number,digits"), &help_roundup,
-	  gnumeric_roundup, NULL, NULL, NULL },
+	  gnumeric_roundup, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "seriessum", 0, N_("x,n,m,coefficients"), &help_seriessum,
-	  NULL, gnumeric_seriessum, NULL, NULL },
+	  NULL, gnumeric_seriessum, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sign",    "f", N_("number"),    &help_sign,
-	  gnumeric_sign, NULL, NULL, NULL },
+	  gnumeric_sign, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sin",     "f", N_("number"),    &help_sin,
-	  gnumeric_sin, NULL, NULL, NULL },
+	  gnumeric_sin, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "sinh",    "f", N_("number"),    &help_sinh,
-	  gnumeric_sinh, NULL, NULL, NULL },
+	  gnumeric_sinh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "sqrt",    "f", N_("number"),    &help_sqrt,
-	  gnumeric_sqrt, NULL, NULL, NULL },
+	  gnumeric_sqrt, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sqrtpi",  "f", N_("number"),    &help_sqrtpi,
-	  gnumeric_sqrtpi, NULL, NULL, NULL },
+	  gnumeric_sqrtpi, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "suma",    0, N_("number,number,"), &help_suma,
-	  NULL, gnumeric_suma, NULL, NULL },
-	{ "sumif",   "r?|r", N_("range,criteria,actual_range"), &help_sumif,
-	  gnumeric_sumif, NULL, NULL, NULL },
+	  NULL, gnumeric_suma, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
+	{ "sumif",   "rS|r", N_("range,criteria,actual_range"), &help_sumif,
+	  gnumeric_sumif, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumproduct", 0, N_("range,range,"), &help_sumproduct,
-	  NULL, gnumeric_sumproduct, NULL, NULL },
+	  NULL, gnumeric_sumproduct, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumsq",   0, N_("number"),      &help_sumsq,
-	  NULL, gnumeric_sumsq, NULL, NULL },
+	  NULL, gnumeric_sumsq, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumx2my2", "AA", N_("array1,array2"), &help_sumx2my2,
-	  gnumeric_sumx2my2, NULL, NULL, NULL },
+	  gnumeric_sumx2my2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumx2py2", "AA", N_("array1,array2"), &help_sumx2py2,
-	  gnumeric_sumx2py2, NULL, NULL, NULL },
+	  gnumeric_sumx2py2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "sumxmy2",  "AA", N_("array1,array2"), &help_sumxmy2,
-	  gnumeric_sumxmy2, NULL, NULL, NULL },
+	  gnumeric_sumxmy2, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "tan",     "f", N_("number"),    &help_tan,
-	  gnumeric_tan, NULL, NULL, NULL },
+	  gnumeric_tan, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "tanh",    "f", N_("number"),    &help_tanh,
-	  gnumeric_tanh, NULL, NULL, NULL },
+	  gnumeric_tanh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "trunc",   "f|f", N_("number,digits"), &help_trunc,
-	  gnumeric_trunc, NULL, NULL, NULL },
+	  gnumeric_trunc, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "pi",      "", "",           &help_pi,
-	  gnumeric_pi, NULL, NULL, NULL },
+	  gnumeric_pi, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_EXHAUSTIVE },
 	{ "mmult",   "AA", N_("array1,array2"), &help_mmult,
-	  gnumeric_mmult, NULL, NULL, NULL },
+	  gnumeric_mmult, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "minverse","A", N_("array"),     &help_minverse,
-	  gnumeric_minverse, NULL, NULL, NULL },
+	  gnumeric_minverse, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 	{ "mdeterm", "A", N_("array,matrix_type,bandsize"), &help_mdeterm,
-	  gnumeric_mdeterm, NULL, NULL, NULL },
+	  gnumeric_mdeterm, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_RETURNS_NON_SCALAR, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
 #if 0
 	{ "logmdeterm", "A|si", N_("array,matrix_type,bandsize"),
 	  &help_logmdeterm, gnumeric_logmdeterm, NULL, NULL, NULL },
