@@ -24,6 +24,7 @@
  */
 
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 
 #include <func.h>
@@ -32,7 +33,6 @@
 #include <value.h>
 #include <auto-format.h>
 
-#include <libgnome/gnome-i18n.h>
 #include <expr.h>
 #include <sheet.h>
 #include <cell.h>
@@ -1091,70 +1091,103 @@ gnumeric_simtable (FunctionEvalInfo *ei, GnmExprList *nodes)
 
 /***************************************************************************/
 
-const ModulePluginFunctionInfo random_functions[] = {
+const GnmFuncDescriptor random_functions[] = {
 	{ "rand",    "", "",           &help_rand,
-	  gnumeric_rand, NULL, NULL, NULL },
+	  gnumeric_rand, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "randbernoulli", "f", N_("p"),   &help_randbernoulli,
-	  gnumeric_randbernoulli, NULL, NULL, NULL },
+	  gnumeric_randbernoulli, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randbeta", "ff", N_("a,b"),   &help_randbeta,
-	  gnumeric_randbeta, NULL, NULL, NULL },
+	  gnumeric_randbeta, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randbetween", "ff", N_("bottom,top"), &help_randbetween,
-	  gnumeric_randbetween, NULL, NULL, NULL },
+	  gnumeric_randbetween, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_COMPLETE, GNM_FUNC_TEST_STATUS_BASIC },
         { "randbinom", "ff", N_("p,trials"), &help_randbinom,
-	  gnumeric_randbinom, NULL, NULL, NULL },
+	  gnumeric_randbinom, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randcauchy", "f", N_("a"),   &help_randcauchy,
-	  gnumeric_randcauchy, NULL, NULL, NULL },
+	  gnumeric_randcauchy, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randchisq", "f", N_("nu"),   &help_randchisq,
-	  gnumeric_randchisq, NULL, NULL, NULL },
+	  gnumeric_randchisq, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
  	{ "randdiscrete", "r|r", N_("value_range,prob_range"),
-	  &help_randdiscrete, gnumeric_randdiscrete, NULL, NULL, NULL },
+	  &help_randdiscrete, gnumeric_randdiscrete, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
 	{ "randexp", "f", N_("b"),         &help_randexp,
-	  gnumeric_randexp, NULL, NULL, NULL },
+	  gnumeric_randexp, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randexppow", "ff", N_("a,b"),         &help_randexppow,
-	  gnumeric_randexppow, NULL, NULL, NULL },
+	  gnumeric_randexppow, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randfdist", "ff", N_("nu1,nu2"),      &help_randfdist,
-	  gnumeric_randfdist, NULL, NULL, NULL },
+	  gnumeric_randfdist, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randgamma", "ff", N_("a,b"),    &help_randgamma,
-	  gnumeric_randgamma, NULL, NULL, NULL },
+	  gnumeric_randgamma, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randgaussiantail", "ff", N_("a,sigma"),    &help_randgaussiantail,
-	  gnumeric_randgaussiantail, NULL, NULL, NULL },
+	  gnumeric_randgaussiantail, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randgeom", "f", N_("p"),    &help_randgeom,
-	  gnumeric_randgeom, NULL, NULL, NULL },
+	  gnumeric_randgeom, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randgumbel", "ff|f", N_("a,b[,type]"),    &help_randgumbel,
-	  gnumeric_randgumbel, NULL, NULL, NULL },
+	  gnumeric_randgumbel, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randhyperg", "fff", N_("n1,n2,t"),    &help_randhyperg,
-	  gnumeric_randhyperg, NULL, NULL, NULL },
+	  gnumeric_randhyperg, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlandau", "", "", &help_randlandau,
-	  gnumeric_randlandau, NULL, NULL, NULL },
+	  gnumeric_randlandau, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlaplace", "f", N_("a"), &help_randlaplace,
-	  gnumeric_randlaplace, NULL, NULL, NULL },
+	  gnumeric_randlaplace, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlevy", "ff|f", N_("c,alpha[,beta]"), &help_randlevy,
-	  gnumeric_randlevy, NULL, NULL, NULL },
+	  gnumeric_randlevy, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlog", "f", N_("p"), &help_randlog,
-	  gnumeric_randlog, NULL, NULL, NULL },
+	  gnumeric_randlog, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlogistic", "f", N_("a"), &help_randlogistic,
-	  gnumeric_randlogistic, NULL, NULL, NULL },
+	  gnumeric_randlogistic, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randlognorm", "ff", N_("zeta,sigma"), &help_randlognorm,
-	  gnumeric_randlognorm, NULL, NULL, NULL },
+	  gnumeric_randlognorm, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randnegbinom", "ff", N_("p,failures"), &help_randnegbinom,
-	  gnumeric_randnegbinom, NULL, NULL, NULL },
+	  gnumeric_randnegbinom, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randgaussian", "ff", N_("mean,stdev"), &help_randgaussian,
-	  gnumeric_randgaussian, NULL, NULL, NULL },
+	  gnumeric_randgaussian, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randpareto", "ff", N_("a,b"), &help_randpareto,
-	  gnumeric_randpareto, NULL, NULL, NULL },
+	  gnumeric_randpareto, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randpoisson", "f", N_("lambda"), &help_randpoisson,
-	  gnumeric_randpoisson, NULL, NULL, NULL },
+	  gnumeric_randpoisson, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randrayleigh", "f", N_("sigma"), &help_randrayleigh,
-	  gnumeric_randrayleigh, NULL, NULL, NULL },
+	  gnumeric_randrayleigh, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randrayleightail", "ff", N_("a,sigma"), &help_randrayleightail,
-	  gnumeric_randrayleightail, NULL, NULL, NULL },
+	  gnumeric_randrayleightail, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randtdist", "f", N_("nu"), &help_randtdist,
-	  gnumeric_randtdist, NULL, NULL, NULL },
+	  gnumeric_randtdist, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randuniform", "ff", N_("a,b"), &help_randuniform,
-	  gnumeric_randuniform, NULL, NULL, NULL },
+	  gnumeric_randuniform, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
         { "randweibull", "ff", N_("a,b"), &help_randweibull,
-	  gnumeric_randweibull, NULL, NULL, NULL },
-	{ "simtable", 0, N_("d1[,d2,...,dN]"), &help_simtable, NULL,
-	  gnumeric_simtable, NULL, NULL },
+	  gnumeric_randweibull, NULL, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
+	{ "simtable", 0, N_("d1[,d2,...,dN]"), &help_simtable,
+	  NULL,	gnumeric_simtable, NULL, NULL, NULL,
+	  GNM_FUNC_SIMPLE, GNM_FUNC_IMPL_STATUS_UNIQUE_TO_GNUMERIC, GNM_FUNC_TEST_STATUS_NO_TESTSUITE },
+
         {NULL}
 };
