@@ -43,11 +43,11 @@
 #include "error-info.h"
 #include "style-border.h"
 #include <rendered-value.h>
-
+#include "mstyle.h"
 #include <errno.h>
-#include <ctype.h>
 #include <string.h>
 
+
 /*
  * html_version_t:
  *
@@ -55,9 +55,10 @@
  *
  */
 typedef enum {
-	HTML40 = 0,
-	HTML32 = 1,
-	HTML40F   = 2
+	HTML40    = 0,
+	HTML32    = 1,
+	HTML40F   = 2,
+	XHTML     = 3
 } html_version_t;
 
 /*
@@ -153,37 +154,47 @@ html_write_cell_content (FILE *fp, Cell *cell, MStyle *mstyle, html_version_t ve
 	guint g = 0;
 	guint b = 0;
 	char *rendered_string;
+	gboolean hidden = mstyle_get_content_hidden (mstyle);
+	
+
+	if (version ==HTML32 && hidden)
+	  fputs ("<!-- 'HIDDEN DATA' -->",fp);
+	else
+	  {
 
 	if (mstyle != NULL) {
-		if (mstyle_get_font_italic (mstyle))
-			fputs ("<I>", fp);
+	        
+	       
+	        if (mstyle_get_font_italic (mstyle))
+			fputs ("<i>", fp);
 		if (mstyle_get_font_bold (mstyle))
-			fputs ("<B>", fp);
+			fputs ("<b>", fp);
 		if (font_is_monospaced (mstyle))
-			fputs ("<TT>", fp);
+			fputs ("<tt>", fp);
 	}
 
 	if (cell != NULL) {
 		if (mstyle != NULL && version != HTML40) {
 			html_get_text_color (cell, mstyle, &r, &g, &b);
 			if (r > 0 || g > 0 || b > 0)
-				fprintf (fp, "<FONT color=\"#%02X%02X%02X\">", r, g, b);
+				fprintf (fp, "<font color=\"#%02X%02X%02X\">", r, g, b);
 		}
+		
 		rendered_string = cell_get_rendered_text (cell);
 		html_print_encoded (fp, rendered_string);
 		g_free (rendered_string);
 	}
 
 	if (r > 0 || g > 0 || b > 0)
-		fputs ("</FONT>", fp);
+		fputs ("</font>", fp);
 	if (mstyle != NULL) {
 		if (font_is_monospaced (mstyle))
-			fputs ("</TT>", fp);
+			fputs ("</tt>", fp);
 		if (mstyle_get_font_bold (mstyle))
-			fputs ("</B>", fp);
+			fputs ("</b>", fp);
 		if (mstyle_get_font_italic (mstyle))
-			fputs ("</I>", fp);
-	}
+			fputs ("</i>", fp);
+	}}
 }
 
 
@@ -249,7 +260,7 @@ html_get_border_style (StyleBorder *border)
 		r = border->color->color.red >> 8;
 		g = border->color->color.green >> 8;
 		b = border->color->color.blue >> 8;
-		g_string_sprintfa (text, " #%02X%02X%02X", r, g, b);
+		g_string_append_printf (text, " #%02X%02X%02X", r, g, b);
 	}
 
 	result = text->str;
@@ -323,7 +334,7 @@ write_cell (FILE *fp, Sheet *sheet, gint row, gint col, html_version_t version)
 	Cell *cell;
 	MStyle *mstyle;
 	guint r, g, b;
-
+	
 	mstyle = sheet_style_get (sheet, col, row);
 	if (mstyle != NULL && version != HTML32 && version != HTML40 && 
 	    mstyle_get_pattern (mstyle) != 0 &&
@@ -370,7 +381,7 @@ write_cell (FILE *fp, Sheet *sheet, gint row, gint col, html_version_t version)
 		}
 
 	}
-	if (version == HTML40) {
+	if (version == HTML40 || version == HTML40F) {
 		if (mstyle != NULL) {
 			fprintf (fp, " style=\"");
 			if (mstyle_get_pattern (mstyle) != 0 &&
@@ -384,14 +395,17 @@ write_cell (FILE *fp, Sheet *sheet, gint row, gint col, html_version_t version)
 				html_get_text_color (cell, mstyle, &r, &g, &b);
 				if (r > 0 || g > 0 || b > 0)
 					fprintf (fp, " color:#%02X%02X%02X;", r, g, b);
+				if(mstyle_get_content_hidden(mstyle))
+				  fputs(" visibility:hidden;",fp);
 			}
+			
 			html_write_border_style_40 (fp, mstyle);
 			fprintf (fp, "\"");
 		}
 	}
 	fprintf (fp, ">");
 	html_write_cell_content (fp, cell, mstyle, version);
-	fputs ("</TD>\n", fp);
+	fputs ("</td>\n", fp);
 }
 
 
@@ -415,6 +429,8 @@ write_row (FILE *fp, Sheet *sheet, gint row, Range *range, html_version_t versio
 	ColRowInfo const * ri;
 
 	ri = sheet_row_get_info (sheet, row);
+	if (ri->needs_respan)
+		row_calc_spans ((ColRowInfo *) ri, sheet);
 
 	for (col = range->start.col; col <= range->end.col; col++) {
 		CellSpanInfo const *the_span;
@@ -426,7 +442,7 @@ write_row (FILE *fp, Sheet *sheet, gint row, Range *range, html_version_t versio
 		/* Is this a span */
 		the_span = row_span_get (ri, col);
 		if (the_span != NULL) {
-			fprintf (fp, "<TD COLSPAN=%i ", the_span->right - col + 1);
+			fprintf (fp, "<td colspan=\"%i\" ", the_span->right - col + 1);
 			write_cell (fp, sheet, row, the_span->cell->pos.col, version);
 			col = the_span->right;
 			continue;
@@ -438,14 +454,14 @@ write_row (FILE *fp, Sheet *sheet, gint row, Range *range, html_version_t versio
 			if (merge_range->start.col != col ||
 			    merge_range->start.row != row)
 				continue;
-			fprintf (fp, "<TD COLSPAN=%i ROWSPAN=%i ",
+			fprintf (fp, "<td colspan=\"%i\" rowspan=\"%i\" ",
 				 merge_range->end.col - merge_range->start.col + 1,
 				 merge_range->end.row - merge_range->start.row + 1);
 			write_cell (fp, sheet, row, col, version);
 			col = merge_range->end.col;
 			continue;
 		}
-		fputs ("<TD ", fp);
+		fputs ("<td ", fp);
 		write_cell (fp, sheet, row, col, version);
 	}
 }
@@ -466,22 +482,30 @@ write_sheet (FILE *fp, Sheet *sheet, html_version_t version)
 	Range total_range;
 	gint row;
 
-	if (version == HTML40)
-		fputs ("<P><TABLE cellspacing=\"0\" cellpadding=\"3\">\n", fp);
-	else
-		fputs ("<P><TABLE border=\"1\">\n", fp);
-
-	fputs ("<CAPTION>", fp);
+	switch (version) {
+	case HTML40:
+		fputs ("<p><table cellspacing=\"0\" cellpadding=\"3\">\n", fp);
+		break;
+	case XHTML:
+		fputs ("<p /><table cellspacing=\"0\" cellpadding=\"3\">\n", fp);
+		break;
+	default:
+		fputs ("<p><table border=\"1\">\n", fp);
+		break;
+	}
+	
+	fputs ("<caption>", fp);
 	html_print_encoded (fp, sheet->name_unquoted);
-	fputs ("</CAPTION>\n", fp);
+	fputs ("</caption>\n", fp);
 
 	total_range = sheet_get_extent (sheet, TRUE);
 	for (row = total_range.start.row; row <=  total_range.end.row; row++) {
-		fputs ("<TR>\n", fp);
-		write_row (fp, sheet, row, &total_range, version);
-		fputs ("</TR>\n", fp);
+		fputs ("<tr>\n", fp);
+		write_row (fp, sheet, row, &total_range, 
+			   (version == XHTML) ? HTML40 : version);
+		fputs ("</tr>\n", fp);
 	}
-	fputs ("</TABLE>\n", fp);
+	fputs ("</table>\n", fp);
 }
 
 /*
@@ -511,47 +535,71 @@ html_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	case HTML32:
 	fputs (
 "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 3.2//EN\">\n"
-"<HTML>\n"
-"<HEAD>\n\t<TITLE>Tables</TITLE>\n"
-"<META http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"
+"<html>\n"
+"<head>\n\t<title>Tables</title>\n"
+"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"
 "\t<!-- \"G_PLUGIN_FOR_HTML\" -->\n"
-"<STYLE><!--\n"
-"TT {\n"
+"<style><!--\n"
+"tt {\n"
 "\tfont-family: courier;\n"
 "}\n"
-"TD {\n"
+"td {\n"
 "\tfont-family: helvetica, sans-serif;\n"
 "}\n"
-"CAPTION {\n"
+"caption {\n"
 "\tfont-family: helvetica, sans-serif;\n"
 "\tfont-size: 14pt;\n"
 "\ttext-align: left;\n"
 "}\n"
-"--></STYLE>\n"
-"</HEAD>\n<BODY>\n", fp);
+"--></style>\n"
+"</head>\n<body>\n", fp);
 		break;
 	case HTML40:
 		fputs (
 "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\"\n"
 "\t\t\"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
-"<HTML>\n"
-"<HEAD>\n\t<TITLE>Tables</TITLE>\n"
-"<META http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"
+"<html>\n"
+"<head>\n\t<title>Tables</title>\n"
+"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\">\n"
 "\t<!-- \"G_PLUGIN_FOR_HTML\" -->\n"
-"<STYLE type=\"text/css\">\n"
-"TT {\n"
+"<style type=\"text/css\">\n"
+"tt {\n"
 "\tfont-family: courier;\n"
 "}\n"
-"TD {\n"
+"td {\n"
 "\tfont-family: helvetica, sans-serif;\n"
 "}\n"
-"CAPTION {\n"
+"caption {\n"
 "\tfont-family: helvetica, sans-serif;\n"
 "\tfont-size: 14pt;\n"
 "\ttext-align: left;\n"
 "}\n"
-"</STYLE>\n"
-"</HEAD>\n<BODY>\n", fp);
+"</style>\n"
+"</head>\n<body>\n", fp);
+		break;
+	case XHTML  :
+		fputs (
+"<?xml version=\"1.0\" encoding=\"iso-8859-1\"?>\n"
+"<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\"\n"
+"\t\t\"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">\n"
+"<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n"
+"<head>\n\t<title>Tables</title>\n"
+"<meta http-equiv=\"Content-Type\" content=\"text/html; charset=iso-8859-1\" />\n"
+"\t<!-- \"G_PLUGIN_FOR_HTML\" -->\n"
+"<style type=\"text/css\">\n"
+"tt {\n"
+"\tfont-family: courier;\n"
+"}\n"
+"td {\n"
+"\tfont-family: helvetica, sans-serif;\n"
+"}\n"
+"caption {\n"
+"\tfont-family: helvetica, sans-serif;\n"
+"\tfont-size: 14pt;\n"
+"\ttext-align: left;\n"
+"}\n"
+"</style>\n"
+"</head>\n<body>\n", fp);
 		break;
 	default:
 		break;
@@ -562,8 +610,8 @@ html_file_save (GnumFileSaver const *fs, IOContext *io_context,
 		write_sheet (fp, (Sheet *) ptr->data, version);
 	}
 	g_list_free (sheets);
-	if (version == HTML32 || version == HTML40)
-		fputs ("</BODY>\n</HTML>\n", fp);
+	if (version == HTML32 || version == HTML40 || version == XHTML)  
+		fputs ("</body>\n</html>\n", fp);
 	fclose (fp);
 }
 
@@ -591,7 +639,7 @@ html32_file_save (GnumFileSaver const *fs, IOContext *io_context,
 }
 
 /*
- * write every sheet of the workbook to an html 3.2 table
+ * write every sheet of the workbook to an html 4.0 fragment table
  *
  */
 void
@@ -600,3 +648,15 @@ html40frag_file_save (GnumFileSaver const *fs, IOContext *io_context,
 {
 	html_file_save (fs, io_context, wb_view, file_name, HTML40F);
 }
+/*
+ * write every sheet of the workbook to an xhtml 1.0 table
+ *
+ */
+
+void
+xhtml_file_save (GnumFileSaver const *fs, IOContext *io_context,
+                  WorkbookView *wb_view, const gchar *file_name)
+{
+	html_file_save (fs, io_context, wb_view, file_name, XHTML);
+}
+
