@@ -127,8 +127,8 @@ static latex_border_connectors_t const conn_styles[LATEX_MAX_BORDER]
 	  {{"8",""}, { "27",""}, { "43",""}},
 	  {{"",""}, { "",""}, { ":t:",""}}},
 	 {{{":b|",""}, { "29",""}, { ":","|"}},
-	  {{":b|",""}, { "30",""}, { "45",""}},
-	  {{"10",""}, { "31",""}, { ":",":"}}}}
+	  {{":b|",""}, { "30",""}, { ":|",""}},
+	  {{":b:",""}, { "31",""}, { ":",":"}}}}
 };
 
 
@@ -171,6 +171,41 @@ latex_fputs (char const *p, FILE *fp)
 	}
 }
 
+/**
+ * latex_math_fputs :
+ *
+ * @p : a pointer to a char, start of the string to be processed.
+ * @fp : a file pointer where the processed characters are written.
+ *
+ * This escapes any special LaTeX characters from the LaTeX engine.
+ * 
+ * We assume that htis will be set in Mathematics mode.
+ */
+static void
+latex_math_fputs (char const *p, FILE *fp)
+{
+	for (; *p; p++) {
+		switch (*p) {
+
+			/* These are the classic TeX symbols $ & % # (see Lamport, p.15) */
+			case '$': case '&': case '%': case '#':
+				fprintf (fp, "\\%c", *p);
+				break;
+			/* These are the other special characters ~ (see Lamport, p.15) */
+			case '~':
+				fprintf (fp, "\\%c{ }", *p);
+				break;
+			case '\\':
+				fputs ("\\backslash", fp);
+				break;
+
+			default:
+				fputc (*p, fp);
+				break;
+		}
+	}
+}
+
 
 /**
  * latex2e_write_file_header:
@@ -212,10 +247,6 @@ latex2e_write_file_header(FILE *fp)
 "%%  optionally (for landscape tables embedded in another document): %%\n"
 "%%    \\usepackage{lscape}                                           %%\n"
 "%%                                                                  %%\n"
-"%%  In addition the following commands need to be executed in the   %%\n"
-"%%  preamble:                                                       %%\n"
-"%%                                                                  %%\n"
-"%%                                                                  %%\n"
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "\n"
 "\n"
@@ -283,12 +314,16 @@ latex2e_write_file_header(FILE *fp)
 "%%                                                                  %%\n"
 "%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n"
 "\n"
+"\\providecommand{\\gnumericmathit}[1]{#1} \n" 
+"%%  Uncomment the next line if you would like your numbers to be in %%\n"
+"%%  italics if they are italizised in the gnumeric table.           %%\n"
+"%\\renewcommand{\\gnumericmathit}[1]{\\mathit{#1}}\n"
 "\\providecommand{\\gnumericPB}[1]%\n"
 "{\\let\\gnumericTemp=\\\\#1\\let\\\\=\\gnumericTemp\\hspace{0pt}}\n"
-" \\ifundefined{gnumericTableWidthDefined}"
+" \\ifundefined{gnumericTableWidthDefined}\n"
 "        \\newlength{\\gnumericTableWidth}\n"
 "        \\newlength{\\gnumericTableWidthComplete}\n"
-"        \\def\\gnumericTableWidthDefined{}\n"
+"        \\global\\def\\gnumericTableWidthDefined{}\n"
 " \\fi\n"
 "\n"
 "%%  The default table format retains the relative column widths of  %%\n"
@@ -302,7 +337,7 @@ latex2e_write_file_header(FILE *fp)
 "\\setlength{\\bigstrutjot}{\\jot}\n"
 "\\setlength{\\extrarowheight}{\\doublerulesep}\n"
 "\n"
-"%%  The \\setlongtables command keeps column widths the same across %%\n"
+"%%  The \\setlongtables command keeps column widths the same across  %%\n"
 "%%  pages. Simply comment out next line for varying column widths.  %%\n"
 "\\setlongtables\n"
 "\n"
@@ -505,6 +540,7 @@ latex2e_write_multicolumn_cell (FILE *fp, Cell const *cell, int num_merged_cols,
 
 	/* Print the cell according to its style. */
 	MStyle *mstyle = cell_get_mstyle (cell);
+	gboolean hidden = mstyle_get_content_hidden (mstyle);
 
 	g_return_if_fail (mstyle != NULL);
 
@@ -651,7 +687,11 @@ latex2e_write_multicolumn_cell (FILE *fp, Cell const *cell, int num_merged_cols,
 
 		/* Establish the font's style for the styles that can be addressed by LaTeX.
 		 * More complicated efforts (like changing fonts) are left to the user.
-		 */
+      		 */
+
+		if (hidden)
+		  fprintf (fp, "\\phantom{");
+
 		if (font_is_monospaced (mstyle))
 			fprintf (fp, "\\texttt{");
 		else if (font_is_sansserif (mstyle))
@@ -666,19 +706,26 @@ latex2e_write_multicolumn_cell (FILE *fp, Cell const *cell, int num_merged_cols,
 							   &cell_format_characteristic);
 		if (cell_format_family == FMT_NUMBER || cell_format_family == FMT_CURRENCY ||
 		    cell_format_family == FMT_PERCENT || cell_format_family == FMT_FRACTION ||
-		    cell_format_family == FMT_SCIENCE)
+		    cell_format_family == FMT_SCIENCE){
 			fprintf (fp, "$");
+		        if (mstyle_get_font_italic(mstyle))
+			    fprintf (fp, "\\gnumericmathit{");
 
-		/* Print the cell contents. */
-		rendered_string = cell_get_rendered_text (cell);
-		latex_fputs (rendered_string, fp);
-		g_free (rendered_string);
+			/* Print the cell contents. */
+			rendered_string = cell_get_rendered_text (cell);
+			latex_math_fputs (rendered_string, fp);
+			g_free (rendered_string);
 
-		if (cell_format_family == FMT_NUMBER || cell_format_family == FMT_CURRENCY ||
-		    cell_format_family == FMT_PERCENT || cell_format_family == FMT_FRACTION ||
-		    cell_format_family == FMT_SCIENCE)
+		        if (mstyle_get_font_italic(mstyle))
+			    fprintf (fp, "}");
 			fprintf (fp, "$");
-
+		} else {
+			/* Print the cell contents. */
+			rendered_string = cell_get_rendered_text (cell);
+			latex_fputs (rendered_string, fp);
+			g_free (rendered_string);
+		}
+		
 		/* Close the styles for the cell. */
 		if (mstyle_get_font_italic (mstyle))
 			fprintf (fp, "}");
@@ -688,6 +735,8 @@ latex2e_write_multicolumn_cell (FILE *fp, Cell const *cell, int num_merged_cols,
 			fprintf (fp, "}");
 		else if (font_is_sansserif (mstyle))
 			fprintf (fp, "}");
+		if (hidden)
+		  fprintf (fp, "}");
 		if (r != 0 || g != 0 || b != 0)
 			fprintf (fp, "}");
 	}
@@ -866,9 +915,12 @@ latex_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	fprintf (fp, "0pt}\n\\def\\gumericNumCols{%i}\n", num_cols);
 
 	fputs (""
-"\\setlength\\gnumericTableWidthComplete{\\gnumericTableWidth+\\tabcolsep*\\gumericNumCols*2+\\arrayrulewidth*\\gumericNumCols}\n"
+"\\setlength\\gnumericTableWidthComplete{\\gnumericTableWidth+%\n"
+"         \\tabcolsep*\\gumericNumCols*2+\\arrayrulewidth*\\gumericNumCols}\n"
 "\\ifthenelse{\\lengthtest{\\gnumericTableWidthComplete > \\textwidth}}%\n"
-"{\\def\\gnumericScale{\\ratio{\\textwidth-\\tabcolsep*\\gumericNumCols*2-\\arrayrulewidth*\\gumericNumCols}%\n"
+"         {\\def\\gnumericScale{\\ratio{\\textwidth-%\n"
+"                        \\tabcolsep*\\gumericNumCols*2-%\n"
+"                        \\arrayrulewidth*\\gumericNumCols}%\n"
 "{\\gnumericTableWidth}}}%\n"
 "{\\def\\gnumericScale{1}}\n"
 "\n"
@@ -904,7 +956,9 @@ latex_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	for (row = total_range.start.row; row <= total_range.end.row; row++) {
 		ColRowInfo const * ri;
 		ri = sheet_row_get_info (current_sheet, row);
-
+		if (ri->needs_respan)
+			row_calc_spans ((ColRowInfo *) ri, current_sheet);
+		
 		/* We need to check for horizontal borders at the top of this row */
 		length = num_cols;
 		clines = g_new0 (StyleBorderType, length);
@@ -1009,7 +1063,7 @@ latex_file_save (GnumFileSaver const *fs, IOContext *io_context,
 	}
 
 	/* We need to check for horizontal borders at the bottom  of  the last  row */
-	clines = g_new0 (StyleBorderType, total_range.end.row - total_range.start.row + 1);
+	clines = g_new0 (StyleBorderType, total_range.end.col - total_range.start.col + 1);
 	needs_hline = FALSE;
 	length = num_cols;
 	this_clines = clines;
