@@ -2,7 +2,7 @@
  * bonobo-io.c: Workbook IO using Bonobo storages.
  *
  * Author:
- *   Michael Meeks <michael@helixcode.com>
+ *   Michael Meeks <michael@ximian.com>
  *
  * Copyright 2000, Helix Code, Inc.
  */
@@ -85,8 +85,13 @@ gnumeric_bonobo_read_from_stream (BonoboPersistStream       *ps,
 		workbook_enable_recursive_dirty (wb, old);
 		if (gnumeric_io_error_occurred (ioc))
 			workbook_unref (wb);		
-	} else
-		gnumeric_io_error_read (ioc, _("Unsupported file format."));
+	} else {
+#warning "These two are doubtful."
+		wb_view = NULL;
+		wb = NULL;
+		gnumeric_error_read (COMMAND_CONTEXT (ioc),
+			_("Unsupported file format."));
+	}
 	if (gnumeric_io_error_occurred (ioc)) {
 		gnumeric_io_error_display (ioc);
 		/* This may be a bad exception to throw, but they're all bad */
@@ -339,7 +344,7 @@ gnumeric_bonobo_write_workbook (GnumFileSaver const *fs,
 
 	if (!storage) {
 		char *msg = g_strdup_printf ("Can't open '%s'", filename);
-		gnumeric_io_error_save (context, msg);
+		gnumeric_error_save (COMMAND_CONTEXT (context), msg);
 		g_free (msg);
 		return;
 	}
@@ -349,7 +354,7 @@ gnumeric_bonobo_write_workbook (GnumFileSaver const *fs,
 	 */
 	xml = xmlNewDoc ((xmlChar *)"1.0");
 	if (!xml) {
-		gnumeric_io_error_save (context, "");
+		gnumeric_error_save (COMMAND_CONTEXT (context), "");
 		bonobo_object_unref (BONOBO_OBJECT (storage));
 		return;
 	}
@@ -388,7 +393,7 @@ gnumeric_bonobo_write_workbook (GnumFileSaver const *fs,
 							    size, &ev);
 
 			if (BONOBO_EX (&ev)) {
-				gnumeric_io_error_save (context,
+				gnumeric_error_save (COMMAND_CONTEXT (context),
 					"Error storing workbook stream");
 			}
 		}
@@ -428,7 +433,7 @@ gnumeric_bonobo_read_workbook (GnumFileOpener const *fo,
 
 	if (!storage) {
 		char *msg = g_strdup_printf ("Can't open '%s'", filename);
-		gnumeric_io_error_save (context, msg);
+		gnumeric_error_save (COMMAND_CONTEXT (context), msg);
 		g_free (msg);
 		return;
 	}
@@ -441,7 +446,7 @@ gnumeric_bonobo_read_workbook (GnumFileOpener const *fo,
 	if (BONOBO_EX (&ev) || stream == CORBA_OBJECT_NIL) {
 		char *txt = g_strdup_printf (_("Error '%s' opening workbook stream"),
 					     bonobo_exception_get_text (&ev));
-		gnumeric_io_error_save (context, txt);
+		gnumeric_error_save (COMMAND_CONTEXT (context), txt);
 		g_free (txt);
 		goto storage_err;
 	}
@@ -451,14 +456,14 @@ gnumeric_bonobo_read_workbook (GnumFileOpener const *fo,
 	 */
 	doc = hack_xmlSAXParseFile (stream);
 	if (!doc) {
-		gnumeric_io_error_read (
-			context, "Failed to parse file");
+		gnumeric_error_read (COMMAND_CONTEXT (context),
+			_("Failed to parse file"));
 		goto storage_err;
 	}
 	if (!doc->xmlRootNode) {
 		xmlFreeDoc (doc);
-		gnumeric_io_error_read (
-			context, _("Invalid xml file. Tree is empty ?"));
+		gnumeric_error_read (COMMAND_CONTEXT (context),
+			_("Invalid xml file. Tree is empty ?"));
 		goto storage_err;
 	}
 
@@ -468,8 +473,8 @@ gnumeric_bonobo_read_workbook (GnumFileOpener const *fo,
 	gmr = xml_check_version (doc, &version);
 	if (!gmr) {
 		xmlFreeDoc (doc);
-		gnumeric_io_error_read (
-			context, _("Does not contain a Workbook file"));
+		gnumeric_error_read (COMMAND_CONTEXT (context),
+			_("Does not contain a Workbook file"));
 		goto storage_err;
 	}
 
@@ -528,3 +533,44 @@ gnumeric_bonobo_io_init (void)
 	register_file_saver (gnumeric_bonobo_saver);
 }
 #endif
+
+#if 0
+static int
+workbook_persist_file_load (BonoboPersistFile *ps, const CORBA_char *filename,
+			    CORBA_Environment *ev, void *closure)
+{
+	WorkbookView *wbv = closure;
+
+	return wb_view_open_file (filename, /* FIXME */ NULL, FALSE, NULL) ? 0 : -1;
+}
+
+static int
+workbook_persist_file_save (BonoboPersistFile *ps, const CORBA_char *filename,
+			    CORBA_Environment *ev, void *closure)
+{
+	WorkbookView *wbv = closure;
+	GnumFileSaver *fs;
+
+	fs = get_file_saver_by_id ("Gnumeric_XmlIO:gnum_xml");
+	return wb_view_save_as (wbv, fs, filename, NULL /* FIXME */) ? 0 : -1;
+}
+
+static void
+workbook_bonobo_setup (WorkbookPrivate *wbp)
+{
+	BonoboPersistFile *persist_file;
+
+	/* FIXME : This is totaly broken.
+	 * 1) it does not belong here at the workbook level
+	 * 2) which view use ?
+	 * 3) it should not be in this file.
+	 */
+	persist_file = bonobo_persist_file_new (
+		workbook_persist_file_load,
+		workbook_persist_file_save,
+		wbv);
+	bonobo_object_add_interface (
+		BONOBO_OBJECT (wbp),
+		BONOBO_OBJECT (persist_file));
+}
+#endif
