@@ -7,13 +7,13 @@
  *
  * (C) 2002 Jon K Hellan
  */
-#include <stdio.h>
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
+#include <stdio.h>
 #include "gnumeric.h"
 #include "command-context-stderr.h"
 #include "command-context-priv.h"
 #include <gsf/gsf-impl-utils.h>
-#include <libgnome/gnome-i18n.h>
 #include "error-info.h"
 #include "ranges.h"
 
@@ -55,66 +55,13 @@ command_context_stderr_get_status (CommandContextStderr *ccs)
 }
 
 static void
-ccs_system (CommandContext *ctxt, char const *msg)
+ccs_error_error (CommandContext *ctxt, GError *error)
 {
 	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
 
-	fprintf (stderr, "Error: %s\n", msg);
+	fprintf (stderr, "Error: %s\n", error->message);
 	ccs->status = -1;
 }
-
-static void
-ccs_plugin (CommandContext *ctxt, char const *msg)
-{
-	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
-
-	fprintf (stderr, "Error: %s\n", msg);
-	ccs->status = -1;
-}
-
-static void
-ccs_read (CommandContext *ctxt, char const *msg)
-{
-	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
-
-	fprintf (stderr, "Error: %s\n", msg);
-	ccs->status = -1;
-}
-
-static void
-ccs_save (CommandContext *ctxt, char const *msg)
-{
-	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
-
-	fprintf (stderr, "Error: %s\n", msg);
-	ccs->status = -1;
-}
-
-static void
-ccs_splits_array (CommandContext *ctxt, char const *cmd, Range const *array)
-{
-	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
-	char *msg;
-
-	if (array != NULL)
-		msg = g_strdup_printf (_("Would split array %s"),
-					   range_name (array));
-	else
-		msg = g_strdup (_("Would split an array"));
-	fprintf (stderr, "Error: %s\n", msg);
-	g_free (msg);
-	ccs->status = -1;
-}
-
-static void
-ccs_invalid (CommandContext *ctxt, char const *msg, char const *val)
-{
-	CommandContextStderr *ccs = COMMAND_CONTEXT_STDERR (ctxt);
-
-	fprintf (stderr, "Error: %s : %s\n", msg, val);
-	ccs->status = -1;
-}
-
 static void
 ccs_error_info (CommandContext *ctxt, ErrorInfo *error)
 {
@@ -146,15 +93,10 @@ ccs_class_init (GObjectClass *object_class)
 	CommandContextClass *cc_class = COMMAND_CONTEXT_CLASS (object_class);
 
 	g_return_if_fail (cc_class != NULL);
-	cc_class->get_password	     = ccs_get_password;
-	cc_class->set_sensitive	     = ccs_set_sensitive;
-	cc_class->error.system       = ccs_system;
-	cc_class->error.plugin       = ccs_plugin;
-	cc_class->error.read         = ccs_read;
-	cc_class->error.save         = ccs_save;
-	cc_class->error.splits_array = ccs_splits_array;
-	cc_class->error.invalid      = ccs_invalid;
-	cc_class->error.error_info   = ccs_error_info;
+	cc_class->get_password	   = ccs_get_password;
+	cc_class->set_sensitive	   = ccs_set_sensitive;
+	cc_class->error.error      = ccs_error_error;
+	cc_class->error.error_info = ccs_error_info;
 }
 
 GSF_CLASS (CommandContextStderr, command_context_stderr,
