@@ -2,8 +2,8 @@
 #ifndef GNUMERIC_COMMANDS_H
 #define GNUMERIC_COMMANDS_H
 
-#include "consolidate.h"
 #include "gnumeric.h"
+#include "consolidate.h"
 #include "sort.h"
 #include "tools/tools.h"
 
@@ -89,26 +89,28 @@ gboolean cmd_object_move	(WorkbookControl *wbc, SheetObject *so,
 				 SheetObjectAnchor const *old_anchor,
 				 gboolean resize);
 
-gboolean cmd_reorganize_sheets (WorkbookControl *wbc, GSList *old_order, GSList *new_order, 
-				GSList *changed_names, GSList *new_names, GSList *deleted_sheets,
-				GSList *color_changed, GSList *new_colors_back,
-				GSList *new_colors_fore, 
-				GSList *protection_changed, GSList *new_locks);
-gboolean cmd_rename_sheet      (WorkbookControl *wbc, Sheet *sheet, 
-				char const *old_name, char const *new_name);
+gboolean cmd_reorganize_sheets	(WorkbookControl *wbc, GSList *old_order, GSList *new_order, 
+				 GSList *changed_names, GSList *new_names, GSList *deleted_sheets,
+				 GSList *color_changed, GSList *new_colors_back,
+				 GSList *new_colors_fore, 
+				 GSList *protection_changed, GSList *new_locks);
+gboolean cmd_rename_sheet	(WorkbookControl *wbc, Sheet *sheet, 
+				 char const *old_name, char const *new_name);
 
-gboolean cmd_set_comment       (WorkbookControl *wbc, Sheet *sheet,
+gboolean cmd_set_comment	(WorkbookControl *wbc, Sheet *sheet,
 				 CellPos const *pos, const char *new_text);
 
-gboolean cmd_analysis_tool     (WorkbookControl *wbc, Sheet *sheet, 
-				data_analysis_output_t *dao, gpointer specs, 
-				analysis_tool_engine engine);
+gboolean cmd_analysis_tool	(WorkbookControl *wbc, Sheet *sheet, 
+				 data_analysis_output_t *dao, gpointer specs, 
+				 analysis_tool_engine engine);
 
-gboolean cmd_merge_data        (WorkbookControl *wbc, Sheet *sheet,
-				Value *merge_zone, GSList *merge_fields, GSList *merge_data);
+gboolean cmd_merge_data		(WorkbookControl *wbc, Sheet *sheet,
+				 Value *merge_zone, GSList *merge_fields, GSList *merge_data);
 
-gboolean cmd_change_summary    (WorkbookControlGUI *wbcg, GSList *sin_changes);
-gboolean cmd_print_set_up    (WorkbookControlGUI *wbcg, Sheet *sheet, PrintInformation const *pi);
+gboolean cmd_change_summary	(WorkbookControl *wbc, GSList *sin_changes);
+gboolean cmd_print_setup	(WorkbookControl *wbc, Sheet *sheet, PrintInformation const *pi);
+gboolean cmd_define_name	(WorkbookControl *wbc, char const *name, ParsePos const *pp,
+				 GnmExpr const *expr, GnmNamedExpr *nexpr);
 
 typedef enum  {
 	cmd_object_raise_up ,
