@@ -22,8 +22,8 @@
 #include <gnumeric-config.h>
 #include "gnumeric.h"
 #include "dependent.h"
-#include "workbook.h"
 
+#include "workbook-priv.h"
 #include "value.h"
 #include "cell.h"
 #include "sheet.h"
@@ -39,17 +39,32 @@
 
 #define BUCKET_SIZE	128
 
+static void dynamic_dep_eval 	   (Dependent *dep);
+static void dynamic_dep_debug_name (Dependent const *dep, FILE *out);
+
+static CellPos const dummy = { 0, 0 };
 static GPtrArray *dep_classes = NULL;
+static DependentClass dynamic_dep_class = {
+	dynamic_dep_eval,
+	NULL,
+	dynamic_dep_debug_name,
+};
+typedef struct {
+	Dependent  base;
+	Dependent *container;
+	GSList    *ranges;
+} DynamicDep;
 
 void
 dependent_types_init (void)
 {
 	g_return_if_fail (dep_classes == NULL);
 
-	/* Init with a pair of NULL classes so we can access directly */
+	/* Init with a trio of NULL classes so we can access directly */
 	dep_classes = g_ptr_array_new ();
-	g_ptr_array_add	(dep_classes, NULL);
-	g_ptr_array_add	(dep_classes, NULL);
+	g_ptr_array_add	(dep_classes, NULL); /* bogus filler */
+	g_ptr_array_add	(dep_classes, NULL); /* Cell */
+	g_ptr_array_add	(dep_classes, &dynamic_dep_class);
 }
 
 void
@@ -108,7 +123,7 @@ dependent_changed (Dependent *dep)
 	dependent_link (dep, &pos);
 
 	if (dep->sheet->workbook->recursive_dirty_enabled)
-		cb_dependent_queue_recalc (dep,  NULL);
+		dependent_queue_recalc (dep);
 	else
 		dependent_flag_recalc (dep);
 }
@@ -235,13 +250,14 @@ dependent_queue_recalc_list (GSList *list)
 
 	while (work) {
 		Dependent *dep = work->data;
+		int const t = dependent_type (dep);
 
 		/* Pop the top element.  */
 		list = work;
 		work = work->next;
 		g_slist_free_1 (list);
 
-		if (dependent_is_cell (dep)) {
+		if (t == DEPENDENT_CELL) {
 			GSList *deps = cell_list_deps (DEP_TO_CELL (dep));
 			GSList *waste = NULL;
 			GSList *next;
@@ -258,13 +274,16 @@ dependent_queue_recalc_list (GSList *list)
 				}
 			}
 			g_slist_free (waste);
+		} else if (t == DEPENDENT_DYNAMIC_DEP) {
+			DynamicDep const *dyn = (DynamicDep *)dep;
+			dependent_flag_recalc (dyn->container);
+			work = g_slist_prepend (work, dyn->container);
 		}
 	}
 }
 
-
 void
-cb_dependent_queue_recalc (Dependent *dep, gpointer ignore)
+dependent_queue_recalc (Dependent *dep)
 {
 	g_return_if_fail (dep != NULL);
 
@@ -290,19 +309,16 @@ typedef struct {
 
 #define MICRO_HASH_MIN_SIZE 11
 #define MICRO_HASH_MAX_SIZE 13845163
-#define MICRO_HASH_RESIZE(hash_table)				\
-G_STMT_START {							\
-	if ((hash_table->num_buckets > MICRO_HASH_MIN_SIZE &&		\
-	     hash_table->num_buckets >= 3 * hash_table->num_elements) || 	\
-	    (hash_table->num_buckets < MICRO_HASH_MAX_SIZE &&		\
-	     3 * hash_table->num_buckets <= hash_table->num_elements))	\
-		micro_hash_resize (hash_table);			\
+#define MICRO_HASH_RESIZE(hash_table)						\
+G_STMT_START {									\
+	if ((hash_table->num_buckets > MICRO_HASH_MIN_SIZE &&			\
+	     hash_table->num_buckets >= 3 * hash_table->num_elements) ||	\
+	    (hash_table->num_buckets < MICRO_HASH_MAX_SIZE &&			\
+	     3 * hash_table->num_buckets <= hash_table->num_elements))		\
+		micro_hash_resize (hash_table);					\
 } G_STMT_END
 
-/* The records are aligned so the bottom few bits don't hold much
- * entropy
- */
-#define MICRO_HASH_hash(key)	((guint)((int) (key) >> 9))
+#define MICRO_HASH_hash(key) ((guint)(key))
 
 static void
 micro_hash_resize (MicroHash *hash_table)
@@ -427,6 +443,8 @@ typedef MicroHash	DepCollection;
 	micro_hash_insert (&(dc), dep)
 #define dep_collection_remove(dc, dep)	\
 	micro_hash_remove (&(dc), dep)
+#define dep_collection_release(dc)      \
+	micro_hash_release (&(dc))
 #define dep_collection_is_empty(dc)				\
 	(dc.num_elements == 0)
 #define dep_collection_foreach_dep(dc, dep, code) do {		\
@@ -463,6 +481,8 @@ typedef GSList *	DepCollection;
 	if (!g_slist_find (dc, dep)) dc = g_slist_prepend (dc, dep)
 #define dep_collection_remove(dc, dep)	\
 	dc = g_slist_remove (dc, dep);
+#define dep_collection_release(dc)      \
+	g_slist_free (dc)
 #define dep_collection_is_empty(dc)	\
 	(dc == NULL)
 #define dep_collection_foreach_dep(dc, dep, code) do {		\
@@ -583,6 +603,7 @@ unlink_single_dep (Dependent *dep, CellPos const *pos, CellRef const *a)
 		dep_collection_remove (single->deps, dep);
 		if (dep_collection_is_empty (single->deps)) {
 			g_hash_table_remove (deps->single_hash, single);
+			dep_collection_release (single->deps);
 			gnm_mem_chunk_free (deps->single_pool, single);
 		}
 	}
@@ -657,7 +678,7 @@ link_cellrange_dep (Dependent *dep, CellPos const *pos,
 
 	if (a->sheet != NULL) {
 		if (a->sheet != dep->sheet)
-			flag = (a->sheet->workbook != a->sheet->workbook)
+			flag = (a->sheet->workbook != dep->sheet->workbook)
 				? DEPENDENT_GOES_INTERBOOK : DEPENDENT_GOES_INTERSHEET;
 
 		if (b->sheet != NULL && a->sheet != b->sheet) {
@@ -733,14 +754,14 @@ link_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 	case GNM_EXPR_OP_FUNCALL: {
 		GnmExprList *l;
 		DependentFlags flag = DEPENDENT_NO_FLAG;
-		if (tree->func.func->fn_type == FUNCTION_NAMEONLY)
-			func_def_load (tree->func.func);
-		if (tree->func.func->link) {
+		if (tree->func.func->fn_type == GNM_FUNC_TYPE_STUB)
+			gnm_func_load_stub (tree->func.func);
+		if (tree->func.func->linker) {
 			EvalPos		 ep;
 			FunctionEvalInfo fei;
 			fei.pos = eval_pos_init_dep (&ep, dep);
 			fei.func_call = (GnmExprFunction const *)tree;
-			flag = tree->func.func->link (&fei);
+			flag = tree->func.func->linker (&fei);
 		}
 		for (l = tree->func.arg_list; l; l = l->next)
 			flag |= link_expr_dep (dep, pos, l->data);
@@ -749,9 +770,9 @@ link_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 
 	case GNM_EXPR_OP_NAME:
 		expr_name_add_dep (tree->name.name, dep);
-		if (!tree->name.name->builtin && tree->name.name->active)
-			return link_expr_dep (dep, pos, tree->name.name->t.expr_tree);
-		return DEPENDENT_NO_FLAG;
+		if (tree->name.name->active)
+			return link_expr_dep (dep, pos, tree->name.name->expr_tree) | DEPENDENT_USES_NAME;
+		return DEPENDENT_USES_NAME;
 
 	case GNM_EXPR_OP_ARRAY:
 		if (tree->array.x != 0 || tree->array.y != 0) {
@@ -785,8 +806,10 @@ link_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 	case GNM_EXPR_OP_INTERSECT:
 		return DEPENDENT_NO_FLAG; /* handled at run time */
 
+#ifndef DEBUG_SWITCH_ENUM
 	default:
-		g_warning ("Unknown Operation type, dependencies lost");
+		g_assert_not_reached ();
+#endif
 	}
 	return 0;
 }
@@ -818,12 +841,12 @@ unlink_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 	 */
 	case GNM_EXPR_OP_FUNCALL: {
 		GnmExprList *l;
-		if (tree->func.func->unlink) {
+		if (tree->func.func->unlinker) {
 			EvalPos		 ep;
 			FunctionEvalInfo fei;
 			fei.pos = eval_pos_init_dep (&ep, dep);
 			fei.func_call = (GnmExprFunction const *)tree;
-			tree->func.func->unlink (&fei);
+			tree->func.func->unlinker (&fei);
 		}
 		for (l = tree->func.arg_list; l; l = l->next)
 			unlink_expr_dep (dep, pos, l->data);
@@ -832,8 +855,8 @@ unlink_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 
 	case GNM_EXPR_OP_NAME:
 		expr_name_remove_dep (tree->name.name, dep);
-		if (!tree->name.name->builtin && tree->name.name->active)
-			unlink_expr_dep (dep, pos, tree->name.name->t.expr_tree);
+		if (tree->name.name->active)
+			unlink_expr_dep (dep, pos, tree->name.name->expr_tree);
 		return;
 
 	case GNM_EXPR_OP_ARRAY:
@@ -869,28 +892,112 @@ unlink_expr_dep (Dependent *dep, CellPos const *pos, GnmExpr const *tree)
 	case GNM_EXPR_OP_INTERSECT:
 		return;
 
+#ifndef DEBUG_SWITCH_ENUM
 	default:
-		g_warning ("Unknown Operation type, dependencies lost");
+		g_assert_not_reached ();
 		break;
+#endif
 	}
 }
 
 static void
 workbook_link_3d_dep (Dependent *dep)
 {
-	if (dep->sheet->workbook->sheet_order_dependents == NULL)
-		dep->sheet->workbook->sheet_order_dependents =
+	Workbook *wb = dep->sheet->workbook;
+
+	if (wb->being_reordered)
+		return;
+	if (wb->sheet_order_dependents == NULL)
+		wb->sheet_order_dependents =
 			g_hash_table_new (g_direct_hash, g_direct_equal);
-	g_hash_table_insert (dep->sheet->workbook->sheet_order_dependents, dep, dep);
+	g_hash_table_insert (wb->sheet_order_dependents, dep, dep);
 }
 
 static void
 workbook_unlink_3d_dep (Dependent *dep)
 {
-	g_return_if_fail (dep->sheet->workbook->sheet_order_dependents != NULL);
+	Workbook *wb = dep->sheet->workbook;
+
+	/* during destruction */
+	if (wb->sheet_order_dependents == NULL)
+		return;
+
+	if (wb->being_reordered)
+		return;
 	g_hash_table_remove (dep->sheet->workbook->sheet_order_dependents, dep);
 }
 
+/*****************************************************************************/
+
+static void dynamic_dep_eval (Dependent *dep) { }
+
+static void
+dynamic_dep_debug_name (Dependent const *dep, FILE *out)
+{
+	fprintf (out, "DynamicDep%p", dep);
+}
+void
+dependent_add_dynamic_dep (Dependent *dep, ValueRange const *v)
+{
+	DependentFlags   flags;
+	DynamicDep	*dyn;
+	CellPos const	*pos;
+	
+	g_return_if_fail (dep != NULL);
+
+	pos = (dependent_is_cell (dep))
+		? &DEP_TO_CELL(dep)->pos : &dummy;
+
+	if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS)
+		dyn = g_hash_table_lookup (dep->sheet->deps->dynamic_deps, dep);
+	else {
+		dep->flags |= DEPENDENT_HAS_DYNAMIC_DEPS;
+		dyn = g_new (DynamicDep, 1);
+		dyn->base.flags		= DEPENDENT_DYNAMIC_DEP;
+		dyn->base.sheet		= dep->sheet;
+		dyn->base.expression	= NULL;
+		dyn->container		= dep;
+		dyn->ranges		= NULL;
+		g_hash_table_insert (dep->sheet->deps->dynamic_deps, dep, dyn);
+	}
+
+	flags = link_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
+	dyn->ranges = g_slist_prepend (dyn->ranges, value_duplicate ((Value *)v));
+	if (flags & DEPENDENT_HAS_3D)
+		workbook_link_3d_dep (dep);
+}
+
+static void
+dynamic_dep_free (DynamicDep *dyn)
+{
+	Dependent *dep = dyn->container;
+	CellPos const *pos = (dependent_is_cell (dep))
+		? &DEP_TO_CELL(dep)->pos : &dummy;
+	ValueRange *v;
+	GSList *ptr = dyn->ranges;
+
+	for (ptr = dyn->ranges ; ptr != NULL ; ptr = ptr->next) {
+		v = ptr->data;
+		unlink_cellrange_dep (&dyn->base, pos, &v->cell.a, &v->cell.b);
+		value_release ((Value *)v);
+	}
+	g_slist_free (dyn->ranges);
+	dyn->ranges = NULL;
+
+	if (dyn->base.flags & DEPENDENT_HAS_3D)
+		workbook_unlink_3d_dep (&dyn->base);
+}
+
+static void
+dependent_clear_dynamic_deps (Dependent *dep)
+{
+	DynamicDep *dyn = g_hash_table_lookup (dep->sheet->deps->dynamic_deps, dep);
+	if (dyn != NULL)
+		dynamic_dep_free (dyn);
+}
+
+/*****************************************************************************/
+
 /**
  * dependent_link:
  * @dep : the dependent that changed
@@ -936,8 +1043,6 @@ dependent_link (Dependent *dep, CellPos const *pos)
 void
 dependent_unlink (Dependent *dep, CellPos const *pos)
 {
-	static CellPos const dummy = { 0, 0 };
-
 	g_return_if_fail (dep != NULL);
 	g_return_if_fail (dependent_is_linked (dep));
 	g_return_if_fail (dep->expression != NULL);
@@ -958,6 +1063,8 @@ dependent_unlink (Dependent *dep, CellPos const *pos)
 
 	if (dep->flags & DEPENDENT_HAS_3D)
 		workbook_unlink_3d_dep (dep);
+	if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS)
+		dependent_clear_dynamic_deps (dep);
 	dep->flags &= ~DEPENDENT_LINK_FLAGS;
 }
 
@@ -980,6 +1087,15 @@ cell_eval_content (Cell *cell)
 	if (!cell_has_expr (cell))
 		return TRUE;
 
+	/* do this here rather than dependent_eval
+	 * because this routine is sometimes called
+	 * directly
+	 */
+	if (cell->base.flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
+		dependent_clear_dynamic_deps (CELL_TO_DEP (cell));
+		cell->base.flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
+	}
+
 #ifdef DEBUG_EVALUATION
 	{
 		ParsePos pp;
@@ -1032,7 +1148,8 @@ cell_eval_content (Cell *cell)
 	max_iteration = cell->base.sheet->workbook->iteration.max_number;
 
 iterate :
-	v = gnm_expr_eval (cell->base.expression, &pos, 0);
+	v = gnm_expr_eval (cell->base.expression, &pos,
+		GNM_EXPR_EVAL_SCALAR_NON_EMPTY);
 	if (v == NULL)
 		v = value_new_error (&pos, "Internal error");
 
@@ -1063,8 +1180,8 @@ iterate :
 			}
 			value_release (cell->value);
 			cell->value = v;
-			puts ("/* LOOP */");
 #ifdef DEBUG_EVALUATION
+			puts ("/* LOOP */");
 #endif
 			goto iterate;
 		}
@@ -1093,6 +1210,7 @@ iterate :
 	printf ("} (%d)\n", iterating == NULL);
 #endif
 	cell->base.flags &= ~DEPENDENT_BEING_CALCULATED;
+	cell->row_info->needs_respan = TRUE;
 	return iterating == NULL;
 }
 
@@ -1110,8 +1228,17 @@ dependent_eval (Dependent *dep)
 			DependentClass *klass = g_ptr_array_index (dep_classes, t);
 
 			g_return_val_if_fail (klass, FALSE);
+
+			if (dep->flags & DEPENDENT_HAS_DYNAMIC_DEPS) {
+				dependent_clear_dynamic_deps (dep);
+				dep->flags &= ~DEPENDENT_HAS_DYNAMIC_DEPS;
+			}
+
 			(*klass->eval) (dep);
 		} else {
+			/* This will clear the dynamic deps too, see comment there
+			 * to explain asymetry.
+			 */
 			gboolean finished = cell_eval_content (DEP_TO_CELL (dep));
 
 			/* This should always be the top of the stack */
@@ -1373,6 +1500,41 @@ dependents_unrelocate (GSList *info)
 	g_slist_free (info);
 }
 
+/**
+ * dependents_link :
+ * @deps : An slist of dependents that GETS FREED
+ * @rwinfo : optionally NULL
+ *
+ * link a list of dependents, BUT if the optional @rwinfo is specified and we
+ * are invalidating a sheet or workbook don't bother to link things in the same
+ * sheet or workbook.
+ */
+void
+dependents_link (GSList *deps, GnmExprRewriteInfo const *rwinfo)
+{
+	GSList *ptr = deps;
+
+	/* put them back */
+	for (; ptr != NULL ; ptr = ptr->next) {
+		Dependent *dep = ptr->data;
+		if (rwinfo != NULL) {
+			if (rwinfo->type == GNM_EXPR_REWRITE_WORKBOOK) {
+				if (rwinfo->u.workbook == dep->sheet->workbook)
+					continue;
+			} else if (rwinfo->type == GNM_EXPR_REWRITE_SHEET)
+				if (rwinfo->u.sheet == dep->sheet)
+					continue;
+		}
+		if (dep->sheet->deps != NULL && !dependent_is_linked (dep)) {
+			dependent_link (dep, dependent_is_cell (dep)
+				? &DEP_TO_CELL (dep)->pos : &dummy);
+			dependent_queue_recalc (dep);
+		}
+	}
+
+	g_slist_free (deps);
+}
+
 typedef struct {
 	Range const *target;
 	GSList *list;
@@ -1500,7 +1662,7 @@ dependents_relocate (GnmExprRelocateInfo const *info)
 			/* queue the things that depend on the changed dep
 			 * even if it is going to move.
 			 */
-			cb_dependent_queue_recalc (dep,  NULL);
+			dependent_queue_recalc (dep);
 
 			/* relink if it is not going to move, if it is moving
 			 * then the caller is responsible for relinking.
@@ -1608,15 +1770,37 @@ cb_name_invalidate (GnmNamedExpr *nexpr, gpointer value,
 {
 	GnmExpr const *new_expr = NULL;
 
-	if (!nexpr->builtin &&
-	    ((rwinfo->type == GNM_EXPR_REWRITE_SHEET &&
+	if (((rwinfo->type == GNM_EXPR_REWRITE_SHEET &&
 	     rwinfo->u.sheet != nexpr->pos.sheet) ||
 	    (rwinfo->type == GNM_EXPR_REWRITE_WORKBOOK &&
-	     rwinfo->u.workbook == nexpr->pos.wb))) {
-		new_expr = gnm_expr_rewrite (nexpr->t.expr_tree, rwinfo);
+	     rwinfo->u.workbook != nexpr->pos.wb))) {
+		new_expr = gnm_expr_rewrite (nexpr->expr_tree, rwinfo);
 		g_return_if_fail (new_expr != NULL);
 	}
-	expr_name_set_expr (nexpr, new_expr, rwinfo);
+
+	g_return_if_fail (nexpr->dependents == NULL ||
+			  g_hash_table_size (nexpr->dependents) == 0);
+	expr_name_set_expr (nexpr, new_expr);
+}
+
+static void
+cb_collect_deps_of_name (Dependent *dep, gpointer value,
+			 GSList **accum)
+{
+	/* grab unflagged linked depends */
+	if ((dep->flags & (DEPENDENT_FLAGGED|DEPENDENT_IS_LINKED)) == DEPENDENT_IS_LINKED) {
+		dep->flags |= DEPENDENT_FLAGGED;
+		*accum = g_slist_prepend (*accum, dep);
+	}
+}
+
+static void
+cb_collect_deps_of_names (GnmNamedExpr *nexpr, gpointer value,
+			  GSList **accum)
+{
+	if (nexpr->dependents)
+		g_hash_table_foreach (nexpr->dependents,
+			(GHFunc)cb_collect_deps_of_name, (gpointer)accum);
 }
 
 /*
@@ -1627,12 +1811,20 @@ cb_name_invalidate (GnmNamedExpr *nexpr, gpointer value,
 static void
 do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 {
-	static CellPos const dummy = { 0, 0 };
 	DependentFlags filter = DEPENDENT_LINK_FLAGS; /* unlink everything */
 	GnmDepContainer *deps;
 
 	g_return_if_fail (IS_SHEET (sheet));
 
+	/* The GnmDepContainer contains the names that reference this, not the
+	 * names it contains.  Remove them here. NOTE : they may continue to exist
+	 * inactively for a bit.  Be careful to remove them _before_ destroying
+	 * the deps.  This is a bit wasteful in that we unlink and relink a few
+	 * things that are going to be deleted.  However, it is necessary to
+	 * catch all the different life cycles
+	 */
+	gnm_named_expr_collection_free (&sheet->names);
+
 	deps = sheet->deps;
 	if (deps == NULL)
 		return;
@@ -1673,17 +1865,45 @@ do_deps_destroy (Sheet *sheet, GnmExprRewriteInfo const *rwinfo)
 		deps->single_pool = NULL;
 	}
 
-	if (deps->names) {
-		g_hash_table_foreach (deps->names,
+	if (deps->referencing_names) {
+		GSList *ptr, *accum = NULL;
+		Dependent *dep;
+		GHashTable *names = deps->referencing_names;
+
+		deps->referencing_names = NULL;
+
+		/* collect the deps of the names */
+		g_hash_table_foreach (names,
+			(GHFunc)cb_collect_deps_of_names,
+			(gpointer)&accum);
+
+		for (ptr = accum ; ptr != NULL ; ptr = ptr->next) {
+			dep = ptr->data;
+			dep->flags &= ~DEPENDENT_FLAGGED;
+			dependent_unlink (dep, NULL);
+		}
+
+		/* now that all of the dependents of these names are unlinked.
+		 * change the references in the names to avoid this sheet */
+		g_hash_table_foreach (names,
 			(GHFunc)cb_name_invalidate, (gpointer)rwinfo);
-		g_hash_table_destroy (deps->names);
-		deps->names = NULL;
+
+		/* the relink things en-mass in case one of the deps outside
+		 * this sheet used multiple names that referenced us */
+		dependents_link (accum, rwinfo);
+
+		g_hash_table_destroy (names);
+	}
+
+	if (deps->dynamic_deps) {
+		g_hash_table_destroy (deps->dynamic_deps);
+		deps->dynamic_deps = NULL;
 	}
 
 	/* TODO : when we support inter-app depends we'll need a new flag */
 	/* TODO : Add an 'application quit flag' to ignore interbook too */
 	if (sheet->deps == NULL) {
-		filter = DEPENDENT_GOES_INTERBOOK;
+		filter = DEPENDENT_GOES_INTERBOOK | DEPENDENT_USES_NAME;
 		if (rwinfo->type == GNM_EXPR_REWRITE_SHEET)
 			filter |= DEPENDENT_GOES_INTERSHEET;
 	}
@@ -1713,14 +1933,6 @@ sheet_deps_destroy (Sheet *sheet)
 	rwinfo.type = GNM_EXPR_REWRITE_SHEET;
 	rwinfo.u.sheet = sheet;
 
-	/* The GnmDepContainer contains the names that reference this, not the
-	 * names it contains.  Remove them here. NOTE : they may continue to exist
-	 * inactively for a bit.  Be careful to remove them _before_ destroying
-	 * the deps.  This is a bit wasteful in that we unlink and relink a few
-	 * things that are going to be deleted.  However, it is necessary to
-	 * catch all the different life cycles
-	 */
-	expr_name_list_destroy (&sheet->names);
 	do_deps_destroy (sheet, &rwinfo);
 }
 
@@ -1740,12 +1952,8 @@ workbook_deps_destroy (Workbook *wb)
 		wb->sheet_order_dependents = NULL;
 	}
 
-	/* See above for explantion */
-	expr_name_list_destroy (&wb->names);
-	WORKBOOK_FOREACH_SHEET (wb, sheet, {
-		expr_name_list_destroy (&sheet->names);
-		do_deps_destroy (sheet, &rwinfo);
-	});
+	gnm_named_expr_collection_free (&wb->names);
+	WORKBOOK_FOREACH_SHEET (wb, sheet, do_deps_destroy (sheet, &rwinfo););
 }
 
 void
@@ -1809,8 +2017,11 @@ gnm_dep_container_new (void)
 	deps->single_pool = gnm_mem_chunk_new ("single pool",
 					       sizeof (DependencySingle),
 					       16 * 1024 - 100);
-	deps->names       = g_hash_table_new (g_direct_hash,
-					      g_direct_equal);
+	deps->referencing_names = g_hash_table_new (g_direct_hash,
+						    g_direct_equal);
+
+	deps->dynamic_deps = g_hash_table_new_full (g_direct_hash, g_direct_equal,
+		NULL, (GDestroyNotify) dynamic_dep_free);
 
 	return deps;
 }
@@ -1919,3 +2130,4 @@ dependent_debug_name (Dependent const *dep, FILE *out)
 	} else
 		fprintf (out, "%s", cell_name (DEP_TO_CELL (dep)));
 }
+
