@@ -3,7 +3,7 @@
  *
  * Authors:
  *  JP Rosevear   <jpr@arcavia.com>
- *  Michael Meeks <michael@imaginator.com>
+ *  Michael Meeks <michael@ximian.com>
  *  Andreas J. Guelzow <aguelzow@taliesin.ca>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -46,7 +46,6 @@
 
 #include <glade/glade.h>
 #include <gsf/gsf-impl-utils.h>
-#include <ctype.h>
 #include <stdio.h>
 
 #define GLADE_FILE "cell-sort.glade"
