@@ -5,7 +5,7 @@
  *
  * Author:
  *	Jody Goldberg <jody@gnome.org>
- *	Michael Meeks <michael@imaginator.com>
+ *	Michael Meeks <michael@ximian.com>
  *	Chema Celorio <chema@celorio.com>
  *
  * This program is free software; you can redistribute it and/or modify
@@ -157,7 +157,7 @@ name_guru_in_list (NameGuruState *state, char const *name,
 		if (ignore_placeholders && expr_name_is_placeholder (nexpr))
 			continue;
 
-		/* no need for utf8 or collation magic, just equality */
+		/* no need for UTF-8 or collation magic, just equality */
 		if (strcmp (name, nexpr->name->str) == 0)
 			return nexpr;
 	}
@@ -537,6 +537,10 @@ name_guru_init (NameGuruState *state, WorkbookControlGUI *wbcg)
 
 	name_guru_populate_list (state);
 
+	gnumeric_init_help_button (
+		glade_xml_get_widget (state->gui, "helpbutton"),
+		"names.html");
+
 	/* a candidate for merging into attach guru */
 	gnumeric_keyed_dialog (state->wbcg, GTK_WINDOW (state->dialog),
 			       DEFINE_NAMES_KEY);
