@@ -43,7 +43,6 @@
 
 #include <glade/glade.h>
 #include <gal/widgets/widget-color-combo.h>
-#include <gal/util/e-util.h>
 
 typedef struct {
 	WorkbookControlGUI  *wbcg;
@@ -427,8 +426,8 @@ cb_add_clicked (GtkWidget *ignore, SheetManager *state)
 					    SHEET_NEW_NAME, &new_name,
 					    -1);
 			n++;
-			match = (0 == g_str_compare (name, new_name) ||
-				 0 == g_str_compare (name, old_name));
+			match = (new_name != NULL && !strcmp (name, new_name)) ||
+				(old_name != NULL && !strcmp (name, old_name));
 			if (match)
 				break;
 		}
@@ -552,8 +551,7 @@ cb_ok_clicked (GtkWidget *ignore, SheetManager *state)
 			new_order = g_slist_prepend (new_order, this_sheet);
 			
 			if (this_sheet == NULL || 
-			    (strlen(new_name) > 0 && 
-			     0 != g_str_compare (old_name, new_name))) {
+			    (strlen (new_name) > 0 && strcmp (old_name, new_name))) {
 				changed_names = g_slist_prepend (changed_names, this_sheet);
 				new_names = g_slist_prepend (new_names, 
 							     strlen(new_name) > 0 ? new_name : NULL);
@@ -606,10 +604,14 @@ cb_ok_clicked (GtkWidget *ignore, SheetManager *state)
 	changed_names = g_slist_reverse (changed_names);
 	old_order = g_slist_copy (state->old_order);
 	
-	if (deleted_sheets && 
+	if ((new_order == NULL) || (deleted_sheets && 
 	    !gnumeric_dialog_question_yes_no (state->wbcg,
 					      _("Deletion of sheets is not undoable. "
-						"Do you want to proceed?"), FALSE)) {
+						"Do you want to proceed?"), FALSE))) {
+		if (new_order == NULL)
+			gnumeric_notice (state->wbcg, GTK_MESSAGE_ERROR, 
+					 _("You may not delete all sheets in a workbook!"));
+		
 		/* clean-up */
 		g_slist_free (deleted_sheets);
 		deleted_sheets = NULL;
@@ -619,7 +621,8 @@ cb_ok_clicked (GtkWidget *ignore, SheetManager *state)
 		changed_names = NULL;
 		g_slist_free (old_order);
 		old_order = NULL;
-		e_free_string_slist (new_names);
+		g_slist_foreach (new_names, (GFunc)g_free, NULL);
+		g_slist_free (new_names);
 		new_names = NULL;
 
 		g_slist_free (color_changed);
@@ -686,7 +689,7 @@ cb_ok_clicked (GtkWidget *ignore, SheetManager *state)
 }
 
 static void
-cb_sheet_order_destroy (GtkWidget *ignored, SheetManager *state)
+cb_sheet_order_destroy (SheetManager *state)
 {
 	Workbook *wb = wb_control_workbook (WORKBOOK_CONTROL (state->wbcg));
 
@@ -877,11 +880,11 @@ dialog_sheet_order (WorkbookControlGUI *wbcg)
 	state->old_order  = NULL;
 	state->initial_colors_set = FALSE; 
 	state->image_padlock =  gtk_widget_render_icon (state->dialog,
-                                             "Gnumeric_Padlock",
+                                             "Gnumeric_Protection_Yes",
                                              GTK_ICON_SIZE_LARGE_TOOLBAR,
                                              "Gnumeric-Sheet-Manger");
 	state->image_padlock_no =  gtk_widget_render_icon (state->dialog,
-                                             "Gnumeric_PadlockNo",
+                                             "Gnumeric_Protection_No",
                                              GTK_ICON_SIZE_LARGE_TOOLBAR,
                                              "Gnumeric-Sheet-Manger");
 
@@ -957,11 +960,9 @@ dialog_sheet_order (WorkbookControlGUI *wbcg)
 		"sheet-order.html");
 
 	/* a candidate for merging into attach guru */
-	g_signal_connect (G_OBJECT (state->dialog),
-		"destroy",
-		G_CALLBACK (cb_sheet_order_destroy), state);
+	g_object_set_data_full (G_OBJECT (state->dialog),
+		"state", state, (GDestroyNotify) cb_sheet_order_destroy);
 	gnumeric_non_modal_dialog (state->wbcg, GTK_WINDOW (state->dialog));
 	wbcg_edit_attach_guru (state->wbcg, GTK_WIDGET (state->dialog));
 	gtk_widget_show_all (GTK_WIDGET (state->dialog));
-	
 }
