@@ -19,15 +19,14 @@
  */
 
 #include <gnumeric-config.h>
+#include <gnumeric-i18n.h>
 #include <gnumeric.h>
 #include "dialog-stf.h"
 
 #include <format.h>
 #include <command-context.h>
-#include <workbook-control.h>
 #include <gui-util.h>
-
-#include <libgnome/gnome-i18n.h>
+#include <gdk/gdkkeysyms.h>
 
 #define GLADE_FILE "dialog-stf.glade"
 
@@ -99,19 +98,8 @@ stf_dialog_set_scroll_region_and_prevent_center (GnomeCanvas *canvas, GnomeCanva
 static gboolean
 stf_dialog_druid_page_cancel (GnomeDruidPage *page, GnomeDruid *druid, DruidPageData_t *data)
 {
-	GtkWidget *dialog, *no_button;
-	int ret;
-
-	dialog = gnome_question_dialog_parented (_("Are you sure you want to cancel?"),
-						 NULL,
-						 NULL,
-						 data->window);
-
-	no_button = g_list_last (GNOME_DIALOG (dialog)->buttons)->data;
-	gtk_widget_grab_focus (no_button);
-	ret = gnome_dialog_run (GNOME_DIALOG (dialog));
-
-	return (ret==1);
+	return !gnumeric_dialog_question_yes_no (data->wbcg,
+		_("Are you sure you want to cancel?"), FALSE);
 }
 
 /**
@@ -275,7 +263,7 @@ stf_dialog_druid_page_previous (GnomeDruidPage *page, GnomeDruid *druid, DruidPa
 	}
 
 	if (newpos == DPG_MAIN)
-		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE);
+		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE, TRUE);
 
         nextpage = stf_dialog_druid_position_to_page (data, newpos);
 	if (!nextpage) {
@@ -288,7 +276,7 @@ stf_dialog_druid_page_previous (GnomeDruidPage *page, GnomeDruid *druid, DruidPa
 
 	stf_dialog_set_initial_keyboard_focus (data);
 	if (newpos == DPG_MAIN)
-		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE);
+		gnome_druid_set_buttons_sensitive (druid, FALSE, TRUE, TRUE, TRUE);
 	else
 		gtk_widget_grab_default (data->druid->next);
 	return TRUE;
@@ -374,92 +362,74 @@ stf_dialog_attach_page_signals (GladeXML *gui, DruidPageData_t *pagedata)
 /*	pagedata->stop_page   = GNOME_DRUID_PAGE (glade_xml_get_widget (gui, "stop_page"));*/
 
 	pagedata->position  = DPG_MAIN;
-	gnome_druid_set_buttons_sensitive (pagedata->druid, FALSE, TRUE, TRUE);
+	gnome_druid_set_buttons_sensitive (pagedata->druid, FALSE, TRUE, TRUE, TRUE);
 
 	/* Signals for individual pages */
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->main_page),
-			    "next",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_next),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->csv_page),
-			    "next",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_next),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->fixed_page),
-			    "next",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_next),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->format_page),
-			    "next",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_next),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->main_page),
+		"next",
+		G_CALLBACK (stf_dialog_druid_page_next), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->csv_page),
+		"next",
+		G_CALLBACK (stf_dialog_druid_page_next), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->fixed_page),
+		"next",
+		G_CALLBACK (stf_dialog_druid_page_next), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format_page),
+		"next",
+		G_CALLBACK (stf_dialog_druid_page_next), pagedata);
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->main_page),
-			    "back",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_previous),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->csv_page),
-			    "back",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_previous),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->fixed_page),
-			    "back",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_previous),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->format_page),
-			    "back",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_previous),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->main_page),
+		"back",
+		G_CALLBACK (stf_dialog_druid_page_previous), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->csv_page),
+		"back",
+		G_CALLBACK (stf_dialog_druid_page_previous), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->fixed_page),
+		"back",
+		G_CALLBACK (stf_dialog_druid_page_previous), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format_page),
+		"back",
+		G_CALLBACK (stf_dialog_druid_page_previous), pagedata);
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->main_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_cancel),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->csv_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_cancel),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->fixed_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_cancel),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->format_page),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_page_cancel),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->main_page),
+		"cancel",
+		G_CALLBACK (stf_dialog_druid_page_cancel), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->csv_page),
+		"cancel",
+		G_CALLBACK (stf_dialog_druid_page_cancel), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->fixed_page),
+		"cancel",
+		G_CALLBACK (stf_dialog_druid_page_cancel), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format_page),
+		"cancel",
+		G_CALLBACK (stf_dialog_druid_page_cancel), pagedata);
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->format_page),
-			    "finish",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_format_page_finish),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format_page),
+		"finish",
+		G_CALLBACK (stf_dialog_druid_format_page_finish), pagedata);
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->csv_page),
-			    "prepare",
-			    GTK_SIGNAL_FUNC (stf_dialog_csv_page_prepare),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->fixed_page),
-			    "prepare",
-			    GTK_SIGNAL_FUNC (stf_dialog_fixed_page_prepare),
-			    pagedata);
-	gtk_signal_connect (GTK_OBJECT (pagedata->format_page),
-			    "prepare",
-			    GTK_SIGNAL_FUNC (stf_dialog_format_page_prepare),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->csv_page),
+		"prepare",
+		G_CALLBACK (stf_dialog_csv_page_prepare), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->fixed_page),
+		"prepare",
+		G_CALLBACK (stf_dialog_fixed_page_prepare), pagedata);
+	g_signal_connect (G_OBJECT (pagedata->format_page),
+		"prepare",
+		G_CALLBACK (stf_dialog_format_page_prepare), pagedata);
 
 	/* Signals for the druid itself */
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->druid),
-			    "cancel",
-			    GTK_SIGNAL_FUNC (stf_dialog_druid_cancel),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->druid),
+		"cancel",
+		G_CALLBACK (stf_dialog_druid_cancel), pagedata);
 
 	/* And for the surrounding window */
 
-	gtk_signal_connect (GTK_OBJECT (pagedata->window),
-			    "key_press_event",
-			    GTK_SIGNAL_FUNC (stf_dialog_check_escape),
-			    pagedata);
+	g_signal_connect (G_OBJECT (pagedata->window),
+		"key_press_event",
+		G_CALLBACK (stf_dialog_check_escape), pagedata);
 }
 
 /**
@@ -475,22 +445,22 @@ stf_dialog_editables_enter (DruidPageData_t *pagedata)
 {
 	gnumeric_editable_enters
 		(pagedata->window,
-		 GTK_EDITABLE (&pagedata->main_info->main_startrow->entry));
+		 GTK_WIDGET (&pagedata->main_info->main_startrow->entry));
 	gnumeric_editable_enters
 		(pagedata->window,
-		 GTK_EDITABLE (&pagedata->main_info->main_stoprow->entry));
+		 GTK_WIDGET (&pagedata->main_info->main_stoprow->entry));
 	gnumeric_editable_enters
 		(pagedata->window,
-		 GTK_EDITABLE (pagedata->csv_info->csv_customseparator));
+		 GTK_WIDGET (pagedata->csv_info->csv_customseparator));
 	gnumeric_combo_enters
 		(pagedata->window,
 		pagedata->csv_info->csv_textindicator);
 	gnumeric_editable_enters
 		(pagedata->window,
-		 GTK_EDITABLE (&pagedata->fixed_info->fixed_colend->entry));
+		 GTK_WIDGET (&pagedata->fixed_info->fixed_colend->entry));
 	gnumeric_editable_enters
 		(pagedata->window,
-		 GTK_EDITABLE (pagedata->format_info->format_format));
+		 GTK_WIDGET (pagedata->format_info->format_format));
 }
 
 /**
@@ -534,6 +504,7 @@ stf_dialog (WorkbookControlGUI *wbcg, const char *filename, const char *data)
 
 	pagedata.canceled = FALSE;
 
+	pagedata.wbcg	  = wbcg;
 	pagedata.filename = filename;
 	pagedata.data     = data;
 	pagedata.cur      = data;
@@ -594,7 +565,7 @@ stf_dialog (WorkbookControlGUI *wbcg, const char *filename, const char *data)
 	stf_dialog_format_page_cleanup (&pagedata);
 
 	gtk_widget_destroy (GTK_WIDGET (pagedata.window));
-	gtk_object_unref (GTK_OBJECT (gui));
+	g_object_unref (G_OBJECT (gui));
 
 	return dialogresult;
 }
@@ -626,12 +597,3 @@ stf_dialog_result_free (DialogStfResult_t *dialogresult)
 
 	g_free (dialogresult);
 }
-
-
-
-
-
-
-
-
-
