@@ -39,7 +39,6 @@
 #include "formats.h"
 
 #include <locale.h>
-#include <ctype.h>
 #include <string.h>
 #include <stdio.h>
 #include <errno.h>
@@ -52,12 +51,14 @@
 
 /* Points to the locale information for number display */
 static struct lconv *lc = NULL;
+static char *locale_currency = NULL; /* in UTF-8 */
 static gboolean date_order_cached = FALSE;
 
 char const *
 gnumeric_setlocale (int category, char const *val)
 {
 	lc = NULL;
+	g_free (locale_currency); locale_currency = NULL;
 	date_order_cached = FALSE;
 	return setlocale (category, val);
 }
@@ -116,9 +117,14 @@ format_get_currency (gboolean *precedes, gboolean *space_sep)
 	if (space_sep)
 		*space_sep = (lc->p_sep_by_space == 1);
 
-	if (lc->currency_symbol == NULL || *lc->currency_symbol == '\0')
-		return "$";
-	return lc->currency_symbol;
+	if (locale_currency == NULL) {
+		locale_currency = (lc->currency_symbol == NULL ||
+				   *lc->currency_symbol == '\0')
+			? g_strdup ("$")
+			: g_locale_to_utf8 (lc->currency_symbol, -1,
+					    NULL, NULL, NULL);
+	}
+	return locale_currency;
 }
 
 /*
@@ -140,11 +146,11 @@ format_month_before_day (void)
 		month_first = TRUE;
 		if (ptr)
 			while (*ptr) {
-				char const tmp = tolower((unsigned char)*ptr++);
-				if (tmp == 'd') {
+				char c = *ptr++;
+				if (c == 'd' || c == 'D') {
 					month_first = FALSE;
 					break;
-				} else if (tmp == 'm')
+				} else if (c == 'm' || c == 'M')
 					break;
 			}
 	}
@@ -219,19 +225,21 @@ static int
 append_year (GString *string, const guchar *format, const struct tm *time_split)
 {
 	char temp[5];
+	int year = time_split->tm_year + 1900;
 
-	if (tolower (format[1]) != 'y'){
+	if (format[1] != 'y' && format[1] != 'Y') {
 		g_string_append_c (string, 'y');
 		return 1;
 	}
 
-	if (tolower (format[2]) != 'y' || tolower (format[3]) != 'y'){
-		sprintf (temp, "%02d", time_split->tm_year % 100);
+	if ((format[2] != 'y' && format[2] != 'Y') ||
+	    (format[3] != 'y' && format[3] != 'Y')) {
+		sprintf (temp, "%02d", year % 100);
 		g_string_append (string, temp);
 		return 2;
 	}
 
-	sprintf (temp, "%04d", time_split->tm_year + 1900);
+	sprintf (temp, "%04d", year);
 	g_string_append (string, temp);
 
 	return 4;
@@ -276,19 +284,19 @@ append_day (GString *string, const guchar *format, const struct tm *time_split)
 {
 	char temp[3];
 
-	if (tolower (format[1]) != 'd'){
+	if (format[1] != 'd' && format[1] != 'D') {
 		sprintf (temp, "%d", time_split->tm_mday);
 		g_string_append (string, temp);
 		return 1;
 	}
 
-	if (tolower (format[2]) != 'd'){
+	if (format[2] != 'd' && format[2] != 'D') {
 		sprintf (temp, "%02d", time_split->tm_mday);
 		g_string_append (string, temp);
 		return 2;
 	}
 
-	if (tolower (format[3]) != 'd'){
+	if (format[3] != 'd' && format[3] != 'D') {
 		g_string_append (string, _(day_short[time_split->tm_wday]) + 1);
 		return 3;
 	}
@@ -510,7 +518,7 @@ format_compile (StyleFormat *format)
 		}
 
 		case '/':
-			if (fmt[1] == '?' || isdigit ((unsigned char)fmt[1])) {
+			if (fmt[1] == '?' || (fmt[1] >= '0' && fmt[1] <= '9')) {
 				entry->has_fraction = TRUE;
 				fmt++;
 			}
@@ -518,7 +526,7 @@ format_compile (StyleFormat *format)
 
 		case 'a': case 'A':
 		case 'p': case 'P':
-			if (tolower (fmt[1]) == 'm')
+			if (fmt[1] == 'm' || fmt[1] == 'M')
 				entry->want_am_pm = TRUE;
 			break;
 
@@ -773,7 +781,7 @@ do_render_number (gnum_float number, format_info_t *info, GString *result)
 		decimal_point);
 #endif
 
-	render_number (result, number, info);
+	render_number (result, info->scale * number, info);
 }
 
 /*
@@ -984,7 +992,7 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 	gboolean time_display_elapsed = FALSE;
 	gboolean ignore_further_elapsed = FALSE;
 
-	char fill_char = '\0';
+	gunichar fill_char = 0;
 	int fill_start = -1;
 
 	struct tm *time_split = 0;
@@ -998,9 +1006,10 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 		number = -number;
 	}
 	info.has_fraction = entry->has_fraction;
+	info.scale = 1;
 
 	while (*format) {
-		int c = *format;
+		gunichar c = g_utf8_get_char (format);
 
 		switch (c) {
 
@@ -1015,7 +1024,7 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 					else if (no_locale)
 						g_string_append_c (result, *format);
 				if (!*format)
-					--format;
+					continue;
 			} else if (!ignore_further_elapsed)
 				time_display_elapsed = TRUE;
 			break;
@@ -1057,14 +1066,20 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 			break;
 		}
 
-		case ',': {
-			if (!can_render_number) {
-				char const sep = format_get_thousand ();
-				g_string_append_c (result, sep);
-			} else
+		case ',':
+			if (can_render_number) {
+				guchar const *tmp = format;
+				while (*++tmp == ',')
+					;
+				if (*tmp == '\0' || *tmp == '.' || *tmp == ';')
+					/* NOTE : format-tmp is NEGATIVE */
+					info.scale = gpow10 (3*(format-tmp));
 				info.group_thousands = TRUE;
+				format = tmp;
+				continue;
+			} else
+				g_string_append_c (result, format_get_thousand ());
 			break;
-		}
 
 		/* FIXME: this is a gross hack */
 		case 'E': case 'e': {
@@ -1100,18 +1115,22 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 					do_render_number (number, &info, result);
 
 				format++;
-				g_string_append_c (result, *format);
+				g_string_append_len (result, format,
+					g_utf8_skip[*(format)]);
 			}
 			break;
 
 		case '"': {
+			guchar const *tmp = ++format;
 			if (can_render_number && !info.rendered)
 				do_render_number (number, &info, result);
 
-			for (format++; *format && *format != '"'; format++)
-				g_string_append_c (result, *format);
+			for (; *tmp && *tmp != '"'; tmp++)
+				;
+			g_string_append_len (result, format, tmp-format);
+			format = tmp;
 			if (!*format)
-				--format;
+				continue;
 			break;
 		}
 
@@ -1169,13 +1188,13 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 
 		case ' ': /* eg # ?/? */
 		case '$':
-		case '':
-		case '':
-		case '':
+		case 0x00A3 : /* pound */
+		case 0x00A5 : /* yen */
+		case 0x20AC : /* Euro */
 		case ')':
 			if (can_render_number && !info.rendered)
 				do_render_number (number, &info, result);
-			g_string_append_c (result, *format);
+			g_string_append_unichar (result, c);
 			break;
 
 		/* percent */
@@ -1209,7 +1228,8 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 			if (format[1]) {
 				if (can_render_number && !info.rendered)
 					do_render_number (number, &info, result);
-				fill_char = *(++format);
+				++format;
+				fill_char = g_utf8_get_char (format);
 				fill_start = result->len;
 			}
 			break;
@@ -1234,7 +1254,8 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 				ignore_further_elapsed = TRUE;
 				append_minute_elapsed (result, time_split, number);
 			} else if (hour_seen ||
-				   (format[1] == ':' && tolower (format[2]) == 's')){
+				   (format[1] == ':' &&
+				    (format[2] == 's' || format[2] == 'S'))) {
 				append_minute (result, n, time_split);
 			} else
 				append_month (result, n, time_split);
@@ -1344,7 +1365,7 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 			 */
 			break;
 		}
-		format++;
+		format = g_utf8_next_char (format);
 	}
 
 	if (!info.rendered && can_render_number)
@@ -1354,7 +1375,7 @@ format_number (gnum_float number, int col_width, StyleFormatEntry const *entry)
 	if (fill_char != '\0') {
 		int count = col_width - result->len;
 		while (count-- > 0)
-			g_string_insert_c (result, fill_start, fill_char);
+			g_string_insert_unichar (result, fill_start, fill_char);
 	}
 
  finish:
@@ -1480,7 +1501,7 @@ fmt_general_int (int val, int col_width)
 		if (val < 0) {
 			/* leave space for minus sign */
 			col_width--;
-			log_val = ceil (log10 (-val));
+			log_val = ceil (log10 ((unsigned int)-val));
 		} else
 			log_val = (val > 0) ? ceil (log10 (val)) : 0;
 
@@ -1517,7 +1538,7 @@ format_value (StyleFormat const *format, Value const *value, StyleColor **color,
 	 * This wont work for ranges because we dont't have a location
 	 */
 	if (value->type == VALUE_ARRAY)
-		value = value_area_fetch_x_y (NULL, value, 0, 0);
+		value = value_area_fetch_x_y (value, 0, 0, NULL);
 
 	if (format) {
 		/* get format */
@@ -1555,7 +1576,7 @@ format_value (StyleFormat const *format, Value const *value, StyleColor **color,
 	case VALUE_BOOLEAN:
 		return g_strdup (value->v_bool.val ? _("TRUE"):_("FALSE"));
 	case VALUE_INTEGER: {
-		gnum_int val = value->v_int.val;
+		int val = value->v_int.val;
 		if (need_abs)
 			val = ABS (val);
 
