@@ -36,7 +36,7 @@ StyleFormat   *style_format_default_money	(void);
 
 void   format_destroy (StyleFormat *format);
 char  *format_value   (StyleFormat const *format, Value const *value, StyleColor **color,
-		       float col_width);
+		       double col_width);
 
 void   format_color_init     (void);
 void   format_color_shutdown (void);
@@ -47,6 +47,7 @@ char  *format_remove_decimal   (StyleFormat const *fmt);
 typedef struct {
 	int  right_optional, right_spaces, right_req, right_allowed;
 	int  left_spaces, left_req;
+	float scale;
 	gboolean rendered;
 	gboolean negative;
 	gboolean decimal_separator_seen;
