@@ -31,8 +31,8 @@ typedef struct
 	gint	 negative_fmt;	/* 0 - 3 */
 	gint	 currency_symbol_index;
 	gint	 list_element;
-	gboolean date_has_days : 1;
-	gboolean date_has_months : 1;
+	gboolean date_has_days;
+	gboolean date_has_months;
 } FormatCharacteristics;
 
 FormatFamily cell_format_classify (StyleFormat const *fmt, FormatCharacteristics *info);
