@@ -63,8 +63,8 @@ GnmFuncGroup *gnm_func_group_fetch_with_translation (char const *name,
  **/
 
 typedef enum {
-	GNM_FUNC_TYPE_ARGS,	/* Takes unevaulated expers directly */
-	GNM_FUNC_TYPE_NODES,	/* arguments get marshalled by type */
+	GNM_FUNC_TYPE_ARGS,	/* Arguments get marshalled by type */
+	GNM_FUNC_TYPE_NODES,	/* Takes unevaulated expers directly */
 
 	/* implementation has not been loaded yet, but we know where it is */
 	GNM_FUNC_TYPE_STUB
@@ -83,7 +83,8 @@ typedef enum {
 	GNM_FUNC_RECALC_ONLOAD 		= 0x04,
 
 	/* an unknown function that will hopefully be defined later */
-	GNM_FUNC_IS_PLACEHOLDER		= 0x08
+	GNM_FUNC_IS_PLACEHOLDER		= 0x08,
+	GNM_FUNC_FREE_NAME		= 0x10
 } GnmFuncFlags;
 
 /* I do not like this it is going to be different for different apps
