@@ -4,7 +4,6 @@
 #include "gnumeric.h"
 #include <gtk/gtkwidget.h>
 #include <libxml/tree.h>
-#include <bonobo.h>
 /* Do not include idl here due to automake irritaion for the non-bonobo case */
 
 typedef enum {
@@ -27,10 +26,18 @@ void	   gnm_graph_range_to_vectors	  (GnmGraph *g, Sheet *sheet,
 					   Range const *src, gboolean as_cols);
 xmlDoc	  *gnm_graph_get_spec		  (GnmGraph *g, gboolean force_update);
 void	   gnm_graph_import_specification (GnmGraph *graph, xmlDoc *spec);
-int	   gnm_graph_add_vector	   	  (GnmGraph *graph, GnmExpr *expr,
+int	   gnm_graph_add_vector	   	  (GnmGraph *graph, GnmExpr const *expr,
 					   GnmGraphVectorType type, Sheet *s);
+
 GnmGraphVector *gnm_graph_get_vector	  (GnmGraph *graph, int id);
 
+void	      gnm_graph_remove_plot   (GnmGraph *graph, GnmGraphPlot *plot);
+GnmGraphPlot *gnm_graph_add_plot      (GnmGraph *graph);
+
+void	      gnm_graph_plot_set_type (GnmGraphPlot *plot, xmlNode *type);
+void	      gnm_graph_plot_remove_series (GnmGraphPlot *plot, GnmGraphSeries *series);
+GnmGraphPlot *gnm_graph_plot_add_series    (GnmGraphPlot *plot);
+void	      gnm_graph_plot_set_type (GnmGraphPlot *plot, xmlNode *type);
 extern char const * const gnm_graph_vector_type_name [];
 
 #define GNUMERIC_GRAPH_VECTOR_TYPE	(gnm_graph_vector_get_type ())
@@ -44,7 +51,4 @@ Dependent const *gnm_graph_vector_get_dependent (GnmGraphVector const *v);
 xmlNode *gnm_graph_series_get_dimension (xmlNode *series, xmlChar const *element);
 xmlNode *gnm_graph_series_add_dimension (xmlNode *series, char const *element);
 
-char 	       *gnm_graph_exception	     (CORBA_Environment *ev);
-Bonobo_Control  gnm_graph_get_config_control (GnmGraph *g, char const *which);
-
 #endif /* GNUMERIC_GRAPH_H */
