@@ -16,12 +16,10 @@ size_t	g_unichar_strlen  (gunichar const *str);
 int	g_unichar_strncmp (gunichar const *a, gunichar const *b, size_t n);
 
 void	  g_ptr_array_insert (GPtrArray *array, gpointer value, int index);
-GList    *g_list_map         (GList *list, GnmMapFunc map_func);
+int	  g_str_compare	     (void const *x, void const *y);
 GList    *g_create_list	     (gpointer item1, ...);
 gint      g_list_index_custom (GList *list, gpointer data, GCompareFunc cmp_func);
 void      g_list_free_custom (GList *list, GFreeFunc free_func);
-#define   g_string_list_copy(list) g_list_map (list, (GnmMapFunc) g_strdup)
-GList    *g_strsplit_to_list (const gchar *string, const gchar *delimiter);
 #define GNM_LIST_FOREACH(list,valtype,val,stmnt) \
 G_STMT_START { \
 	GList *gnm_l; \
@@ -73,8 +71,9 @@ G_STMT_START { \
 
 #define GNM_SIZEOF_ARRAY(array) ((int) (sizeof (array) / sizeof ((array)[0])))
 
-guint     gnumeric_strcase_hash  (gconstpointer v);
-gint      gnumeric_strcase_equal (gconstpointer v, gconstpointer v2);
+guint     gnumeric_ascii_strcase_hash  (gconstpointer v);
+gint      gnumeric_ascii_strcase_equal (gconstpointer v, gconstpointer v2);
+gint      gnumeric_utf8_collate_casefold (const char *a, const char *b);
 
 /**
  * System and user paths
@@ -91,6 +90,8 @@ char *    gnumeric_strescape (const char *string);
 
 char *    gnumeric_utf8_strcapital (const char *p, ssize_t len);
 
+gboolean  gnumeric_valid_filename (const char *filename);
+
 gnm_mem_chunk *gnm_mem_chunk_new (const char *, size_t, size_t);
 void gnm_mem_chunk_destroy (gnm_mem_chunk *, gboolean);
 gpointer gnm_mem_chunk_alloc (gnm_mem_chunk *);
