@@ -19,6 +19,7 @@
 #include "workbook-control-gui-priv.h"
 #include "sheet-control-gui-priv.h"
 #include "sheet.h"
+#include "sheet-view.h"
 #include "sheet-style.h"
 #include "sheet-merge.h"
 #include "sheet-object-impl.h"
@@ -58,7 +59,7 @@ static GnomeCanvasItemClass *item_grid_parent_class;
 enum {
 	ARG_0,
 	ARG_SHEET_CONTROL_GUI,
-	ARG_SHEET_BOUND,
+	ARG_SHEET_BOUND
 };
 typedef enum {
 	ITEM_GRID_NO_SELECTION,
@@ -301,9 +302,9 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 		int draw_x, int draw_y, int width, int height)
 {
 	GnomeCanvas *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	Sheet const *sheet = ((SheetControl *) gcanvas->simple.scg)->sheet;
-	Cell const * const edit_cell = gcanvas->simple.scg->wbcg->editing_cell;
+	Cell const * const edit_cell = gcanvas->simple.scg->wbcg->wb_control.editing_cell;
 	ItemGrid *ig = ITEM_GRID (item);
 	ColRowInfo const *ri = NULL, *next_ri = NULL;
 
@@ -428,6 +429,12 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 			}
 		}
 
+		/* its safe to const_cast because only the a non-default row
+		 * will ever get flagged.
+		 */
+		if (ri->needs_respan)
+			row_calc_spans ((ColRowInfo *)ri, sheet);
+
 		/* look for merges that start on this row, on the first painted row
 		 * also check for merges that start above. */
 		view.start.row = row;
@@ -654,12 +661,9 @@ item_grid_draw (GnomeCanvasItem *item, GdkDrawable *drawable,
 	    (gcanvas->pane->index == 1 || gcanvas->pane->index == 2))
 		gdk_draw_line (drawable, ig->gc.bound, x, 0, x, y);
 
-	if (merged_used != NULL)	/* ranges whose bottoms are in the view */
-		g_slist_free (merged_used);
-	if (merged_active_seen != NULL) /* ranges whose bottoms are below the view */
-		g_slist_free (merged_active_seen);
-	if (merged_unused != NULL);	/* merges in hidden rows */
-		g_slist_free (merged_unused);
+	g_slist_free (merged_used);	   /* merges with bottom in view */
+	g_slist_free (merged_active_seen); /* merges with bottom the view */
+	g_slist_free (merged_unused);	   /* merges in hidden rows */
 	g_return_if_fail (merged_active == NULL);
 }
 
@@ -781,13 +785,13 @@ ig_obj_create_finish (ItemGrid *ig, GdkEventButton *event)
  * ig_obj_create_begin :
  *
  * Starts the process of creating a SheetObject.  Handles the initial
- * button press on the GnumericCanvas.
+ * button press on the GnmCanvas.
  */
 static gboolean
 ig_obj_create_begin (ItemGrid *ig, GdkEventButton *event)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (ig);
-	GnumericCanvas  *gcanvas = GNUMERIC_CANVAS (item->canvas);
+	GnmCanvas  *gcanvas = GNM_CANVAS (item->canvas);
 	SheetControlGUI *scg;
 	SheetObject *so;
 
@@ -857,12 +861,13 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 {
 	GnomeCanvasItem *item = GNOME_CANVAS_ITEM (ig);
 	GnomeCanvas    *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	SheetControlGUI *scg = ig->scg;
 	SheetControl *sc = (SheetControl *) scg;
 	Sheet *sheet = sc->sheet;
 	CellPos	pos;
 	int x, y;
+	gboolean already_selected;
 
 	gnm_canvas_slide_stop (gcanvas);
 
@@ -931,7 +936,8 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
 	/* button 1 will always change the selection,  the other buttons will
 	 * only effect things if the target is not already selected.
 	 */
-	if (event->button == 1 || !sv_is_pos_selected (sc->view, pos.col, pos.row)) {
+	already_selected = sv_is_pos_selected (sc->view, pos.col, pos.row);
+	if (event->button == 1 || !already_selected) {
 		if (!(event->state & (GDK_CONTROL_MASK|GDK_SHIFT_MASK)))
 			sv_selection_reset (sc->view);
 
@@ -947,13 +953,20 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
       	switch (event->button) {
 	case 1: {
 		guint32 double_click_time;
-		g_object_get (gtk_widget_get_settings (GTK_WIDGET (canvas)),
-			"gtk-double-click-time", &double_click_time,
-			NULL);
 
-		if ((ig->last_click_time + double_click_time) > event->time &&
-		    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
-			break;
+		/*
+		 *  If the second click is on a different cell than the
+		 *  first one this cannot be a double-click
+		 */
+		if (already_selected) {
+			g_object_get (gtk_widget_get_settings (GTK_WIDGET (canvas)),
+				      "gtk-double-click-time", &double_click_time,
+				      NULL);
+			
+			if ((ig->last_click_time + double_click_time) > event->time &&
+			    wbcg_edit_start (scg->wbcg, FALSE, FALSE))
+				break;
+		}
 
 		ig->last_click_time = event->time;
 		ig->selecting = ITEM_GRID_SELECTING_CELL_RANGE;
@@ -982,21 +995,21 @@ item_grid_button_press (ItemGrid *ig, GdkEventButton *event)
  */
 
 static gboolean
-cb_extend_cell_range (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_cell_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	sv_selection_extend_to (((SheetControl *) gcanvas->simple.scg)->view, col, row);
 	return TRUE;
 }
 
 static gboolean
-cb_extend_expr_range (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_expr_range (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	scg_rangesel_extend_to (gcanvas->simple.scg, col, row);
 	return TRUE;
 }
 
 static gboolean
-cb_extend_object_creation (GnumericCanvas *gcanvas, int col, int row, gpointer ignored)
+cb_extend_object_creation (GnmCanvas *gcanvas, int col, int row, gpointer ignored)
 {
 	int x, y;
 	gdouble new_x, new_y;
@@ -1017,7 +1030,7 @@ cb_cursor_come_to_rest (ItemGrid *ig)
 {
 	Sheet const *sheet = ((SheetControl *) ig->scg)->view->sheet;
 	GnomeCanvas *canvas = ig->canvas_item.canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	GnmHLink *link;
 	int x, y;
 	CellPos pos;
@@ -1049,7 +1062,7 @@ cb_cursor_motion (ItemGrid *ig)
 {
 	Sheet const *sheet = ((SheetControl *) ig->scg)->view->sheet;
 	GnomeCanvas *canvas = ig->canvas_item.canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	int x, y, cursor;
 	CellPos pos;
 	GnmHLink *old_link;
@@ -1078,7 +1091,7 @@ static gint
 item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 {
 	GnomeCanvas *canvas = item->canvas;
-	GnumericCanvas *gcanvas = GNUMERIC_CANVAS (canvas);
+	GnmCanvas *gcanvas = GNM_CANVAS (canvas);
 	ItemGrid *ig = ITEM_GRID (item);
 	SheetControlGUI *scg = ig->scg;
 	SheetControl *sc = (SheetControl *) scg;
@@ -1142,7 +1155,7 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 	}
 
 	case GDK_MOTION_NOTIFY: {
-		GnumericCanvasSlideHandler slide_handler = NULL;
+		GnmCanvasSlideHandler slide_handler = NULL;
 		switch (ig->selecting) {
 		case ITEM_GRID_NO_SELECTION:
 			if (ig->cursor_timer == 0)
@@ -1173,7 +1186,7 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 			 * that is more work than I want to do right now.
 			 */
 			if (gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
-						      GNM_SLIDE_X | GNM_SLIDE_Y,
+						      GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y,
 						      &cb_extend_object_creation, NULL))
 				ig_obj_create_motion (ig, event->motion.x, event->motion.y);
 
@@ -1183,7 +1196,8 @@ item_grid_event (GnomeCanvasItem *item, GdkEvent *event)
 		};
 
 		gnm_canvas_handle_motion (gcanvas, canvas, &event->motion,
-			GNM_SLIDE_X | GNM_SLIDE_Y | GNM_SLIDE_AT_COLROW_BOUND,
+			GNM_CANVAS_SLIDE_X | GNM_CANVAS_SLIDE_Y |
+			GNM_CANVAS_SLIDE_AT_COLROW_BOUND,
 			slide_handler, NULL);
 		return TRUE;
 	}
@@ -1253,7 +1267,7 @@ item_grid_class_init (ItemGridClass *item_grid_class)
 	GnomeCanvasItemClass *item_class;
 
 	item_grid_parent_class =
-		gtk_type_class (gnome_canvas_item_get_type ());
+		g_type_class_peek (gnome_canvas_item_get_type ());
 
 	object_class = (GtkObjectClass *) item_grid_class;
 	item_class = (GnomeCanvasItemClass *) item_grid_class;
