@@ -67,6 +67,7 @@ typedef struct {
 
 		Validation      *validation;
 		GnmHLink        *hlink;
+		GnmInputMsg	*input_msg;
 
 		/* Convenience members */
 		gpointer         any_pointer;
@@ -98,7 +99,8 @@ struct _MStyle {
 #define MSTYLE_ANY_POINTER           MSTYLE_FONT_NAME: \
 				case MSTYLE_FORMAT: \
 				case MSTYLE_VALIDATION: \
-				case MSTYLE_HLINK
+				case MSTYLE_HLINK: \
+				case MSTYLE_INPUT_MSG
 
 #define MSTYLE_ANY_BOOLEAN           MSTYLE_FONT_BOLD: \
 				case MSTYLE_FONT_ITALIC: \
@@ -146,7 +148,8 @@ const char *mstyle_names[MSTYLE_ELEMENT_MAX] = {
 	"Content.Locked",
 	"Content.Hidden",
 	"Validation",
-	"Hyper Link"
+	"Hyper Link",
+	"Input Msg"
 };
 
 /* Some ref/link count debugging */
@@ -205,9 +208,12 @@ mstyle_hash (gconstpointer st)
 		case MSTYLE_FONT_UNDERLINE:
 			hash = hash ^ e->u.font.underline;
 			break;
+
+#ifndef DEBUG_SWITCH_ENUM
 		default:
-			g_warning ("Unimplemented hash item");
+			g_assert_not_reached ();
 			break;
+#endif
 		}
 	}
 
@@ -303,6 +309,10 @@ mstyle_element_dump (const MStyleElement *e)
 		g_string_printf (ans, "hlink %p", e->u.hlink);
 		break;
 
+	case MSTYLE_INPUT_MSG :
+		g_string_printf (ans, "input msg %p", e->u.input_msg);
+		break;
+
 	default:
 		g_string_printf (ans, "%s", mstyle_names[e->type]);
 		break;
@@ -365,6 +375,8 @@ mstyle_element_equal (MStyleElement const *a,
 		return (a->u.validation == b->u.validation);
 	case MSTYLE_HLINK:
 		return (a->u.hlink == b->u.hlink);
+	case MSTYLE_INPUT_MSG:
+		return (a->u.input_msg == b->u.input_msg);
 	default:
 		return TRUE;
 	}
@@ -396,6 +408,10 @@ mstyle_element_ref (const MStyleElement *e)
 		if (e->u.hlink)
 			g_object_ref (G_OBJECT (e->u.hlink));
 		break;
+	case MSTYLE_INPUT_MSG:
+		if (e->u.input_msg)
+			g_object_ref (G_OBJECT (e->u.input_msg));
+		break;
 	default:
 		break;
 	}
@@ -426,6 +442,10 @@ mstyle_element_unref (MStyleElement e)
 		if (e.u.hlink)
 			g_object_unref (G_OBJECT (e.u.hlink));
 		break;
+	case MSTYLE_INPUT_MSG:
+		if (e.u.input_msg)
+			g_object_unref (G_OBJECT (e.u.input_msg));
+		break;
 	default:
 		break;
 	}
@@ -600,6 +620,7 @@ mstyle_new_default (void)
 	mstyle_set_font_strike (mstyle, FALSE);
 
 	mstyle_set_hlink       (mstyle, NULL);
+	mstyle_set_input_msg   (mstyle, NULL);
 
 	mstyle_set_color       (mstyle, MSTYLE_COLOR_FORE,
 				style_color_black ());
@@ -902,7 +923,7 @@ mstyle_equal_XL (const MStyle *a, const MStyle *b)
 
 	ea = a->elements;
 	eb = b->elements;
-	for (i = 1; i <= MSTYLE_HLINK; i++) {
+	for (i = 1; i < MSTYLE_VALIDATION; i++) {
 		/* Elements in the same position should have the same types */
 		if (ea[i].type != eb[i].type) {
 			if (ea[i].type != MSTYLE_ELEMENT_UNSET &&
@@ -1369,7 +1390,7 @@ mstyle_set_content_locked (MStyle *style, gboolean f)
 	g_return_if_fail (style != NULL);
 
 	style->elements[MSTYLE_CONTENT_LOCKED].type = MSTYLE_CONTENT_LOCKED;
-	style->elements[MSTYLE_CONTENT_LOCKED].u.wrap_text = f;
+	style->elements[MSTYLE_CONTENT_LOCKED].u.content_locked = f;
 }
 
 gboolean
@@ -1377,7 +1398,7 @@ mstyle_get_content_locked (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_CONTENT_LOCKED), FALSE);
 
-	return style->elements [MSTYLE_CONTENT_LOCKED].u.wrap_text;
+	return style->elements [MSTYLE_CONTENT_LOCKED].u.content_locked;
 }
 void
 mstyle_set_content_hidden (MStyle *style, gboolean f)
@@ -1385,7 +1406,7 @@ mstyle_set_content_hidden (MStyle *style, gboolean f)
 	g_return_if_fail (style != NULL);
 
 	style->elements[MSTYLE_CONTENT_HIDDEN].type = MSTYLE_CONTENT_HIDDEN;
-	style->elements[MSTYLE_CONTENT_HIDDEN].u.wrap_text = f;
+	style->elements[MSTYLE_CONTENT_HIDDEN].u.content_hidden = f;
 }
 
 gboolean
@@ -1393,7 +1414,7 @@ mstyle_get_content_hidden (const MStyle *style)
 {
 	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_CONTENT_HIDDEN), FALSE);
 
-	return style->elements [MSTYLE_CONTENT_HIDDEN].u.wrap_text;
+	return style->elements [MSTYLE_CONTENT_HIDDEN].u.content_hidden;
 }
 
 void
@@ -1405,7 +1426,6 @@ mstyle_set_validation (MStyle *style, Validation *v)
 	mstyle_element_unref (style->elements[MSTYLE_VALIDATION]);
 	style->elements[MSTYLE_VALIDATION].type = MSTYLE_VALIDATION;
 	style->elements[MSTYLE_VALIDATION].u.validation = v;
-
 }
 
 Validation *
@@ -1424,7 +1444,6 @@ mstyle_set_hlink (MStyle *style, GnmHLink *link)
 	mstyle_element_unref (style->elements[MSTYLE_HLINK]);
 	style->elements[MSTYLE_HLINK].type = MSTYLE_HLINK;
 	style->elements[MSTYLE_HLINK].u.hlink = link;
-
 }
 
 GnmHLink *
@@ -1435,6 +1454,24 @@ mstyle_get_hlink (const MStyle *style)
 	return style->elements[MSTYLE_HLINK].u.hlink;
 }
 
+void
+mstyle_set_input_msg (MStyle *style, GnmInputMsg *msg)
+{
+	g_return_if_fail (style != NULL);
+
+	mstyle_element_unref (style->elements[MSTYLE_INPUT_MSG]);
+	style->elements[MSTYLE_INPUT_MSG].type = MSTYLE_INPUT_MSG;
+	style->elements[MSTYLE_INPUT_MSG].u.input_msg = msg;
+}
+
+GnmInputMsg *
+mstyle_get_input_msg (const MStyle *style)
+{
+	g_return_val_if_fail (mstyle_is_element_set (style, MSTYLE_INPUT_MSG), NULL);
+
+	return style->elements[MSTYLE_INPUT_MSG].u.input_msg;
+}
+
 gboolean
 mstyle_visible_in_blank (const MStyle *st)
 {
