@@ -44,12 +44,15 @@ typedef enum _MStyleElementType {
 		MSTYLE_ORIENTATION,
 
 		MSTYLE_WRAP_TEXT,
+		MSTYLE_SHRINK_TO_FIT,
 
 	        MSTYLE_CONTENT_LOCKED,
 	        MSTYLE_CONTENT_HIDDEN,
 
+	/* Things not in MS Excel's Style */
 	        MSTYLE_VALIDATION,
-	        MSTYLE_HLINK,
+	        MSTYLE_HLINK,		/* patch equal_XL if this is changed */
+	        MSTYLE_INPUT_MSG,	/* patch equal_XL if this is changed */
 	/* Delimiter */
 	MSTYLE_ELEMENT_MAX
 } MStyleElementType;
@@ -66,8 +69,9 @@ void        mstyle_link          (MStyle *st);
 void        mstyle_link_multiple (MStyle *st, int count);
 void        mstyle_unlink        (MStyle *st);
 
-gboolean    mstyle_equal         (const MStyle *a, const MStyle *b);
-gboolean    mstyle_verify        (const MStyle *st);
+gboolean    mstyle_equal         (MStyle const *a, MStyle const *b);
+gboolean    mstyle_equal_XL	 (MStyle const *a, MStyle const *b);
+gboolean    mstyle_verify        (MStyle const *st);
 guint       mstyle_hash          (gconstpointer st);
 gboolean    mstyle_empty         (const MStyle *st);
 
@@ -115,19 +119,24 @@ void                mstyle_set_orientation (MStyle *st, StyleOrientation o);
 StyleOrientation    mstyle_get_orientation (const MStyle *st);
 void                mstyle_set_wrap_text   (MStyle *st, gboolean f);
 gboolean            mstyle_get_wrap_text   (const MStyle *st);
+void                mstyle_set_shrink_to_fit (MStyle *st, gboolean f);
+gboolean            mstyle_get_shrink_to_fit (const MStyle *st);
 
 void                mstyle_set_content_locked (MStyle *st, gboolean f);
 gboolean            mstyle_get_content_locked (const MStyle *st);
 void                mstyle_set_content_hidden (MStyle *st, gboolean f);
 gboolean            mstyle_get_content_hidden (const MStyle *st);
 
-void                mstyle_set_validation       (MStyle *st, Validation *v);
-Validation         *mstyle_get_validation       (const MStyle *st);
+void                mstyle_set_validation	(MStyle *st, Validation *v);
+Validation         *mstyle_get_validation	(const MStyle *st);
 
 void                mstyle_set_hlink		(MStyle *st, GnmHLink *link);
 GnmHLink	   *mstyle_get_hlink		(const MStyle *st);
 
-gboolean            mstyle_visible_in_blank(const MStyle *st);
+void                mstyle_set_input_msg	(MStyle *st, GnmInputMsg *msg);
+GnmInputMsg   	   *mstyle_get_input_msg	(const MStyle *st);
+
+gboolean            mstyle_visible_in_blank (const MStyle *st);
 
 char       *mstyle_to_string   (const MStyle *st); /* Debug only ! leaks like a sieve */
 void        mstyle_dump        (const MStyle *st);
