@@ -26,7 +26,7 @@
 #include "parse-util.h"
 
 #include "application.h"
-#include "workbook.h"
+#include "workbook-priv.h"
 #include "sheet.h"
 #include "value.h"
 #include "ranges.h"
@@ -38,32 +38,40 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <stdio.h>
-#include <ctype.h>
 #include <glib.h>
 #include <string.h>
 
 inline static char *
 col_name_internal (char *buf, int col)
 {
-	g_return_val_if_fail (col < SHEET_MAX_COLS, buf);
-	g_return_val_if_fail (col >= 0, buf);
+  static const int steps[] = {
+	  26,
+	  26 * 26,
+	  26 * 26 * 26,
+	  26 * 26 * 26 * 26,
+	  26 * 26 * 26 * 26 * 26,
+	  26 * 26 * 26 * 26 * 26 * 26,
+	  INT_MAX
+  };
+  int i;
+  char *res;
 
-	if (col <= 'Z'-'A') {
-		*buf++ = col + 'A';
-	} else {
-		int a = col / ('Z'-'A'+1);
-		int b = col % ('Z'-'A'+1);
+  for (i = 0; col >= steps[i]; i++)
+    col -= steps[i];
 
-		*buf++ = a + 'A' - 1;
-		*buf++ = b + 'A';
-	}
-	return buf;
+  res = buf + i + 1;
+  while (i >= 0) {
+    buf[i--] = 'A' + col % 26;
+    col /= 26;
+  }
+
+  return res;
 }
 
 char const *
 col_name (int col)
 {
-	static char buffer [3]; /* What if SHEET_MAX_COLS is changed ? */
+	static char buffer[8];  /* 26^7 > 2^32.  */
 	char *res = col_name_internal (buffer, col);
 	*res = '\0';
 	return buffer;
@@ -72,7 +80,7 @@ col_name (int col)
 char const *
 cols_name (int start_col, int end_col)
 {
-	static char buffer [16]; /* Why is this 16 ? */
+	static char buffer[16]; /* See col_name.  */
 	char *res = col_name_internal (buffer, start_col);
 
 	if (start_col != end_col) {
@@ -86,18 +94,18 @@ cols_name (int start_col, int end_col)
 char const *
 col_parse (char const *str, int *res, unsigned char *relative)
 {
-	char const *ptr = str;
+	char const *ptr, *start = str;
 	int col = -1;
 
-	if (!(*relative = (*ptr != '$')))
-		ptr++;
+	if (!(*relative = (*start != '$')))
+		start++;
 
-	for (; col < SHEET_MAX_COLS ; ptr++)
+	for (ptr = start; col < SHEET_MAX_COLS ; ptr++)
 		if (('a' <= *ptr && *ptr <= 'z'))
 			col = 26 * (col + 1) + (*ptr - 'a');
 		else if (('A' <= *ptr && *ptr <= 'Z'))
 			col = 26 * (col + 1) + (*ptr - 'A');
-		else if (ptr != str) {
+		else if (ptr != start) {
 			*res = col;
 			return ptr;
 		} else
@@ -110,14 +118,14 @@ col_parse (char const *str, int *res, unsigned char *relative)
 inline static char *
 row_name_internal (char *buf, int row)
 {
-	int len = g_snprintf (buf, 6, "%d", row + 1); /* The 6 is hardcoded, see comments in row{s}_name */
+	int len = g_snprintf (buf, 4 * sizeof (int), "%d", row + 1);
 	return buf + len;
 }
 
 char const *
 row_name (int row)
 {
-	static char buffer [6]; /* What if SHEET_MAX_ROWS changes? */
+	static char buffer[4 * sizeof (int)];
 	char *res = row_name_internal (buffer, row);
 	*res = '\0';
 	return buffer;
@@ -126,7 +134,7 @@ row_name (int row)
 char const *
 rows_name (int start_row, int end_row)
 {
-	static char buffer [13]; /* What if SHEET_MAX_ROWS changes? */
+	static char buffer[2 * 4 * sizeof (int)];
 	char *res = row_name_internal (buffer, start_row);
 
 	if (start_row != end_row) {
@@ -137,6 +145,24 @@ rows_name (int start_row, int end_row)
 	return buffer;
 }
 
+char const *
+row_parse (char const *str, int *res, unsigned char *relative)
+{
+	char const *end, *ptr = str;
+	int row;
+
+	if (!(*relative = (*ptr != '$')))
+		ptr++;
+
+	errno = 0;
+	row = strtol (ptr, (char **)&end, 10);
+	if (ptr != end && 0 < row && row <= SHEET_MAX_ROWS && errno != ERANGE) {
+		*res = row - 1;
+		return end;
+	} else
+		return str;
+}
+
 /***************************************************************************/
 
 inline static int
@@ -314,7 +340,7 @@ cellref_a1_get (CellRef *out, char const *in, CellPos const *pos)
 {
 	int col = 0;
 	int row = 0;
-	unsigned char uc;
+	char c;
 
 	g_return_val_if_fail (in != NULL, NULL);
 	g_return_val_if_fail (out != NULL, NULL);
@@ -326,20 +352,21 @@ cellref_a1_get (CellRef *out, char const *in, CellPos const *pos)
 	} else
 		out->col_relative = TRUE;
 
-	/*
-	 * Careful here!  'A' and 'a' are not necessarily the only
-	 * characters which toupper maps to 'A'.
-	 */
-	uc = (unsigned char)*in;
-	if (!((uc >= 'A' && uc <= 'Z') ||
-	      (uc >= 'a' && uc <= 'z')))
+	c = *in;
+	if (c >= 'A' && c <= 'Z')
+		col = c - 'A';
+	else if (c >= 'a' && c <= 'z')
+		col = c - 'a';
+	else
 		return NULL;
-	col = toupper (uc) - 'A';
 	in++;
 
-	uc = (unsigned char)*in;
-	if ((uc >= 'A' && uc <= 'Z') || (uc >= 'a' && uc <= 'z')) {
-		col = (col + 1) * ('Z' - 'A' + 1) + toupper (uc) - 'A';
+	c = *in;
+	if (c >= 'A' && c <= 'Z') {
+		col = (col + 1) * ('Z' - 'A' + 1) + (c - 'A');
+		in++;
+	} else if (c >= 'a' && c <= 'z') {
+		col = (col + 1) * ('Z' - 'A' + 1) + (c - 'a');
 		in++;
 	}
 	if (col >= SHEET_MAX_COLS)
@@ -355,8 +382,8 @@ cellref_a1_get (CellRef *out, char const *in, CellPos const *pos)
 	if (!(*in >= '1' && *in <= '9'))
 		return NULL;
 
-	while (isdigit ((unsigned char)*in)) {
-		row = row * 10 + *in - '0';
+	while (*in >= '0' && *in <= '9') {
+		row = row * 10 + (*in - '0');
 		if (row > SHEET_MAX_ROWS)
 			return NULL;
 		in++;
@@ -402,8 +429,8 @@ r1c1_get_item (int *num, unsigned char *rel, char const * *in)
 	}
 	*num = 0;
 
-	while (**in && isdigit ((unsigned char)**in)) {
-		*num = *num * 10 + **in - '0';
+	while (**in >= '0' && **in <= '9') {
+		*num = *num * 10 + (**in - '0');
 		(*in)++;
 	}
 
@@ -509,24 +536,23 @@ gboolean
 cellpos_parse (char const *cell_str, CellPos *res, gboolean strict, int *chars_read)
 {
 	char const * const original = cell_str;
-	unsigned char c;
 	gboolean found_digits = FALSE;
 
 	if (*cell_str == '$')
 		cell_str++;
 
 	/* Parse column name: one or two letters.  */
-	c = toupper ((unsigned char) *cell_str);
-	cell_str++;
-	if (c < 'A' || c > 'Z')
+	if (*cell_str >= 'A' && *cell_str <= 'Z')
+		res->col = *cell_str++ - 'A';
+	else if (*cell_str >= 'a' && *cell_str <= 'z')
+		res->col = *cell_str++ - 'a';
+	else
 		return FALSE;
 
-	res->col = c - 'A';
-	c = toupper ((unsigned char)*cell_str);
-	if (c >= 'A' && c <= 'Z') {
-		res->col = ((res->col + 1) * ('Z' - 'A' + 1)) + (c - 'A');
-		cell_str++;
-	}
+	if (*cell_str >= 'A' && *cell_str <= 'Z')
+		res->col = ((res->col + 1) * ('Z' - 'A' + 1)) + (*cell_str++ - 'A');
+	else if (*cell_str >= 'a' && *cell_str <= 'z')
+		res->col = ((res->col + 1) * ('Z' - 'A' + 1)) + (*cell_str++ - 'a');
 	if (res->col >= SHEET_MAX_COLS)
 		return FALSE;
 
@@ -644,10 +670,9 @@ parse_text_value_or_expr (ParsePos const *pos, char const *text,
 ParseError *
 parse_error_init (ParseError *pe)
 {
-	pe->id         = PERR_NONE;
-	pe->message    = NULL;
-	pe->begin_char = -1;
-	pe->end_char   = -1;
+	pe->err		= NULL;
+	pe->begin_char	= 0;
+	pe->end_char	= 0;
 
 	return pe;
 }
@@ -655,28 +680,31 @@ parse_error_init (ParseError *pe)
 void
 parse_error_free (ParseError *pe)
 {
-	if (pe->message != NULL) {
-		g_free (pe->message);
-		pe->message = NULL;
+	if (pe->err != NULL) {
+		g_error_free (pe->err);
+		pe->err = NULL;
 	}
 }
 
 /***************************************************************************/
 
 static char const *
-check_quoted (char const *str, int *num_escapes)
+check_quoted (char const *start, int *num_escapes)
 {
+	char const *str = start;
 	if (*str == '\'' || *str == '\"') {
-		char const quote = *str;
+		char const quote = *str++;
 		*num_escapes = 0;
 		for (; *str && *str != quote; str = g_utf8_next_char (str))
 			if (*str == '\\' && str[1]) {
 				str++;
 				(*num_escapes)++;
 			}
+		if (*str)
+			return str+1;
 	} else
 		*num_escapes = -1;
-	return str;
+	return start;
 }
 
 static void
@@ -782,24 +810,6 @@ sheetref_parse (char const *start, Sheet **sheet, Workbook const *wb,
 	return *sheet != NULL ? end : start;
 }
 
-char const *
-row_parse (char const *str, int *res, unsigned char *relative)
-{
-	char const *end, *ptr = str;
-	int row;
-
-	if (!(*relative = (*ptr != '$')))
-		ptr++;
-
-	errno = 0;
-	row = strtol (ptr, (char **)&end, 10);
-	if (ptr != end && 0 < row && row <= SHEET_MAX_ROWS && errno != ERANGE) {
-		*res = row - 1;
-		return end;
-	} else
-		return str;
-}
-
 /** rangeref_parse :
  * @res : where to store the result
  * @start : the start of the string to parse
@@ -846,6 +856,10 @@ rangeref_parse (RangeRef *res, char const *start, ParsePos const *pp)
 			return start;
 		res->a.col_relative = res->b.col_relative = FALSE;
 		res->a.col = 0; res->b.col = SHEET_MAX_COLS-1;
+		if (res->a.row_relative)
+			res->a.row -= pp->eval.row;
+		if (res->b.row_relative)
+			res->b.row -= pp->eval.row;
 		return tmp2;
 	}
 
@@ -858,6 +872,10 @@ rangeref_parse (RangeRef *res, char const *start, ParsePos const *pp)
 			return start;
 		res->a.row_relative = res->b.row_relative = FALSE;
 		res->a.row = 0; res->b.row = SHEET_MAX_ROWS-1;
+		if (res->a.col_relative)
+			res->a.col -= pp->eval.col;
+		if (res->b.col_relative)
+			res->b.col -= pp->eval.col;
 		return tmp2;
 	}
 
