@@ -31,7 +31,6 @@
 #include <gsf/gsf-impl-utils.h>
 
 #include <bonobo/bonobo-ui-node.h>
-#include <gal/util/e-util.h>
 #include <gal/util/e-xml-utils.h>
 
 #include <string.h>
@@ -561,13 +560,11 @@ gnum_plugin_file_opener_probe (GnumFileOpener const *fo, GsfInput *input,
 				if (pattern->case_sensitive) {
 					match = fnmatch (pattern->value, base_file_name, FNM_PATHNAME) == 0;
 				} else {
-					gchar *pattern_str, *name_str;
-
-					pattern_str = g_alloca (strlen (pattern->value) + 1);
-					name_str = g_alloca (strlen (base_file_name) + 1);
-					g_strdown (strcpy (pattern_str, pattern->value));
-					g_strdown (strcpy (name_str, base_file_name));
+					char *pattern_str = g_utf8_strdown (pattern->value, -1);
+					char *name_str = g_utf8_strdown (base_file_name, -1);
 					match = fnmatch (pattern_str, name_str, FNM_PATHNAME) == 0;
+					g_free (pattern_str);
+					g_free (name_str);
 				}
 			} else {
 				g_assert_not_reached ();
@@ -603,41 +600,37 @@ gnum_plugin_file_opener_open (GnumFileOpener const *fo, IOContext *io_context,
 {
 	GnumPluginFileOpener *pfo = GNUM_PLUGIN_FILE_OPENER (fo);
 	PluginServiceFileOpener *service_file_opener = GNM_PLUGIN_SERVICE_FILE_OPENER (pfo->service);
+	InputFileSaveInfo *save_info;
 	ErrorInfo *error = NULL;
 
 	g_return_if_fail (GSF_IS_INPUT (input));
 
 	plugin_service_load (pfo->service, &error);
-	if (error == NULL) {
-		g_return_if_fail (service_file_opener->cbs.plugin_func_file_open != NULL);
-		service_file_opener->cbs.plugin_func_file_open (fo, pfo->service, io_context, wbv, input);
-		if (!gnumeric_io_error_occurred (io_context)) {
-			if (service_file_opener->save_info != NULL) {
-				InputFileSaveInfo *save_info;
-
-				save_info = service_file_opener->save_info;
-				if (save_info->saver_id_str[0] == '\0') {
-					workbook_set_saveinfo (wb_view_workbook (wbv), gsf_input_name (input),
-					                       save_info->format_level, NULL);
-				} else {
-					GHashTable *file_savers_hash;
-					GnumFileSaver *saver;
-
-					file_savers_hash = get_plugin_file_savers_hash (pfo->service->plugin);
-					saver = (GnumFileSaver *) g_hash_table_lookup (file_savers_hash,
-					                                           save_info->saver_id_str);
-					if (saver != NULL) {
-						workbook_set_saveinfo (wb_view_workbook (wbv), gsf_input_name (input),
-						                       save_info->format_level, saver);
-					}
-				}
-			}
-		}
-	} else {
+	if (error != NULL) {
 		gnumeric_io_error_info_set (io_context, error);
 		gnumeric_io_error_push (io_context, error_info_new_str (
 		                        _("Error while reading file.")));
+		return;
 	}
+	g_return_if_fail (service_file_opener->cbs.plugin_func_file_open != NULL);
+
+	/* Set the saver info before we load so that the nascent
+	 * workbook has a reasonable file name
+	 */
+	save_info = service_file_opener->save_info;
+	if (save_info != NULL) {
+		GnumFileSaver	  *saver = NULL;
+
+		if (save_info->saver_id_str[0] != '\0') {
+			GHashTable *savers = get_plugin_file_savers_hash (pfo->service->plugin);
+			saver = g_hash_table_lookup (savers, save_info->saver_id_str);
+		}
+
+		workbook_set_saveinfo (wb_view_workbook (wbv),
+			gsf_input_name (input), save_info->format_level, saver);
+	}
+
+	service_file_opener->cbs.plugin_func_file_open (fo, pfo->service, io_context, wbv, input);
 }
 
 static void
@@ -942,7 +935,7 @@ struct _PluginServiceFunctionGroup {
 	gchar *category_name, *translated_category_name;
 	GSList *function_name_list;
 
-	FunctionCategory *category;
+	GnmFuncGroup	*func_group;
 	PluginServiceFunctionGroupCallbacks cbs;
 };
 
@@ -956,7 +949,7 @@ plugin_service_function_group_init (GObject *obj)
 	service_function_group->category_name = NULL;
 	service_function_group->translated_category_name = NULL;
 	service_function_group->function_name_list = NULL;
-	service_function_group->category = NULL;
+	service_function_group->func_group = NULL;
 }
 
 static void
@@ -1054,61 +1047,33 @@ plugin_service_function_group_read_xml (PluginService *service, xmlNode *tree, E
 }
 
 static gboolean
-plugin_service_function_group_get_full_info_callback (
-	FunctionDefinition *fn_def,
-	const gchar       **args_ptr,
-	const gchar       **arg_names_ptr,
-	const gchar      ***help_ptr,
-	FunctionArgs       *fn_args_ptr,
-	FunctionNodes      *fn_nodes_ptr,
-	FuncLinkHandle     *fn_link_ptr,
-	FuncUnlinkHandle   *fn_unlink_ptr)
+plugin_service_function_group_func_desc_load (GnmFunc const *fn_def,
+					      GnmFuncDescriptor *res)
 {
 	PluginService *service;
 	PluginServiceFunctionGroup *service_function_group;
-	ErrorInfo *error =NULL;
+	ErrorInfo *error = NULL;
 
 	g_return_val_if_fail (fn_def != NULL, FALSE);
 
-	service = function_def_get_user_data (fn_def);
+	service = gnm_func_get_user_data (fn_def);
 	service_function_group = GNM_PLUGIN_SERVICE_FUNCTION_GROUP (service);
 	plugin_service_load (service, &error);
-	if (error == NULL) {
-		const gchar	 *args;
-		const gchar	 *arg_names;
-		const gchar	**help;
-		FunctionArgs	 fn_args;
-		FunctionNodes	 fn_nodes;
-		FuncLinkHandle   fn_link;
-		FuncUnlinkHandle fn_unlink;
-
-		if (service_function_group->cbs.plugin_func_get_full_function_info (
-		    service, function_def_get_name (fn_def),
-		    &args, &arg_names, &help, &fn_args, &fn_nodes, &fn_link, &fn_unlink)) {
-			*args_ptr = args;
-			*arg_names_ptr = arg_names;
-			*help_ptr = help;
-			*fn_args_ptr	= fn_args;
-			*fn_nodes_ptr	= fn_nodes;
-			*fn_link_ptr	= fn_link;
-			*fn_unlink_ptr	= fn_unlink;
-			return TRUE;
-		} else {
-			return FALSE;
-		}
-	} else {
+	if (error != NULL) {
 		error_info_print (error);
 		error_info_free (error);
 		return FALSE;
 	}
+	return service_function_group->cbs.func_desc_load (service,
+		gnm_func_get_name (fn_def), res);
 }
 
 static void
-plugin_service_function_group_func_ref_notify  (FunctionDefinition *fn_def, int refcount)
+plugin_service_function_group_func_ref_notify  (GnmFunc *fn_def, int refcount)
 {
 	PluginService *service;
 
-	service = function_def_get_user_data (fn_def);
+	service = gnm_func_get_user_data (fn_def);
 	g_return_if_fail (GNM_IS_PLUGIN_SERVICE_FUNCTION_GROUP (service));
 	if (refcount == 0) {
 		gnm_plugin_use_unref (service->plugin);
@@ -1123,17 +1088,17 @@ plugin_service_function_group_activate (PluginService *service, ErrorInfo **ret_
 	PluginServiceFunctionGroup *service_function_group = GNM_PLUGIN_SERVICE_FUNCTION_GROUP (service);
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
-	service_function_group->category = function_get_category_with_translation (
+	service_function_group->func_group = gnm_func_group_fetch_with_translation (
 		service_function_group->category_name,
 		service_function_group->translated_category_name);
 	GNM_SLIST_FOREACH (service_function_group->function_name_list, char, fname,
-		FunctionDefinition *fn_def;
+		GnmFunc *fn_def;
 
-		fn_def = function_add_name_only (
-			service_function_group->category, fname,
-			plugin_service_function_group_get_full_info_callback,
+		fn_def = gnm_func_add_stub (
+			service_function_group->func_group, fname,
+			plugin_service_function_group_func_desc_load,
 			plugin_service_function_group_func_ref_notify);
-		function_def_set_user_data (fn_def, service);
+		gnm_func_set_user_data (fn_def, service);
 	);
 	service->is_active = TRUE;
 }
@@ -1145,7 +1110,7 @@ plugin_service_function_group_deactivate (PluginService *service, ErrorInfo **re
 
 	GNM_INIT_RET_ERROR_INFO (ret_error);
 	GNM_SLIST_FOREACH (service_function_group->function_name_list, char, fname,
-		function_remove (service_function_group->category, fname);
+		function_remove (service_function_group->func_group, fname);
 	);
 	service->is_active = FALSE;
 }
@@ -1279,6 +1244,7 @@ GSF_CLASS (PluginServicePluginLoader, plugin_service_plugin_loader,
            GNM_PLUGIN_SERVICE_TYPE)
 
 
+#ifdef WITH_BONOBO
 /*
  * PluginServiceUI
  */
@@ -1460,6 +1426,7 @@ GSF_CLASS (PluginServiceUI, plugin_service_ui,
            plugin_service_ui_class_init, plugin_service_ui_init,
            GNM_PLUGIN_SERVICE_TYPE)
 
+#endif
 
 /* ---------------------------------------------------------------------- */
 
@@ -1505,8 +1472,10 @@ static struct {
 	{"file_opener", plugin_service_file_opener_get_type},
 	{"file_saver", plugin_service_file_saver_get_type},
 	{"function_group", plugin_service_function_group_get_type},
-	{"plugin_loader", plugin_service_plugin_loader_get_type},
-	{"ui", plugin_service_ui_get_type}
+	{"plugin_loader", plugin_service_plugin_loader_get_type}
+#ifdef WITH_BONOBO
+	,{"ui", plugin_service_ui_get_type}
+#endif
 };
 
 PluginService *
