@@ -46,7 +46,6 @@
 #include <libxml/xmlmemory.h>
 #include <gsf/gsf-impl-utils.h>
 
-#include <gal/util/e-util.h>
 #include <gal/util/e-xml-utils.h>
 
 #include <glib-object.h>
@@ -1421,7 +1420,7 @@ gnumeric_extra_plugin_dirs (void)
 	GSList *extra_dirs;
 	gchar const *plugin_path_env;
 	
-	extra_dirs = gnm_gconf_get_plugin_extra_dirs ();
+	extra_dirs = gnm_app_prefs->plugin_extra_dirs;
 	plugin_path_env = g_getenv ("GNUMERIC_PLUGIN_PATH");
 	if (plugin_path_env != NULL) {
 		GNM_SLIST_CONCAT (extra_dirs, g_strsplit_to_slist (plugin_path_env, ":"));
@@ -1446,7 +1445,8 @@ plugin_info_list_read_for_all_dirs (ErrorInfo **ret_error)
 				   NULL);
 	GNM_SLIST_CONCAT (dir_list, gnumeric_extra_plugin_dirs ());
 	plugin_info_list = plugin_info_list_read_for_subdirs_of_dir_list (dir_list, &error);
-	e_free_string_slist (dir_list);
+	g_slist_foreach (dir_list, (GFunc)g_free, NULL);
+	g_slist_free (dir_list);
 	*ret_error = error;
 
 	return plugin_info_list;
@@ -1723,7 +1723,7 @@ plugins_init (CommandContext *context)
 
 	/* initialize hash table with information about known plugin.xml files */
 	plugin_file_state_dir_hash = g_hash_table_new_full (g_str_hash, g_str_equal, NULL, plugin_file_state_free);
-	state_str_list = gnm_gconf_get_plugin_file_states ();
+	state_str_list = gnm_app_prefs->plugin_file_states;
 	GNM_SLIST_FOREACH (state_str_list, char, state_str,
 		PluginFileState *state;
 
@@ -1749,8 +1749,8 @@ plugins_init (CommandContext *context)
 	}
 
 	/* activate all previously active and (optionally) new plugins */
-	saved_active_ids = gnm_gconf_get_active_plugins ();
-	if (gnm_gconf_get_activate_new_plugins ()) {
+	saved_active_ids = gnm_app_prefs->active_plugins;
+	if (gnm_app_prefs->activate_new_plugins) {
 		g_hash_table_foreach (
 			plugin_file_state_dir_hash,
 			ghf_collect_new_plugin_ids,
@@ -1766,7 +1766,8 @@ plugins_init (CommandContext *context)
 		}
 	);
 	g_slist_free_custom (saved_active_ids, g_free);
-	plugin_db_activate_plugin_list (g_slist_reverse (plugin_list), &error);
+	plugin_list = g_slist_reverse (plugin_list);
+	plugin_db_activate_plugin_list (plugin_list, &error);
 	g_slist_free (plugin_list);
 	if (error != NULL) {
 		GNM_SLIST_PREPEND (error_list, error_info_new_str_with_details (
@@ -1818,7 +1819,8 @@ plugins_shutdown (void)
 			GNM_SLIST_PREPEND (active_list, (gpointer) gnm_plugin_get_id (plugin));
 		}
 	);
-	gnm_gconf_set_active_plugins (g_slist_reverse (active_list));
+	active_list = g_slist_reverse (active_list);
+	gnm_gconf_set_active_plugins (active_list);
 	g_slist_free (active_list);
 
 	if (plugins_marked_for_deactivation_hash != NULL) {
