@@ -26,6 +26,7 @@
 #include "position.h"
 
 #include "sheet.h"
+#include "sheet-view.h"
 #include "cell.h"
 #include "value.h"
 #include "ranges.h"
@@ -258,7 +259,7 @@ cellref_make_abs (CellRef *dest, CellRef const *src, EvalPos const *ep)
 	if (src->row_relative) {
 		dest->row = (dest->row + ep->eval.row) % SHEET_MAX_ROWS;
 		if (dest->row < 0)
-			dest->col += SHEET_MAX_ROWS;
+			dest->row += SHEET_MAX_ROWS;
 	}
 
 	dest->row_relative = dest->col_relative = FALSE;
@@ -297,7 +298,7 @@ value_to_rangeref (Value *v, gboolean release)
  *     by converting to absolute coords and handling inversions.
  */
 void
-rangeref_normalize (EvalPos const *ep, RangeRef const *ref,
+rangeref_normalize (RangeRef const *ref, EvalPos const *ep,
 		    Sheet **start_sheet, Sheet **end_sheet, Range *dest)
 {
 	g_return_if_fail (ref != NULL);
@@ -308,5 +309,5 @@ rangeref_normalize (EvalPos const *ep, RangeRef const *ref,
 	range_normalize (dest);
 
 	*start_sheet = eval_sheet (ref->a.sheet, ep->sheet);
-	*end_sheet = eval_sheet (ref->b.sheet, ep->sheet);
+	*end_sheet = eval_sheet (ref->b.sheet, *start_sheet);
 }
