@@ -30,6 +30,7 @@
 #include "cell.h"
 #include "style.h"
 #include "style-color.h"
+#include "number-match.h"
 #include "sheet.h"
 #include "sheet-merge.h"
 #include "sheet-style.h"
@@ -41,7 +42,6 @@
 #include "str.h"
 
 #include <math.h>
-#include <ctype.h>
 
 #ifndef USE_RV_POOLS
 #define USE_RV_POOLS 1
@@ -265,8 +265,7 @@ rendered_value_calc_size_ext (Cell const *cell, MStyle *mstyle)
 				last_whitespace = p;
 				first_whitespace = p+1;
 				prev_was_space = TRUE;
-#warning utf8_isspace
-			} else if (isspace (*(unsigned char *)p)) {
+			} else if (g_unichar_isspace (g_utf8_get_char (p))) {
 				used_last_space = used;
 				last_whitespace = p;
 				if (!prev_was_space)
@@ -373,8 +372,18 @@ cell_get_entered_text (Cell const *cell)
 	}
 
 	if (cell->value != NULL) {
-		if (cell->value->type == VALUE_STRING)
-			return g_strconcat ("\'", cell->value->v_str.val->str, NULL);
+		if (cell->value->type == VALUE_STRING) {
+			/* Try to be reasonably smart about adding a leading quote */
+			char const *tmp = cell->value->v_str.val->str;
+			if (NULL == gnm_expr_char_start_p (tmp)) {
+				Value *val = format_match_number (tmp,
+					cell_get_format	(cell));
+				if (val == NULL)
+					return g_strdup (tmp);
+				value_release (val);
+			}
+			return g_strconcat ("\'", tmp, NULL);
+		}
 		return format_value (NULL, cell->value, NULL, -1);
 	}
 
