@@ -160,7 +160,7 @@ in_list (AutoFillList const *afl, char const *s, int *n, int *is_i18n)
 		char const *translated_text = _(afl->items [i]);
 		if (*translated_text == '*')
 			translated_text++;
-		if (g_strcasecmp (translated_text, s) == 0) {
+		if (g_ascii_strcasecmp (translated_text, s) == 0) {
 			*is_i18n = TRUE;
 			*n = i;
 			return TRUE;
@@ -829,7 +829,7 @@ sheet_autofill_dir (Sheet *sheet, gboolean singleton_increment,
 			range_init (&tmp, col, row,
 				    col + fi->merged_size.col - 1,
 				    row + fi->merged_size.row - 1);
-			sheet_merge_add	(NULL, sheet, &tmp, TRUE);
+			sheet_merge_add	(sheet, &tmp, TRUE, NULL);
 		}
 
 		if (col_inc != 0) {
