@@ -30,7 +30,6 @@
 #include <cell.h>
 #include <number-match.h>
 
-#define USE_VALUE_POOLS 0
 #ifndef USE_VALUE_POOLS
 #define USE_VALUE_POOLS 1
 #endif
@@ -923,7 +922,7 @@ value_diff (Value const *a, Value const *b)
 		/* If both are strings compare as string */
 		case VALUE_STRING :
 		{
-			gint t = strcoll (a->v_str.val->str, b->v_str.val->str);
+			gint t = g_utf8_collate (a->v_str.val->str, b->v_str.val->str);
 			if (t == 0)
 				return 0.;
 		}
@@ -1016,15 +1015,14 @@ value_compare (Value const *a, Value const *b, gboolean case_sensitive)
 			gint t;
 
 			if (case_sensitive) {
-				t = strcoll (a->v_str.val->str, b->v_str.val->str);
+				t = g_utf8_collate (a->v_str.val->str, b->v_str.val->str);
 			} else {
-				gchar *str_a, *str_b;
+				char *str_a = g_utf8_casefold (a->v_str.val->str, -1);
+				char *str_b = g_utf8_casefold (b->v_str.val->str, -1);
 
-				str_a = g_alloca (strlen (a->v_str.val->str) + 1);
-				str_b = g_alloca (strlen (b->v_str.val->str) + 1);
-				g_strdown (strcpy (str_a, a->v_str.val->str));
-				g_strdown (strcpy (str_b, b->v_str.val->str));
-				t = strcoll (str_a, str_b);
+				t = g_utf8_collate (str_a, str_b);
+				g_free (str_a);
+				g_free (str_b);
 			}
 
 			if (t == 0)
@@ -1089,67 +1087,6 @@ value_set_fmt (Value *v, StyleFormat const *fmt)
 	VALUE_FMT (v) = (StyleFormat *)fmt;
 }
 
-/**
- * value_intersection :
- * @v: a VALUE_CELLRANGE
- * @ei: EvalInfo containing valid fd!
- *
- * Handle the implicit union of a single row or column with the eval position.
- *
- * NOTE : We do not need to know if this is expression is being evaluated as an
- * array or not because we can differentiate based on the required type for the
- * argument.
- *
- * Always release the value passed in.
- *
- * Return value:
- *     If the intersection succeeded return a duplicate of the value
- *     at the intersection point.  This value needs to be freed.
- * Returns the upper left corner of an array.
- *
- * FIXME FIXME FIXME : This will need to be reworked
- **/
-Value *
-value_intersection (Value *v, EvalPos const *pos)
-{
-	Value *res = NULL;
-	Range rng;
-	Sheet *start_sheet, *end_sheet;
-
-	if (v->type == VALUE_ARRAY) {
-		res = value_duplicate (v->v_array.vals[0][0]);
-		value_release (v);
-		return res;
-	}
-
-	/* handle inverted ranges */
-	value_cellrange_normalize (pos, v, &start_sheet, &end_sheet, &rng);
-
-	if (start_sheet == end_sheet) {
-		if (rng.start.row == rng.end.row) {
-			int const c = pos->eval.col;
-			if (rng.start.col <= c && c <= rng.end.col) {
-				Value const *tmp = value_area_get_x_y (pos, v,
-					c - rng.start.col, 0);
-				if (tmp != NULL)
-					res = value_duplicate (tmp);
-			}
-		}
-
-		if (rng.start.col == rng.end.col) {
-			int const r = pos->eval.row;
-			if (rng.start.row <= r && r <= rng.end.row) {
-				Value const *tmp = value_area_get_x_y (pos, v,
-					0, r - rng.start.row);
-				if (tmp != NULL)
-					res = value_duplicate (tmp);
-			}
-		}
-	}
-	value_release (v);
-	return res;
-}
-
 /****************************************************************************/
 
 gboolean
@@ -1279,6 +1216,7 @@ find_column_of_field (const EvalPos *ep, Value *database, Value *field)
 	        cell = sheet_cell_get (sheet, n, row);
 		if (cell == NULL)
 		        continue;
+		cell_eval (cell);
 
 		txt = cell_get_rendered_text (cell);
 		match = (g_ascii_strcasecmp (field_name, txt) == 0);
@@ -1360,7 +1298,7 @@ parse_criteria_range (Sheet *sheet, int b_col, int b_row, int e_col, int e_row,
 	database_criteria_t *new_criteria;
 	GSList              *criterias = NULL;
 	GSList              *conditions;
-	Cell const          *cell;
+	Cell 		    *cell;
 	func_criteria_t     *cond;
 	gchar               *cell_str;
 
@@ -1372,6 +1310,8 @@ parse_criteria_range (Sheet *sheet, int b_col, int b_row, int e_col, int e_row,
 
 		for (j = b_col; j <= e_col; j++) {
 		        cell = sheet_cell_get (sheet, j, i);
+			if (cell != NULL)
+				cell_eval (cell);
 			if (cell_is_blank (cell))
 			        continue;
 
@@ -1406,9 +1346,9 @@ GSList *
 parse_database_criteria (const EvalPos *ep, Value *database,
 			 Value *criteria)
 {
-	Sheet               *sheet;
-	GSList              *criterias;
-	Cell const          *cell;
+	Sheet	*sheet;
+	GSList	*criterias;
+	Cell	*cell;
 
         int   i;
 	int   b_col, b_row, e_col, e_row;
@@ -1425,6 +1365,8 @@ parse_database_criteria (const EvalPos *ep, Value *database,
 	/* Find the index numbers for the columns of criterias */
 	for (i = b_col; i <= e_col; i++) {
 	        cell = sheet_cell_get (sheet, i, b_row);
+		if (cell != NULL)
+			cell_eval (cell);
 		if (cell_is_blank (cell))
 		        continue;
 		field_ind[i - b_col] =
@@ -1451,7 +1393,7 @@ find_rows_that_match (Sheet *sheet, int first_col, int first_row,
 		      GSList *criterias, gboolean unique_only)
 {
 	GSList *current, *conditions, *rows;
-	Cell const *test_cell;
+	Cell   *test_cell;
 	int    row, add_flag;
 	rows = NULL;
 
@@ -1472,6 +1414,8 @@ find_rows_that_match (Sheet *sheet, int first_col, int first_row,
 
 				test_cell = sheet_cell_get (sheet,
 					first_col + cond->column, row);
+				if (test_cell != NULL)
+					cell_eval (test_cell);
 				if (cell_is_blank (test_cell))
 					continue;
 
@@ -1525,6 +1469,8 @@ filter_row:
 /****************************************************************************/
 
 const ValueErr value_terminate_err = { VALUE_ERROR, NULL, NULL };
+static const ValueInt the_value_zero = { VALUE_INTEGER, NULL, 0 };
+const Value *value_zero = (const Value *)&the_value_zero;
 
 void
 value_init (void)
