@@ -6,13 +6,12 @@
 #include <gnumeric.h>
 #include "gnumeric-combo-text.h"
 
-#include <ctype.h>
 #include <gtk/gtksignal.h>
 #include <gtk/gtkentry.h>
 #include <gtk/gtklist.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtkscrolledwindow.h>
-#include <gal/util/e-util.h>
+#include <gsf/gsf-impl-utils.h>
 
 #define GNM_COMBO_TEXT_CLASS(klass) G_TYPE_CHECK_CLASS_CAST (klass, gnm_combo_text_get_type (), GnmComboTextClass)
 typedef struct _GnmComboTextClass   GnmComboTextClass;
@@ -173,15 +172,15 @@ cb_scroll_size_request (GtkWidget *widget, GtkRequisition *requisition,
 
 	gtk_widget_size_request	(ct->list, &list_req);
 	if (requisition->height < list_req.height) {
-		int height = 0;
+		int height = list_req.height;
 		GtkWidget const *w = GTK_CONTAINER (ct->list)->focus_child;
 
-		if (w != NULL)
+		if (w != NULL) {
 			/* Magic number, max number of items before we scroll */
 			height = w->requisition.height * 10;
-
-		if (height > list_req.height)
-			height = list_req.height;
+			if (height > list_req.height)
+				height = list_req.height;
+		}
 
 		/* FIXME : Why do we need 4 ??
 		 * without it things end up scrolling.
@@ -299,10 +298,15 @@ gnm_combo_text_new (GCompareFunc cmp_func)
 	ct->cmp_func = cmp_func;
 	return GTK_WIDGET (ct);
 }
+GtkWidget *
+gnm_combo_text_glade_new (void)
+{
+	return gnm_combo_text_new (NULL);
+}
 
-E_MAKE_TYPE(gnm_combo_text, "GnmComboText", GnmComboText,
-	    gnm_combo_text_class_init, gnm_combo_text_init,
-	    gtk_combo_box_get_type ())
+GSF_CLASS (GnmComboText, gnm_combo_text,
+	   gnm_combo_text_class_init, gnm_combo_text_init,
+	   gtk_combo_box_get_type ())
 
 /**
  * gnm_combo_text_set_text :
