@@ -29,6 +29,7 @@
 #include "history.h"
 #include "application.h"
 #include "sheet.h"
+#include "sheet-view.h"
 #include "sheet-merge.h"
 #include "sheet-style.h"
 #include "str.h"
@@ -43,10 +44,12 @@
 #include "mstyle.h"
 #include "position.h"
 #include "cell.h"
+#include "gutils.h"
 #include "io-context.h"
 
 #include <gsf/gsf-input-memory.h>
 #include <gsf/gsf-input-stdio.h>
+#include <gsf/gsf-utils.h>
 
 #include <gsf/gsf-impl-utils.h>
 #include <locale.h>
@@ -324,7 +327,7 @@ wb_view_auto_expr (WorkbookView *wbv, char const *descr, char const *func_name)
 
 	wbv->auto_expr_desc = g_strdup (descr);
 	wbv->auto_expr = gnm_expr_new_funcall (
-		func_lookup_by_name (func_name, NULL), NULL);
+		gnm_func_lookup (func_name, NULL), NULL);
 
 	if (wbv->current_sheet != NULL)
 		wb_view_auto_expr_recalc (wbv, TRUE);
@@ -406,7 +409,7 @@ wb_view_attach_control (WorkbookView *wbv, WorkbookControl *wbc)
 
 	if (wbv->wb != NULL) {
 		/* Set the title of the newly connected control */
-		char *base_name = g_path_get_basename (wbv->wb->filename);
+		char *base_name = g_path_get_basename (workbook_get_filename (wbv->wb));
 		wb_control_title_set (wbc, base_name);
 		g_free (base_name);
 	}
@@ -516,7 +519,7 @@ workbook_view_new (Workbook *wb)
 		wb_view_sheet_add (wbv, workbook_sheet_by_index (wb, i));
 
 	/* Set the titles of the newly connected view's controls */
-	base_name = g_path_get_basename (wbv->wb->filename);
+	base_name = g_path_get_basename (workbook_get_filename (wbv->wb));
 	WORKBOOK_VIEW_FOREACH_CONTROL (wbv, wbc,
 		wb_control_title_set (wbc, base_name););
 	g_free (base_name);
@@ -596,10 +599,10 @@ wb_view_save (WorkbookView *wbv, CommandContext *context)
 
 	io_context = gnumeric_io_context_new (context);
 	if (fs == NULL)
-		gnumeric_io_error_save (io_context,
+		gnumeric_error_save (COMMAND_CONTEXT (io_context),
 			_("Default file saver is not available."));
 	else
-		gnum_file_saver_save (fs, io_context, wbv, wb->filename);
+		gnum_file_saver_save (fs, io_context, wbv, workbook_get_filename (wb));
 
 	error = gnumeric_io_error_occurred (io_context);
 	if (error)
@@ -636,6 +639,11 @@ wb_view_new_from_input  (GsfInput *input,
 		for (pl = FILE_PROBE_FILE_NAME; pl < FILE_PROBE_LAST && optional_fmt == NULL; pl++) {
 			for (l = get_file_openers (); l != NULL; l = l->next) {
 				GnumFileOpener const *tmp_fo = GNUM_FILE_OPENER (l->data);
+#if 0
+				printf ("Trying format %s at level %d...\n",
+					gnum_file_opener_get_id (tmp_fo),
+					(int)pl);
+#endif
 				if (gnum_file_opener_probe (tmp_fo, input, pl)) {
 					optional_fmt = tmp_fo;
 					break;
@@ -662,7 +670,8 @@ wb_view_new_from_input  (GsfInput *input,
 		} else
 			workbook_set_dirty (new_wb, FALSE);
 	} else
-		gnumeric_io_error_read (io_context, _("Unsupported file format."));
+		gnumeric_error_read (COMMAND_CONTEXT (io_context),
+			_("Unsupported file format."));
 
 	return new_wbv;
 }
@@ -683,32 +692,43 @@ wb_view_new_from_file (char const *file_name,
 		       GnumFileOpener const *optional_fmt,
 		       IOContext *io_context)
 {
-	GError *err = NULL;
 	char *msg = NULL;
-	GsfInput *input;
 
-	/* Only report error if stdio fails too */
-	input = gsf_input_mmap_new (file_name, NULL);
-	if (input == NULL)
-		input = gsf_input_stdio_new (file_name, &err);
+	if (gnumeric_valid_filename (file_name)) {
+		GError *err = NULL;
+		GsfInput *input;
 
-	puts (file_name);
-	if (input != NULL) {
-		WorkbookView *res = wb_view_new_from_input  (input,
-			optional_fmt, io_context);
-		g_object_unref (G_OBJECT (input));
-		return res;
+		/* Only report error if stdio fails too */
+		input = gsf_input_mmap_new (file_name, NULL);
+		if (input == NULL)
+			input = gsf_input_stdio_new (file_name, &err);
+
+		puts (file_name);
+		if (input != NULL) {
+			WorkbookView *res = wb_view_new_from_input (input,
+								    optional_fmt, io_context);
+			g_object_unref (G_OBJECT (input));
+			return res;
+		}
+
+		if (err != NULL) {
+			if (err->message != NULL)
+				msg = g_strdup (err->message);
+			g_error_free (err);
+		}
+
+		if (msg == NULL)
+			msg = g_strdup_printf (_("An unexplained error happened while opening %s"),
+					       file_name);
+	} else {
+		/*
+		 * This should be quite rare.  To provoke, use
+		 * gnumeric `echo -e '\377\376'`
+		 */
+		msg = g_strdup (_("The filename given is not valid."));
 	}
-
-	if (err != NULL) {
-		if (err->message != NULL)
-			msg = g_strdup (err->message);
-		g_error_free (err);
-	}
-	if (msg == NULL)
-		msg = g_strdup_printf (_("An unexplained error happened while opening '%s'"),
-				       file_name);
-	gnumeric_io_error_read (io_context, msg);
+	gnumeric_error_read (COMMAND_CONTEXT (io_context), msg);
+	g_free (msg);
 
 	return NULL;
 }
