@@ -9,19 +9,21 @@
  *
  * (C) 1998, 1999, 2000 Miguel de Icaza
  * (C) 2000-2001 Ximian, Inc.
+ * (C) 2002 Jody Goldberg
  */
 #include <gnumeric-config.h>
 #include <gnumeric-i18n.h>
 #include "gnumeric.h"
-#include "workbook.h"
+#include "workbook-priv.h"
 
 #include "workbook-view.h"
 #include "workbook-control.h"
 #include "command-context.h"
 #include "application.h"
 #include "sheet.h"
-#include "cell.h"
+#include "sheet-view.h"
 #include "sheet-control.h"
+#include "cell.h"
 #include "expr.h"
 #include "expr-name.h"
 #include "dependent.h"
@@ -38,8 +40,6 @@
 
 #include <gtk/gtkmain.h> /* for gtk_main_quit */
 #include <gsf/gsf-impl-utils.h>
-#include <gal/util/e-util.h>
-#include <ctype.h>
 #include <string.h>
 
 static GObjectClass *workbook_parent_class;
@@ -56,7 +56,6 @@ enum {
 
 static GQuark signals [LAST_SIGNAL] = { 0 };
 
-
 /*
  * We introduced numbers in front of the the history file names for two
  * reasons:
@@ -234,7 +233,7 @@ workbook_set_dirty (Workbook *wb, gboolean is_dirty)
 	if (wb->summary_info != NULL)
 		wb->summary_info->modified = is_dirty;
 	g_hash_table_foreach (wb->sheet_hash_private,
-			      cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
+		cb_sheet_mark_dirty, GINT_TO_POINTER (is_dirty));
 }
 
 static void
@@ -261,12 +260,38 @@ workbook_is_dirty (Workbook const *wb)
 	if (wb->summary_info != NULL && wb->summary_info->modified)
 		return TRUE;
 
-	g_hash_table_foreach (wb->sheet_hash_private, cb_sheet_check_dirty,
-			      &dirty);
+	g_hash_table_foreach (wb->sheet_hash_private,
+		cb_sheet_check_dirty, &dirty);
 
 	return dirty;
 }
 
+/**
+ * workbook_set_placeholder :
+ * @wb :
+ * @is_placeholder :
+ **/
+void
+workbook_set_placeholder (Workbook *wb, gboolean is_placeholder)
+{
+	g_return_if_fail (IS_WORKBOOK (wb));
+	wb->is_placeholder = is_placeholder;
+}
+
+/**
+ * workbook_is_placeholder :
+ * @wb :
+ *
+ * Returns TRUE if @wb has no views and was created as a placeholder for data
+ * in an external reference.
+ **/
+gboolean
+workbook_is_placeholder	(Workbook const *wb)
+{
+	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
+	return wb->is_placeholder;
+}
+
 static void
 cb_sheet_check_pristine (gpointer key, gpointer value, gpointer user_data)
 {
@@ -301,8 +326,8 @@ workbook_is_pristine (Workbook const *wb)
 		return FALSE;
 
 	/* Check if we seem to contain anything */
-	g_hash_table_foreach (wb->sheet_hash_private, cb_sheet_check_pristine,
-			      &pristine);
+	g_hash_table_foreach (wb->sheet_hash_private,
+		cb_sheet_check_pristine, &pristine);
 
 	return pristine;
 }
@@ -312,10 +337,12 @@ workbook_init (GObject *object)
 {
 	Workbook *wb = WORKBOOK (object);
 
+	wb->is_placeholder = FALSE;
+	wb->modified	   = FALSE;
+
 	wb->wb_views = NULL;
 	wb->sheets = g_ptr_array_new ();
-	wb->sheet_hash_private = g_hash_table_new (gnumeric_strcase_hash,
-						   gnumeric_strcase_equal);
+	wb->sheet_hash_private = g_hash_table_new (g_str_hash, g_str_equal);
 	wb->sheet_order_dependents = NULL;
 	wb->names        = NULL;
 	wb->summary_info = summary_info_new ();
@@ -326,14 +353,16 @@ workbook_init (GObject *object)
 	wb->undo_commands = wb->redo_commands = NULL;
 
 	/* default to no iteration */
-	wb->iteration.enabled = FALSE;
+	wb->iteration.enabled = TRUE;
 	wb->iteration.max_number = 100;
 	wb->iteration.tolerance = .001;
+	wb->recalc_auto = TRUE;
 
 	wb->file_format_level = FILE_FL_NEW;
 	wb->file_saver        = NULL;
 
 	wb->during_destruction = FALSE;
+	wb->being_reordered    = FALSE;
 	wb->recursive_dirty_enabled = TRUE;
 
 	application_workbook_list_add (wb);
@@ -354,7 +383,7 @@ workbook_class_init (GObjectClass *object_class)
 		gnm__VOID__VOID,
 		G_TYPE_NONE,
 		0, G_TYPE_NONE);
-	
+
 	signals [FILENAME_CHANGED] = g_signal_new ("filename_changed",
 		WORKBOOK_TYPE,
 		G_SIGNAL_RUN_LAST,
@@ -524,12 +553,20 @@ workbook_get_filename (Workbook *wb)
 	return wb->filename;
 }
 
-void workbook_add_summary_info (Workbook *wb, SummaryItem *sit)
+void
+workbook_add_summary_info (Workbook *wb, SummaryItem *sit)
 {
 	if (summary_info_add (wb->summary_info, sit))
 		g_signal_emit (G_OBJECT (wb), signals [SUMMARY_CHANGED], 0);
 }
 
+SummaryInfo *
+workbook_metadata (Workbook *wb)
+{
+	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
+	return wb->summary_info;
+}
+
 /**
  * workbook_set_saveinfo:
  * @wb: the workbook to modify
@@ -577,20 +614,6 @@ workbook_get_file_saver (Workbook *wb)
 	return wb->file_saver;
 }
 
-static void
-cb_sheet_calc_spans (gpointer key, gpointer value, gpointer flags)
-{
-	sheet_calc_spans (value, GPOINTER_TO_INT(flags));
-}
-void
-workbook_calc_spans (Workbook *wb, SpanCalcFlags const flags)
-{
-	g_return_if_fail (wb != NULL);
-
-	g_hash_table_foreach (wb->sheet_hash_private,
-			      &cb_sheet_calc_spans, GINT_TO_POINTER (flags));
-}
-
 void
 workbook_unref (Workbook *wb)
 {
@@ -629,21 +652,21 @@ workbook_foreach_cell_in_range (EvalPos const *pos,
 
 	g_return_val_if_fail (pos != NULL, NULL);
 	g_return_val_if_fail (cell_range != NULL, NULL);
-
 	g_return_val_if_fail (cell_range->type == VALUE_CELLRANGE, NULL);
 
-	value_cellrange_normalize (pos, cell_range, &start_sheet, &end_sheet, &r);
+	rangeref_normalize (&cell_range->v_range.cell, pos,
+			    &start_sheet, &end_sheet, &r);
 
 	if (start_sheet != end_sheet) {
 		Value *res;
 		Workbook const *wb = start_sheet->workbook;
 		int i = start_sheet->index_in_wb;
 		int stop = end_sheet->index_in_wb;
-		if (i < stop) { int tmp = i; i = stop ; stop = tmp; }
+		if (i > stop) { int tmp = i; i = stop ; stop = tmp; }
 
 		g_return_val_if_fail (end_sheet->workbook == wb, VALUE_TERMINATE);
 
-		while (i <= stop) {
+		for (; i <= stop ; i++) {
 			res = sheet_foreach_cell_in_range (
 				g_ptr_array_index (wb->sheets, i), flags,
 				r.start.col, r.start.row, r.end.col, r.end.row,
@@ -703,12 +726,19 @@ workbook_enable_recursive_dirty (Workbook *wb, gboolean enable)
 	gboolean old;
 
 	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
-	
+
 	old = wb->recursive_dirty_enabled;
 	wb->recursive_dirty_enabled = enable;
 	return old;
 }
 
+void
+workbook_autorecalc_enable (Workbook *wb, gboolean enable)
+{
+	g_return_if_fail (IS_WORKBOOK (wb));
+	wb->recalc_auto = enable;
+}
+
 void
 workbook_iteration_enabled (Workbook *wb, gboolean enable)
 {
@@ -807,13 +837,19 @@ cb_dep_unlink (Dependent *dep, gpointer value, gpointer user_data)
 		pos = &DEP_TO_CELL (dep)->pos;
 	dependent_unlink (dep, pos);
 }
+
 static void
 pre_sheet_index_change (Workbook *wb)
 {
+	g_return_if_fail (!wb->being_reordered);
+
+	wb->being_reordered = TRUE;
+
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
 			(GHFunc) cb_dep_unlink, NULL);
 }
+
 static void
 cb_dep_link (Dependent *dep, gpointer value, gpointer user_data)
 {
@@ -825,10 +861,14 @@ cb_dep_link (Dependent *dep, gpointer value, gpointer user_data)
 static void
 post_sheet_index_change (Workbook *wb)
 {
+	g_return_if_fail (wb->being_reordered);
+
 	if (wb->sheet_order_dependents != NULL)
 		g_hash_table_foreach (wb->sheet_order_dependents,
 			(GHFunc) cb_dep_link, NULL);
 
+	wb->being_reordered = FALSE;
+
 	if (wb->during_destruction)
 		return;
 
@@ -858,18 +898,25 @@ workbook_sheet_by_index (Workbook const *wb, int i)
 /**
  * workbook_sheet_by_name:
  * @wb: workbook to lookup the sheet on
- * @sheet_name: the sheet name we are looking for.
+ * @name: the sheet name we are looking for.
  *
  * Returns a pointer to a Sheet or NULL if the sheet
  * was not found.
  */
 Sheet *
-workbook_sheet_by_name (Workbook const *wb, char const *sheet_name)
+workbook_sheet_by_name (Workbook const *wb, char const *name)
 {
-	g_return_val_if_fail (wb != NULL, NULL);
-	g_return_val_if_fail (sheet_name != NULL, NULL);
+	Sheet *sheet;
+	char *tmp;
 
-	return g_hash_table_lookup (wb->sheet_hash_private, sheet_name);
+	g_return_val_if_fail (IS_WORKBOOK (wb), NULL);
+	g_return_val_if_fail (name != NULL, NULL);
+
+	tmp = g_utf8_casefold (name, -1);
+	sheet = g_hash_table_lookup (wb->sheet_hash_private, tmp);
+	g_free (tmp);
+
+	return sheet;
 }
 
 /**
@@ -899,7 +946,7 @@ workbook_sheet_attach (Workbook *wb, Sheet *new_sheet,
 	}
 
 	g_hash_table_insert (wb->sheet_hash_private,
-			     new_sheet->name_unquoted, new_sheet);
+		new_sheet->name_case_insensitive, new_sheet);
 	post_sheet_index_change (wb);
 
 	WORKBOOK_FOREACH_VIEW (wb, view,
@@ -965,7 +1012,7 @@ workbook_sheet_detach (Workbook *wb, Sheet *sheet)
 	g_ptr_array_remove_index (wb->sheets, sheet_index);
 	workbook_sheet_index_update (wb, sheet_index);
 	sheet->index_in_wb = -1;
-	g_hash_table_remove (wb->sheet_hash_private, sheet->name_unquoted);
+	g_hash_table_remove (wb->sheet_hash_private, sheet->name_case_insensitive);
 	sheet_destroy (sheet);
 	post_sheet_index_change (wb);
 
@@ -1078,7 +1125,7 @@ workbook_sheet_move (Sheet *sheet, int direction)
 	if (0 <= new_pos && new_pos < workbook_sheet_count (wb)) {
 		int min_pos = MIN (old_pos, new_pos);
 		int max_pos = MAX (old_pos, new_pos);
-		
+
 		g_ptr_array_remove_index (wb->sheets, old_pos);
 		g_ptr_array_insert (wb->sheets, sheet, new_pos);
 
@@ -1160,15 +1207,17 @@ workbook_sheet_get_free_name (Workbook *wb,
  * Returns FALSE when it was successful
  **/
 gboolean
-workbook_sheet_reorder (WorkbookControl *wbc, GSList *new_order, GSList *new_sheets)
+workbook_sheet_reorder (Workbook *wb, GSList *new_order, GSList *new_sheets)
 {
 	GSList *this_sheet;
 	gint old_pos, new_pos = 0;
-	Workbook *wb = wb_control_workbook (wbc);
 
-	if (!new_order)
+	g_return_val_if_fail (IS_WORKBOOK (wb), FALSE);
+
+	if (new_order == NULL)
 		return TRUE;
 
+	pre_sheet_index_change (wb);
 	this_sheet = new_order;
 	while (this_sheet) {
 		gboolean an_old_sheet = TRUE;
@@ -1193,7 +1242,7 @@ workbook_sheet_reorder (WorkbookControl *wbc, GSList *new_order, GSList *new_she
 					sheet->index_in_wb = max_pos;
 				}
 				WORKBOOK_FOREACH_CONTROL (wb, view, control,
-							  wb_control_sheet_move (control, 
+							  wb_control_sheet_move (control,
 										 sheet, new_pos););
 				if (an_old_sheet)
 					sheet_set_dirty (sheet, TRUE);
@@ -1207,12 +1256,13 @@ workbook_sheet_reorder (WorkbookControl *wbc, GSList *new_order, GSList *new_she
 }
 
 gboolean
-workbook_sheet_reorganize (WorkbookControl *wbc, 
-			   GSList *changed_names, GSList *new_order,  
+workbook_sheet_reorganize (Workbook *wb,
+			   GSList *changed_names, GSList *new_order,
 			   GSList *new_names,  GSList *old_names,
 			   GSList **new_sheets, GSList *color_changed,
 			   GSList *colors_fore, GSList *colors_back,
-			   GSList *protection_changed, GSList *new_locks)
+			   GSList *protection_changed, GSList *new_locks,
+			   CommandContext *cc)
 {
 	GSList *new_sheet = NULL;
 	GSList *the_names;
@@ -1220,7 +1270,6 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 	GSList *the_fore;
 	GSList *the_back;
 	GSList *the_lock;
-	Workbook *wb = wb_control_workbook (wbc);
 
 /* We need to verify validity of the new names */
 	the_names = new_names;
@@ -1234,37 +1283,36 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 
 		if (new_name != NULL ) {
 			sheet = the_sheets->data;
-			
+
 			/* Is the sheet name to short ?*/
 			if (1 > strlen (new_name)) {
-				gnumeric_error_invalid (COMMAND_CONTEXT (wbc), 
-							_("Sheet name must have at least 1 letter"),
-							new_name);
+				gnumeric_error_invalid (cc,
+					_("Sheet name must have at least 1 letter"),
+					new_name);
 				return TRUE;
 			}
-			
+
 			/* Is the sheet name already in use ?*/
-			tmp = (Sheet *) g_hash_table_lookup (wb->sheet_hash_private, 
-							     new_name);
-			
+			tmp = workbook_sheet_by_name (wb, new_name);
+
 			if (tmp != NULL) {
 				/* Perhaps it is a sheet also to be renamed */
 				GSList *tmp_sheets = g_slist_find (changed_names, tmp);
 				if (NULL == tmp_sheets) {
-					gnumeric_error_invalid (COMMAND_CONTEXT (wbc),
-							 _("There is already a sheet named"),
-								new_name);
+					gnumeric_error_invalid (cc,
+						_("There is already a sheet named"),
+						new_name);
 					return TRUE;
 				}
 			}
-			
+
 			/* Will we try to use the same name a second time ?*/
-			if (the_names->next != NULL && 
+			if (the_names->next != NULL &&
 			    g_slist_find_custom (the_names->next, new_name, g_str_compare) != NULL) {
-				gnumeric_error_invalid (COMMAND_CONTEXT (wbc),
-							_("You may not use this name twice"),
-							new_name);
-				return TRUE;			
+				gnumeric_error_invalid (cc,
+					_("You may not use this name twice"),
+					new_name);
+				return TRUE;
 			}
 		}
 		the_names = the_names->next;
@@ -1289,10 +1337,10 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 		if (sheet != NULL) {
 			sheet_rename (sheet, the_names->data);
 			g_hash_table_insert (wb->sheet_hash_private,
-					     sheet->name_unquoted, sheet);
-			
+					     sheet->name_case_insensitive, sheet);
+
 			sheet_set_dirty (sheet, TRUE);
-			
+
 			WORKBOOK_FOREACH_CONTROL (wb, view, control,
 						  wb_control_sheet_rename (control, sheet););
 		}
@@ -1312,9 +1360,9 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 				char *name = the_names->data;
 				Sheet *a_new_sheet ;
 				gboolean free_name = (name == NULL);
-				
+
 				if (free_name)
-					name = workbook_sheet_get_free_name 
+					name = workbook_sheet_get_free_name
 						(wb, _("Sheet"), TRUE, FALSE);
 				a_new_sheet = sheet_new (wb, name);
 				if (free_name)
@@ -1338,15 +1386,15 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 			sheet = new_sheet->data;
 			new_sheet = new_sheet->next;
 		}
-		
+
 		if (sheet != NULL) {
 			GdkColor *back = (GdkColor *) the_back->data;
 			GdkColor *fore = (GdkColor *) the_fore->data;
-			StyleColor *tab_color = back ? 
-				style_color_new (back->red, back->green, 
+			StyleColor *tab_color = back ?
+				style_color_new (back->red, back->green,
 						 back->blue) : NULL;
 			StyleColor *text_color = fore ?
-				style_color_new (fore->red, fore->green, 
+				style_color_new (fore->red, fore->green,
 						 fore->blue) : NULL;
 			sheet_set_tab_color (sheet, tab_color, text_color);
 		}
@@ -1354,7 +1402,7 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 		the_back = the_back->next;
 		the_sheets = the_sheets->next;
 	}
-	
+
 /* Changing Protection */
 	the_lock = new_locks;
 	the_sheets = protection_changed;
@@ -1371,7 +1419,7 @@ workbook_sheet_reorganize (WorkbookControl *wbc,
 	}
 
 /* reordering */
-	workbook_sheet_reorder (wbc, new_order, new_sheets ? *new_sheets : NULL);
+	workbook_sheet_reorder (wb, new_order, new_sheets ? *new_sheets : NULL);
 
 	return FALSE;
 }
